Package: m2r
Type: Package
Title: Interface to 'Macaulay2'
Version: 1.0.3
Authors@R: c(person("David", "Kahle", email = "david@kahle.io", role = c("aut", "cph", "cre"), comment = c(ORCID = "0000-0002-9999-1558")),
    person("Christopher", "O'Neill", email = "musicman3320@gmail.com", role = c("aut", "cph")),
    person("Jeff", "Sommars", email = "sommars1@uic.edu", role = c("aut", "cph")))
Maintainer: David Kahle <david@kahle.io>
Description: Persistent interface to 'Macaulay2' <https://www.macaulay2.com>
    and front-end tools facilitating its use in the 'R' ecosystem. For details see 
    Kahle et. al. (2020) <doi:10.18637/jss.v093.i09>.
License: GPL-2
Depends: mpoly (>= 1.0.5)
Imports: stringr, memoise, gmp, usethis, Rcpp
SystemRequirements: Macaulay2 <https://www.macaulay2.com>
URL: https://github.com/coneill-math/m2r
BugReports: https://github.com/coneill-math/m2r/issues
Suggests: knitr, rmarkdown, microbenchmark, testthat, R.rsp
RoxygenNote: 7.3.2
LinkingTo: Rcpp
VignetteBuilder: R.rsp
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2025-06-10 05:19:47 UTC; david_kahle
Author: David Kahle [aut, cph, cre] (ORCID:
    <https://orcid.org/0000-0002-9999-1558>),
  Christopher O'Neill [aut, cph],
  Jeff Sommars [aut, cph]
Repository: CRAN
Date/Publication: 2025-06-10 05:40:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 19:31:07 UTC; unix
Archs: m2r.so.dSYM

Package: m2r
Type: Package
Title: Interface to 'Macaulay2'
Version: 1.0.3
Authors@R: c(person("David", "Kahle", email = "david@kahle.io", role = c("aut", "cph", "cre"), comment = c(ORCID = "0000-0002-9999-1558")),
    person("Christopher", "O'Neill", email = "musicman3320@gmail.com", role = c("aut", "cph")),
    person("Jeff", "Sommars", email = "sommars1@uic.edu", role = c("aut", "cph")))
Maintainer: David Kahle <david@kahle.io>
Description: Persistent interface to 'Macaulay2' <https://www.macaulay2.com>
    and front-end tools facilitating its use in the 'R' ecosystem. For details see 
    Kahle et. al. (2020) <doi:10.18637/jss.v093.i09>.
License: GPL-2
Depends: mpoly (>= 1.0.5)
Imports: stringr, memoise, gmp, usethis, Rcpp
SystemRequirements: Macaulay2 <https://www.macaulay2.com>
URL: https://github.com/coneill-math/m2r
BugReports: https://github.com/coneill-math/m2r/issues
Suggests: knitr, rmarkdown, microbenchmark, testthat, R.rsp
RoxygenNote: 7.3.2
LinkingTo: Rcpp
VignetteBuilder: R.rsp
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2025-06-10 05:19:47 UTC; david_kahle
Author: David Kahle [aut, cph, cre] (ORCID:
    <https://orcid.org/0000-0002-9999-1558>),
  Christopher O'Neill [aut, cph],
  Jeff Sommars [aut, cph]
Repository: CRAN
Date/Publication: 2025-06-10 05:40:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-29 21:13:16 UTC; unix
Archs: m2r.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method("*",m2_ideal)
S3method("+",m2_ideal)
S3method("==",m2_ideal)
S3method("^",m2_ideal)
S3method(m2_parse_class,default)
S3method(m2_parse_class,m2_hashtable)
S3method(m2_parse_class,m2_optiontable)
S3method(m2_parse_class,m2_verticallist)
S3method(m2_parse_function,default)
S3method(m2_parse_function,m2_hashtable)
S3method(m2_parse_function,m2_ideal)
S3method(m2_parse_function,m2_image)
S3method(m2_parse_function,m2_map)
S3method(m2_parse_function,m2_monoid)
S3method(m2_parse_function,m2_monomialideal)
S3method(m2_parse_function,m2_optiontable)
S3method(m2_parse_function,m2_symbol)
S3method(m2_parse_function,m2_tocc)
S3method(m2_parse_function,m2_verticallist)
S3method(m2_parse_object_as_function,default)
S3method(m2_parse_object_as_function,m2_polynomialring)
S3method(m2_parse_object_as_function,m2_symbol)
S3method(print,m2_array)
S3method(print,m2_boolean)
S3method(print,m2_complex)
S3method(print,m2_float)
S3method(print,m2_hashtable)
S3method(print,m2_ideal)
S3method(print,m2_ideal_list)
S3method(print,m2_image)
S3method(print,m2_integer)
S3method(print,m2_list)
S3method(print,m2_matrix)
S3method(print,m2_module)
S3method(print,m2_option)
S3method(print,m2_pointer)
S3method(print,m2_polynomialring)
S3method(print,m2_sequence)
S3method(print,m2_string)
S3method(print,m2_symbol)
S3method(str_m2,default)
S3method(str_m2,m2_polynomialring)
export("m2_meta<-")
export("m2_name<-")
export(LLL)
export(LLL.)
export(dimension)
export(edit_r_environ)
export(enter_m2)
export(factor_n)
export(factor_n.)
export(factor_poly)
export(factor_poly.)
export(gb)
export(gb.)
export(gb_)
export(gb_.)
export(get_m2_con)
export(get_m2_connection)
export(get_m2_gmp)
export(get_m2_path)
export(get_m2_port)
export(get_m2_procid)
export(has_m2_connection)
export(ideal)
export(ideal.)
export(ideal_)
export(ideal_.)
export(is.m2)
export(is.m2_array)
export(is.m2_grobner_basis)
export(is.m2_ideal)
export(is.m2_ideal_list)
export(is.m2_ideal_list_pointer)
export(is.m2_ideal_pointer)
export(is.m2_list)
export(is.m2_matrix)
export(is.m2_matrix_pointer)
export(is.m2_module)
export(is.m2_option)
export(is.m2_pointer)
export(is.m2_polynomialring)
export(is.m2_polynomialring_pointer)
export(is.m2_sequence)
export(is.ring)
export(m2)
export(m2.)
export(m2_coefrings)
export(m2_exists)
export(m2_getwd)
export(m2_kernel)
export(m2_kernel.)
export(m2_length)
export(m2_ls)
export(m2_matrix)
export(m2_matrix.)
export(m2_meta)
export(m2_name)
export(m2_numcols)
export(m2_numrows)
export(m2_parse)
export(m2_rm)
export(m2_structure)
export(m2_termorders)
export(m2_toggle_gmp)
export(m2r_cloud_url)
export(m2r_version_number)
export(mixed_volume)
export(primary_decomposition)
export(primary_decomposition.)
export(quotient)
export(quotient.)
export(radical)
export(radical.)
export(reset_m2)
export(ring)
export(ring.)
export(ring_)
export(ring_.)
export(saturate)
export(saturate.)
export(set_m2_path)
export(snf)
export(snf.)
export(solve_system)
export(solve_system.)
export(start_m2)
export(stop_m2)
export(str_m2)
export(use_ring)
import(gmp)
import(mpoly)
importFrom(Rcpp,sourceCpp)
importFrom(memoise,forget)
importFrom(memoise,memoise)
importFrom(stringr,fixed)
importFrom(stringr,str_c)
importFrom(stringr,str_detect)
importFrom(stringr,str_replace)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(stringr,str_sub)
importFrom(usethis,edit_r_environ)
useDynLib(m2r)

# **m2r** – Macaulay2 in R

[![CRAN_Status_Badge](http://www.r-pkg.org/badges/version/m2r)](https://cran.r-project.org/package=m2r)
<!-- [![Travis build status](https://travis-ci.org/dkahle/m2r.svg?branch=master)](https://travis-ci.org/dkahle/m2r) -->
<!-- [![AppVeyor build status](https://ci.appveyor.com/api/projects/status/github/dkahle/m2r?branch=master&svg=true)](https://ci.appveyor.com/project/dkahle/m2r) -->

## Overview

**m2r** is an R package that provides a persistent connection between
[R](https://www.r-project.org) and [Macaulay2
(M2)](https://www.macaulay2.com).

The package grew out of a collaboration at the [2016 Mathematics
Research
Community](https://www.ams.org/programs/research-communities/mrc-16) on
algebraic statistics, funded by the [National Science
Foundation](https://www.nsf.gov) through the [American Mathematical
Society](https://www.ams.org/).

If you have a feature request, please file an issue!

## Getting started

**m2r** is loaded like any other R package:

``` r
library("m2r")
# Loading required package: mpoly
#   Please cite m2r! See citation("m2r") for details.
#   M2 found in /Applications/Macaulay2-1.21/bin
```

When loaded, **m2r** initializes a persistent connection to a back-end
Macaulay2 session. The basic function in R that accesses this connection
is `m2()`, which simply accepts a character string that is run by the
Macaulay2 session.

``` r
m2("1 + 1")
# Starting M2... done.
# [1] "2"
```

You can see the persistence by setting variables and accessing them
across different `m2()` calls:

``` r
m2("a = 1")
# [1] "1"
m2("a")
# [1] "1"
```

You can check the variables defined in the M2 session with `m2_ls()`:

``` r
m2_ls()
# [1] "a"
```

You can also check if variables exist with `m2_exists()`:

``` r
m2_exists("a")
# [1] TRUE
m2_exists(c("a","b"))
# [1]  TRUE FALSE
```

Apart from the basic connection to M2, **m2r** has basic data structures
and methods to reference and manipulate the M2 objects within R. For
more on this, see the **m2r** internals section below.

## Rings, ideals, and Grobner bases

**m2r** currently has basic support for
[rings](https://en.wikipedia.org/wiki/Ring_(mathematics)) (think:
[polynomial rings](https://en.wikipedia.org/wiki/Polynomial_ring)):

``` r
(QQtxyz <- ring("t", "x", "y", "z", coefring = "QQ"))
# M2 Ring: QQ[t,x,y,z], grevlex order
```

and [ideals](https://en.wikipedia.org/wiki/Ideal_(ring_theory)) of
rings:

``` r
(I <- ideal("t^4 - x", "t^3 - y", "t^2 - z"))
# M2 Ideal of ring QQ[t,x,y,z] (grevlex) with generators : 
# < t^4  -  x,  t^3  -  y,  t^2  -  z >
```

You can compute [Grobner
bases](https://en.wikipedia.org/wiki/Gröbner_basis) as well. The basic
function to do this is `gb()`:

``` r
gb(I)
# z^2  -  x
# z t  -  y
# -1 z x  +  y^2
# -1 x  +  t y
# -1 z y  +  x t
# -1 z  +  t^2
```

Perhaps an easier way to do this is just to list off the polynomials as
character strings:

``` r
gb("t^4 - x", "t^3 - y", "t^2 - z")
# z^2  -  x
# z t  -  y
# -1 z x  +  y^2
# -1 x  +  t y
# -1 z y  +  x t
# -1 z  +  t^2
```

The result is an `mpolyList` object, from the [**mpoly**
package](https://github.com/dkahle/mpoly). You can see the M2 code by
adding `code = TRUE`:

``` r
gb("t^4 - x", "t^3 - y", "t^2 - z", code = TRUE)
# m2rintgb00000003 = gb(m2rintideal00000003); gens m2rintgb00000003
```

You can compute the basis respective of different [monomial
orders](https://en.wikipedia.org/wiki/Monomial_order) as well. The
default ordering is the one in the respective ring, which defaults to
`grevlex`; however, changing the order is as simple as changing the
ring.

``` r
ring("x", "y", "t", "z", coefring = "QQ", order = "lex")
# M2 Ring: QQ[x,y,t,z], lex order
gb("t^4 - x", "t^3 - y", "t^2 - z")
# t^2  -  z
# -1 t z  +  y
# -1 z^2  +  x
```

On a technical level, `ring()`, `ideal()`, and `gb()` use [nonstandard
evaluation rules](https://adv-r.hadley.nz/metaprogramming.html). A more
stable way to use these functions is to use their standard evaluation
versions `ring_()`, `ideal_()`, and `gb_()`. Each accepts first a data
structure describing the relevant object of interest first as its own
object. For example, at a basic level this simply changes the previous
syntax to

``` r
use_ring(QQtxyz)
poly_chars <- c("t^4 - x", "t^3 - y", "t^2 - z")
gb_(poly_chars)
# z^2  -  x
# z t  -  y
# -1 z x  +  y^2
# -1 x  +  t y
# -1 z y  +  x t
# -1 z  +  t^2
```

`gb_()` is significantly easier to code with than `gb()` in the sense
that its inputs and outputs are more predictable, so we strongly
recommend that you use `gb_()`, especially inside of other functions and
packages.

As far as other kinds of computations are concerned, we present a
potpurri of examples below.

Ideal saturation:

``` r
ring("x", coefring = "QQ")
# M2 Ring: QQ[x], grevlex order
I <- ideal("(x-1) x (x+1)")
saturate(I, "x") # = (x-1) (x+1)
# M2 Ideal of ring QQ[x] (grevlex) with generator : 
# < x^2  -  1 >
```

Radicalization:

``` r
I <- ideal("x^2")
radical(I)
# M2 Ideal of ring QQ[x] (grevlex) with generator : 
# < x >
```

Primary decomposition:

``` r
ring("x", "y", "z", coefring = "QQ")
# M2 Ring: QQ[x,y,z], grevlex order
I <- ideal("x z", "y z")
primary_decomposition(I)
# M2 List of ideals of QQ[x,y,z] (grevlex) : 
# < z >
# < y,  x >
```

Dimension:

``` r
ring("x", "y", coefring = "QQ")
# M2 Ring: QQ[x,y], grevlex order
I <- ideal("y - (x+1)") 
dimension(I)
# [1] 1
```

## Factoring integers and polynomials

You can compute [prime
decompositions](https://en.wikipedia.org/wiki/Integer_factorization) of
integers with `factor_n()`:

``` r
(x <- 2^5 * 3^4 * 5^3 * 7^2 * 11^1)
# [1] 174636000
factor_n(x)
# $prime
# [1]  2  3  5  7 11
# 
# $power
# [1] 5 4 3 2 1
```

You can also [factor
polynomials](https://en.wikipedia.org/wiki/Factorization) over rings
using `factor_poly()`:

``` r
factor_poly("x^4 - y^4")
# $factor
# x  -  y
# x  +  y
# x^2  +  y^2
# 
# $power
# [1] 1 1 1
```

## Smith normal form of a matrix

The Smith normal form of a matrix *M* here refers to the decomposition
of an integer matrix *D = PMQ*, where *D*, *P*, and *Q* are integer
matrices and *D* is diagonal. *P* and *Q* are unimodular matrices (their
determinants are -1 or 1), so they are invertible. This is somewhat like
a singular value decomposition for integer matrices.

``` r
M <- matrix(c(
   2,  4,   4,
  -6,  6,  12,
  10, -4, -16
), nrow = 3, byrow = TRUE)

(mats <- snf(M))
# $D
#      [,1] [,2] [,3]
# [1,]   12    0    0
# [2,]    0    6    0
# [3,]    0    0    2
# M2 Matrix over ZZ[] 
# 
# $P
#      [,1] [,2] [,3]
# [1,]    1    0    1
# [2,]    0    1    0
# [3,]    0    0    1
# M2 Matrix over ZZ[] 
# 
# $Q
#      [,1] [,2] [,3]
# [1,]    4   -2   -1
# [2,]   -2    3    1
# [3,]    3   -2   -1
# M2 Matrix over ZZ[]
P <- mats$P; D <- mats$D; Q <- mats$Q

P %*% M %*% Q                # = D
#      [,1] [,2] [,3]
# [1,]   12    0    0
# [2,]    0    6    0
# [3,]    0    0    2
solve(P) %*% D %*% solve(Q)  # = M
#      [,1] [,2] [,3]
# [1,]    2    4    4
# [2,]   -6    6   12
# [3,]   10   -4  -16

det(P)
# [1] 1
det(Q)
# [1] -1
```

## **m2r** internals: pointers, reference and value functions, and `m2` objects

At a basic level, **m2r** works by passing strings between R and M2.
Originating at the R side, these strings are properly formated M2 code
constructed from the inputs to the R functions. That code goes to M2, is
evaluated there, and then “exported” with M2’s function
`toExternalString()`. The resulting string often, but not always,
produces the M2 code needed to recreate the object resulting from the
evaluation, and in that sense is M2’s version of R’s `dput()`. That
string is passed back into R and parsed there into R-style data
structures, typically [S3-classed
lists](https://adv-r.hadley.nz/s3.html).

The R-side parsing of the external string from M2 is an expensive
process because it is currently implemented in R. Consequently (and for
other reasons, too!), in some cases you’ll want to do a M2 computation
from R, but leave the output in M2. Since you will ultimately want
something in R referencing the result, nearly every **m2r** function
that performs M2 computations has a pointer version. As a simple naming
convention, the name of the function that returns the pointer, called
the reference function, is determined by the name of the ordinary
function, called the value function, by appending a `.`.

For example, we’ve seen that `factor_n()` computes the prime
decomposition of a number. The corresponding reference function is
`factor_n.()`:

``` r
(x <- 2^5 * 3^4 * 5^3 * 7^2 * 11^1)
# [1] 174636000
factor_n.(x)
# M2 Pointer Object
#   ExternalString : new Product from {new Power from {2,5},new Power from {3,...
#          M2 Name : m2o264
#         M2 Class : Product (WrapperType)
```

All value functions simply wrap reference functions and parse the output
with `m2_parse()`, a general M2 parser, often followed by a little more
parsing. `m2_parse()` typically creates an object of class `m2` so that
R knows what kind of thing it is. For example:

``` r
class(factor_n.(x))
# [1] "m2_pointer" "m2"
```

Even more, `m2_parse()` often creates objects that have an inheritance
structure that references `m2` somewhere in the middle of its class
structure, with specific structure preceding and general structure
succeeding (examples below). Apart from its class, the general principle
we follow here for the object itself is this: if the M2 object has a
direct analogue in R, it is parsed into that kind of R object and
additional M2 properties are kept as metadata (attributes); if there is
no direct analogue in R, the object is an `NA` with metadata.

Perhaps the easiest way to see this is with a matrix. `m2_matrix()`
creates a matrix on the M2 side from input on the R side. In the
following, to make things more clear we use [**magrittr**’s pipe
operator](https://github.com/tidyverse/magrittr), with which the
following calls are semantically equivalent: `g(f(x))` and
`x %>% f %>% g`.

``` r
library(magrittr)
mat <- matrix(c(1,2,3,4,5,6), nrow = 3, ncol = 2)
mat %>% m2_matrix.   # = m2_matrix.(mat)
# M2 Pointer Object
#   ExternalString : map(ZZ^3,ZZ^2,{{1, 4}, {2, 5}, {3, 6}})
#          M2 Name : m2rintmatrix00000001
#         M2 Class : Matrix (Type)
mat %>% m2_matrix. %>% m2_parse
#      [,1] [,2]
# [1,]    1    4
# [2,]    2    5
# [3,]    3    6
# M2 Matrix over ZZ[]
mat %>% m2_matrix. %>% m2_parse %>% str
#  'm2_matrix' int [1:3, 1:2] 1 2 3 4 5 6
#  - attr(*, "m2_name")= chr "m2rintmatrix00000003"
#  - attr(*, "m2_meta")=List of 1
#   ..$ ring: 'm2_polynomialring' logi NA
#   .. ..- attr(*, "m2_name")= chr "ZZ"
#   .. ..- attr(*, "m2_meta")=List of 3
#   .. .. ..$ vars    : NULL
#   .. .. ..$ coefring: chr "ZZ"
#   .. .. ..$ order   : chr "grevlex"
mat %>% m2_matrix    # = m2_parse(m2_matrix.(mat))
#      [,1] [,2]
# [1,]    1    4
# [2,]    2    5
# [3,]    3    6
# M2 Matrix over ZZ[]
```

It may be helpful to think of every `m2` object as being a missing value
(`NA`, a `logical(1)`) with two M2 attributes: their name (`m2_name`)
and a capture-all named list (`m2_meta`). These can be accessed with
`m2_name()` and `m2_meta()`. For example, a ring, having no analogous
object in R, is an `NA` with attributes:

``` r
r <- ring("x", "y", coefring = "QQ")
str(r)
#  'm2_polynomialring' logi NA
#  - attr(*, "m2_name")= chr "m2rintring00000006"
#  - attr(*, "m2_meta")=List of 3
#   ..$ vars    :List of 2
#   .. ..$ : chr "x"
#   .. ..$ : chr "y"
#   ..$ coefring: chr "QQ"
#   ..$ order   : chr "grevlex"
class(r)
# [1] "m2_polynomialring" "m2"
m2_name(r)
# [1] "m2rintring00000006"
m2_meta(r)
# $vars
# $vars[[1]]
# [1] "x"
# 
# $vars[[2]]
# [1] "y"
# 
# 
# $coefring
# [1] "QQ"
# 
# $order
# [1] "grevlex"
```

But a matrix of integers isn’t:

``` r
mat <- m2_matrix(matrix(c(1,2,3,4,5,6), nrow = 3, ncol = 2))
str(mat)
#  'm2_matrix' num [1:3, 1:2] 1 2 3 4 5 6
#  - attr(*, "m2_name")= chr "m2rintmatrix00000005"
#  - attr(*, "m2_meta")=List of 1
#   ..$ ring: 'm2_polynomialring' logi NA
#   .. ..- attr(*, "m2_name")= chr "ZZ"
#   .. ..- attr(*, "m2_meta")=List of 3
#   .. .. ..$ vars    : NULL
#   .. .. ..$ coefring: chr "ZZ"
#   .. .. ..$ order   : chr "grevlex"
class(mat)
# [1] "m2_matrix" "m2"        "matrix"
m2_name(mat)
# [1] "m2rintmatrix00000005"
m2_meta(mat)
# $ring
# M2 Ring: ZZ[], grevlex order
```

Since a matrix of integers is an object in R, it’s represented as one,
and consequently we can compute with it directly as it if it were a
matrix; it is. On the other hand, since a ring is not, it’s an `NA`.
When dealing with M2, object like rings, that is to say objects without
R analogues, are more common than those like integer matrices.

## Creating your own **m2r** wrapper

We’ve already wrapped a number of Macaulay2 functions; for a list of
functions in **m2r**, check out `ls("package:m2r")`. But the list is
very far from exhaustive. To create your own wrapper function of a
Macaulay2 command, you’ll need to create an R file that looks like the
one below. This will create both value (e.g. `f`) and reference/pointer
(e.g. `f.`) versions of the function. As a good example of these at
work, see the scripts for
[`factor_n()`](https://github.com/coneill-math/m2r/blob/master/R/factor_n.R)
or
[`factor_poly()`](https://github.com/coneill-math/m2r/blob/master/R/factor_poly.R).

``` r
#' Function documentation header
#'
#' Function header explanation, can run several lines. Function
#' header explanation, can run several lines. Function header
#' explanation, can run several lines.
#'
#' @param esntl_parm_1 esntl_parm_1 description
#' @param esntl_parm_2 esntl_parm_2 description
#' @param code return only the M2 code? (default: \code{FALSE})
#' @param parse_parm_1 parse_parm_1 description
#' @param parse_parm_2 parse_parm_2 description
#' @param ... ...
#' @name f
#' @return (value version) parsed output or (reference/dot version)
#'   \code{m2_pointer}
#' @examples
#'
#' \dontrun{ requires Macaulay2 be installed
#'
#' # put examples here
#' 1 + 1
#'
#' }
#'





# value version of f (standard user version)
#' @rdname f
#' @export
f <- function(esntl_parm_1, esntl_parm_2, code = FALSE, parse_parm_1, parse_parm_2, ...) {

  # run m2
  args <- as.list(match.call())[-1]
  eargs <- lapply(args, eval, envir = parent.frame())
  pointer <- do.call(f., eargs)
  if(code) return(invisible(pointer))

  # parse output
  parsed_out <- m2_parse(pointer)

  # more parsing, like changing classes and such
  TRUE

  # return
  TRUE

}




# reference version of f (returns pointer to m2 object)
#' @rdname f
#' @export
f. <- function(esntl_parm_1, esntl_parm_2, code = FALSE, ...) {

  # basic arg checking
  TRUE

  # create essential parameters to pass to m2 this step regularizes input to m2, so it
  # is the one that deals with pointers, chars, rings, ideals, mpolyLists, etc.
  TRUE

  # construct m2_code from regularized essential parameters
  TRUE

  # message
  if(code) { message(m2_code); return(invisible(m2_code)) }

  # run m2 and return pointer
  m2.(m2_code)

}
```

## Acknowledgements

This material is based upon work supported by the National Science
Foundation under Grant Nos.
[1321794](https://www.nsf.gov/awardsearch/showAward?AWD_ID=1321794) and
[1622449](https://www.nsf.gov/awardsearch/showAward?AWD_ID=1622449).

## Installation

Here’s how you can install the current *developmental* version of
**m2r**.

``` r
if (!requireNamespace("devtools")) install.packages("devtools")
devtools::install_github("coneill-math/m2r")
```

For **m2r** to find Macaulay2, you’ll need to set an environmental
variable in your `~/.Renviron` file. To do that, run this:

``` r
if (!requireNamespace("usethis")) install.packages("usethis")
usethis::edit_r_environ()
```

And, in the text file that opens, add a line such as
`M2=/opt/homebrew/bin`.

---
output:
  md_document:
    variant: markdown_github
---

# Creating your own __m2r__ Amazon cloud server


## EC2

Begin by getting your own [Amazon EC2](https://aws.amazon.com/ec2/)
instance running.  For an operating system, any Unix-alike should
work; we used the [Amazon Linux
AMI](https://aws.amazon.com/amazon-linux-ami/).  If you prefer not to
use a cloud server and would rather use a physical machine, in
principle all that follows should work in the same way, though this
has not been tested.

### Setting up the server

Once you have spun your EC2 instance, log into it and run the
following commands that download the m2rserver python script:
```{bash eval=FALSE}
mkdir m2rserver
cd m2rserver
curl https://raw.githubusercontent.com/coneill-math/m2r/master/inst/server/m2rserver.py > m2rserver.py
echo 'cd /home/ec2-user/m2rserver; python m2rserver.py' >> /etc/rc.local
```
<!-- Using your favorite text editor, in the file `/etc/rc.local`, add the -->
<!-- following line -->
<!-- ```{bash eval=FALSE} -->
<!-- cd /home/ec2-user/m2rserver; python m2rserver.py -->
<!-- ``` -->
The last line will cause the server to start when the system boots.
Reboot the instance, and the python script will start listening for a
connection.

Next, install [docker](https://www.docker.com) on your machine. Your
method of doing this will depend on what distribution you use. For the
Amazon Linux AMI,
[yum](https://en.wikipedia.org/wiki/Yellowdog_Updater,_Modified)
should work, see instructions [here](https://docs.docker.com/engine/installation/linux/centos/).
Once docker has been installed, run
```{bash eval=FALSE}
docker pull sommars/m2r
```
to install our docker image on your machine. As of June 2017, the
image comes provisioned with
[Macaulay2](http://www.math.uiuc.edu/Macaulay2/),
[PHCpack](http://homepages.math.uic.edu/~jan/download.html),
[Bertini](https://bertini.nd.edu), [4ti2](http://www.4ti2.de), and
[LattE](https://www.math.ucdavis.edu/~latte/).

### Local changes

When you use __m2r__, after loading the package, each time you must
run the command 
```{r eval=FALSE}
start_m2(cloud = TRUE, hostname = "host.uri.com")
```
to initialize a connection to your new host.


## Questions?

If you have any issues setting up a server, we're happy to try to
help. File an issue and we'll get back to you.

# Creating your own **m2r** Amazon cloud server

## EC2

Begin by getting your own [Amazon EC2](https://aws.amazon.com/ec2/)
instance running. For an operating system, any Unix-alike should work;
we used the [Amazon Linux
AMI](https://aws.amazon.com/amazon-linux-ami/). If you prefer not to use
a cloud server and would rather use a physical machine, in principle all
that follows should work in the same way, though this has not been
tested.

### Setting up the server

Once you have spun your EC2 instance, log into it and run the following
commands that download the m2rserver python script:

``` bash
mkdir m2rserver
cd m2rserver
curl https://raw.githubusercontent.com/coneill-math/m2r/master/inst/server/m2rserver.py > m2rserver.py
echo 'cd /home/ec2-user/m2rserver; python m2rserver.py' >> /etc/rc.local
```

<!-- Using your favorite text editor, in the file `/etc/rc.local`, add the -->
<!-- following line -->
<!-- ```{bash eval=FALSE} -->
<!-- cd /home/ec2-user/m2rserver; python m2rserver.py -->
<!-- ``` -->

The last line will cause the server to start when the system boots.
Reboot the instance, and the python script will start listening for a
connection.

Next, install [docker](https://www.docker.com) on your machine. Your
method of doing this will depend on what distribution you use. For the
Amazon Linux AMI,
[yum](https://en.wikipedia.org/wiki/Yellowdog_Updater,_Modified) should
work, see instructions
[here](https://docs.docker.com/engine/installation/linux/centos/). Once
docker has been installed, run

``` bash
docker pull sommars/m2r
```

to install our docker image on your machine. As of June 2017, the image
comes provisioned with [Macaulay2](https://www.macaulay2.com),
[PHCpack](http://homepages.math.uic.edu/~jan/download.html),
[Bertini](https://bertini.nd.edu), [4ti2](http://www.4ti2.de), and
[LattE](https://www.math.ucdavis.edu/~latte/).

### Local changes

When you use **m2r**, after loading the package, each time you must run
the command

``` r
start_m2(cloud = TRUE, hostname = "host.uri.com")
```

to initialize a connection to your new host.

## Questions?

If you have any issues setting up a server, we’re happy to try to help.
File an issue and we’ll get back to you.

Creating your own **m2r** Amazon cloud server
=============================================

EC2
---

Begin by getting your own [Amazon EC2](https://aws.amazon.com/ec2/) instance running. For an operating system, any Unix-alike should work; we used the [Amazon Linux AMI](https://aws.amazon.com/amazon-linux-ami/). If you prefer not to use a cloud server and would rather use a physical machine, in principle all that follows should work in the same way, though this has not been tested.

### Setting up the server

Once you have spun your EC2 instance, log into it and run the following commands that download the m2rserver python script:

``` bash
mkdir m2rserver
cd m2rserver
curl https://raw.githubusercontent.com/coneill-math/m2r/master/inst/server/m2rserver.py > m2rserver.py
echo 'cd /home/ec2-user/m2rserver; python m2rserver.py' >> /etc/rc.local
```

<!-- Using your favorite text editor, in the file `/etc/rc.local`, add the -->
<!-- following line -->
<!-- ```{bash eval=FALSE} -->
<!-- cd /home/ec2-user/m2rserver; python m2rserver.py -->
<!-- ``` -->
The last line will cause the server to start when the system boots. Reboot the instance, and the python script will start listening for a connection.

Next, install [docker](https://www.docker.com) on your machine. Your method of doing this will depend on what distribution you use. For the Amazon Linux AMI, [yum](https://en.wikipedia.org/wiki/Yellowdog_Updater,_Modified) should work, see instructions [here](https://docs.docker.com/engine/installation/linux/centos/). Once docker has been installed, run

``` bash
docker pull sommars/m2r
```

to install our docker image on your machine. As of June 2017, the image comes provisioned with [Macaulay2](http://www.math.uiuc.edu/Macaulay2/), [PHCpack](http://homepages.math.uic.edu/~jan/download.html), [Bertini](https://bertini.nd.edu), [4ti2](http://www.4ti2.de), and [LattE](https://www.math.ucdavis.edu/~latte/).

### Local changes

When you use **m2r**, after loading the package, each time you must run the command

``` r
start_m2(cloud = TRUE, hostname = "host.uri.com")
```

to initialize a connection to your new host.

Questions?
----------

If you have any issues setting up a server, we're happy to try to help. File an issue and we'll get back to you.

---
output:
  md_document:
    variant: markdown_github
---

# Creating your own __m2r__ Amazon cloud server


## EC2

Begin by getting your own [Amazon EC2](https://aws.amazon.com/ec2/)
instance running.  For an operating system, any Unix-alike should
work; we used the [Amazon Linux
AMI](https://aws.amazon.com/amazon-linux-ami/).  If you prefer not to
use a cloud server and would rather use a physical machine, in
principle all that follows should work in the same way, though this
has not been tested.

### Setting up the server

Once you have spun your EC2 instance, log into it and run the
following commands that download the m2rserver python script:
```{bash eval=FALSE}
mkdir m2rserver
cd m2rserver
curl https://raw.githubusercontent.com/coneill-math/m2r/master/inst/server/m2rserver.py > m2rserver.py
echo 'cd /home/ec2-user/m2rserver; python m2rserver.py' >> /etc/rc.local
```
<!-- Using your favorite text editor, in the file `/etc/rc.local`, add the -->
<!-- following line -->
<!-- ```{bash eval=FALSE} -->
<!-- cd /home/ec2-user/m2rserver; python m2rserver.py -->
<!-- ``` -->
The last line will cause the server to start when the system boots.
Reboot the instance, and the python script will start listening for a
connection.

Next, install [docker](https://www.docker.com) on your machine. Your
method of doing this will depend on what distribution you use. For the
Amazon Linux AMI,
[yum](https://en.wikipedia.org/wiki/Yellowdog_Updater,_Modified)
should work, see instructions [here](https://docs.docker.com/engine/installation/linux/centos/).
Once docker has been installed, run
```{bash eval=FALSE}
docker pull sommars/m2r
```
to install our docker image on your machine. As of June 2017, the
image comes provisioned with
[Macaulay2](https://www.macaulay2.com),
[PHCpack](http://homepages.math.uic.edu/~jan/download.html),
[Bertini](https://bertini.nd.edu), [4ti2](http://www.4ti2.de), and
[LattE](https://www.math.ucdavis.edu/~latte/).

### Local changes

When you use __m2r__, after loading the package, each time you must
run the command 
```{r eval=FALSE}
start_m2(cloud = TRUE, hostname = "host.uri.com")
```
to initialize a connection to your new host.


## Questions?

If you have any issues setting up a server, we're happy to try to
help. File an issue and we'll get back to you.

#' Create a new matrix in Macaulay2
#'
#' Create a new matrix in Macaulay2
#'
#' @param mat a matrix
#' @param ring a ring containing the matrix entries
#' @param name the \code{m2_name} of the object, which is it's name
#'   on the M2 side
#' @param code return only the M2 code? (default: \code{FALSE})
#' @param x formal argument for print method
#' @param ... ...
#' @return an object of class \code{m2_matrix}
#' @name m2_matrix
#' @examples
#'
#' \dontrun{ requires Macaulay2
#'
#' ##### basic usage
#' ########################################
#'
#' (mat <- m2_matrix(matrix(c(1,2,3,4,5,6), nrow = 3, ncol = 2)))
#' m2_matrix(matrix(c(1,2,3,4,5,6), nrow = 3, ncol = 2))
#'
#' m2_name(mat)
#' m2(m2_name(mat))
#' m2(sprintf("class(%s)", m2_name(mat)))
#' (mat <- m2_matrix.(matrix(c(1,2,3,4,5,6), nrow = 3, ncol = 2)))
#'
#' ##### known issues
#' ########################################
#'
#' ring("x", "y", "z", coefring = "QQ")
#' (mat <- matrix(mp(c("x","y","x+y","y-2","x-3","y-z")), nrow = 2, ncol = 3))
#' m2_matrix(mat, code = TRUE)
#' m2_matrix(mat)
#' # the above is an mpoly problem, not a m2r problem
#' # mpoly does not have a data structure for matrices (as of 12/2016)
#'
#' mat_chars <- sapply(m2_matrix(mat), print, silent = TRUE)
#' dim(mat_chars) <- c(2, 3)
#' mat_chars
#'
#'
#' m2_numrows(mat)
#' m2_numcols(mat)
#' m2_parse(mat)
#'
#' (mat <- m2_matrix(matrix(c(1,2),nrow=1)))
#' m2_kernel(mat)
#'
#' }



#' @rdname m2_matrix
#' @export
m2_matrix <- function(mat, ring, name, code = FALSE) {

  # run m2
  args <- as.list(match.call())[-1]
  eargs <- lapply(args, eval, envir = parent.frame())
  pointer <- do.call(m2_matrix., eargs)
  if (code) return(invisible(pointer))

  # parse output
  parsed_out <- m2_parse(pointer)

  # construct R-side matrix, class and return
  m2_structure(
    mat,
    m2_name = m2_name(pointer),
    m2_class = "m2_matrix",
    m2_meta = list(
      ring = m2_meta(parsed_out, "ring")
    ),
    base_class = "matrix"
  )
}



#' @rdname m2_matrix
#' @export
m2_matrix. <- function(mat, ring, name, code = FALSE) {

  # arg check
  # this errors with vars() in mpoly 1.1.0.903 (at least)
  # in that version, vars requires the input be a mpoly or mpolyList
  # here, it may be (e.g.) a numeric matrix
  # removed until mpoly implements better data structures
  # if ( !all( m2_exists(vars(mat)) )) {
  #   stop(sprintf(
  #     "all variables (%s) in mat must be defined in M2.",
  #     paste(vars(mat), collapse = ", ")
  #   ), call. = FALSE)
  # }

  # prep ring string
  ring_str <- if (missing(ring)) "" else paste0("*1_", m2_name(ring))

  # make matrix name
  if (missing(name)) {
    matrix_name <- name_and_increment("matrix", "m2_matrix_count")
  } else {
    matrix_name <- name
  }


  # prepare matrix string
  mat2 <- matrix(
    vapply(
      mpolyList_to_m2_str(mat),
      function(.) sprintf("(%s)%s", ., ring_str), character(1)
    ),
    nrow(mat), ncol(mat)
  )
  matrix_str <- listify_mat(mat2)

  # construct code and message
  # matrix{{1,2,3},{4,5,6}}
  m2_code <- sprintf("%s = matrix %s", matrix_name, matrix_str)
  if(code) { message(m2_code); return(invisible(m2_code)) }

  # run m2 and add name
  out <- m2.(m2_code)
  m2_name(out) <- matrix_name

  # return
  out
}



#' @rdname m2_matrix
#' @export
m2_numrows <- function(x, code = FALSE, ...) {

  # make mat_param
  if (is.m2_matrix(x)) {
    mat_param <- m2_name(x)
  } else if (is.m2_matrix_pointer(x)) {
    mat_param <- m2_name(x)
  } else {
    stop("input must be a matrix")
  }

  # construct code and message
  m2_code <- sprintf("numrows(%s)", mat_param)
  if(code) { message(m2_code); return(invisible(m2_code)) }

  # run m2 and return
  m2_parse(m2.(m2_code))

}



#' @rdname m2_matrix
#' @export
m2_numcols <- function(x, code = FALSE, ...) {

  # make mat_param
  if (is.m2_matrix(x)) {
    mat_param <- m2_name(x)
  } else if (is.m2_matrix_pointer(x)) {
    mat_param <- m2_name(x)
  } else {
    stop("input must be a matrix")
  }

  # construct code and message
  m2_code <- sprintf("numcols(%s)", mat_param)
  if(code) { message(m2_code); return(invisible(m2_code)) }

  # run m2 and return
  m2_parse(m2.(m2_code))

}



#' @rdname m2_matrix
#' @export
m2_length <- function(x, code = FALSE, ...) {

  # make x_param
  if (is.m2(x)) {
    x_param <- m2_name(x)
  } else {
    stop("input must be an m2 object")
  }

  # construct code and message
  m2_code <- sprintf("length(%s)", x_param)
  if(code) { message(m2_code); return(invisible(m2_code)) }

  # run m2 and return
  m2_parse(m2.(m2_code))

}



m2_parse_function.m2_map <- function(x) {

  R1 <- x[[1]]
  R2 <- x[[2]]

  if (is.m2_module(R1)) R1 <- R1[[1]]
  if (is.m2_module(R2)) R2 <- R2[[1]]

  if (!identical(R1, R2)) {
    stop("Parsing error: map between different rings not supported")
  }

  has_vars <- !is.null(m2_meta(R1, "vars"))

  if (is.integer(x[[3]]) && x[[3]] == 0) {

    if (!has_vars) {
      mat <- matrix(numeric(0), nrow = 0, ncol = 0)
    } else {
      mat <- matrix(character(0), nrow = 0, ncol = 0)
    }

  } else {

    if (!is.list(x[[3]]) || !is.list(x[[c(3,1)]])) {
      stop("Parsing error: unsupported map format")
    }

    if (has_vars) {
      # convert to mpolys
      for (i in 1:length(x[[3]])) {
        x[[c(3,i)]] <- lapply(x[[c(3,i)]], function(.) mp(as.character(.)))
      }
    }

    nrow <- length(x[[3]])
    ncol <- length(x[[c(3,1)]])

    if (!has_vars) {
      mat <- t(matrix(unlist(x[[3]]), nrow = ncol, ncol = nrow))
    } else {
      mat <- t(matrix(unlist(x[[3]], recursive = FALSE), nrow = ncol, ncol = nrow))
    }

  }

  m2_structure(
    mat,
    m2_name = "",
    m2_class = "m2_matrix",
    m2_meta = list(
      ring = R1
    ),
    base_class = "matrix"
  )
}






#' @rdname m2_matrix
#' @export
print.m2_matrix <- function(x, ...){

  x_no_attr <- x
  attr(x_no_attr, "class") <- NULL
  attr(x_no_attr, "m2_name") <- NULL
  attr(x_no_attr, "m2_meta") <- NULL
  print(x_no_attr)

  r <- m2_meta(x)$ring

  s <- sprintf(
    "M2 Matrix over %s[%s]",
    m2_meta(r, "coefring"),
    paste(m2_meta(r, "vars"), collapse = ","),
    m2_meta(r, "order")
  )
  cat(s, "\n")

  invisible(x)
}




m2_parse_function.m2_image <- function(x) {
  m2_structure(
    x[[1]],
    m2_name = "",
    m2_class = "m2_image",
    base_class = "image"
  )
}



#' @rdname m2_matrix
#' @export
print.m2_image <- function(x, ...){
  # cat("M2 Image\n")
  print.m2_matrix(x, ...)

  invisible(x)
}











#' @rdname m2_matrix
#' @export
m2_kernel <- function(mat, name, code = FALSE) {

  # run m2
  args <- as.list(match.call())[-1]
  eargs <- lapply(args, eval, envir = parent.frame())
  pointer <- do.call(m2_kernel., eargs)
  if (code) return(invisible(pointer))

  # parse output
  m2_parse(pointer)
}



#' @rdname m2_matrix
#' @export
m2_kernel. <- function(mat, name, code = FALSE) {
  if ((class(mat)[1] != "m2_matrix") && (class(mat)[1] != "m2_pointer")) {
    stop(print("Class of matrix must be m2_matrix or m2_pointer"))
  }

  # make kernel name
  if (missing(name)) {
    kernel_name <- name_and_increment("matrix", "m2_image_count")
  } else {
    kernel_name <- name
  }

  m2_code <- sprintf("%s = kernel %s",kernel_name, attr(mat, "m2_name"))
  if(code) { message(m2_code); return(invisible(m2_code)) }

  # run m2 and add name
  out <- m2.(m2_code)
  m2_name(out) <- kernel_name

  # return
  out
}

context("snf")


test_that("snf(mat)", {
  skip_on_cran()

  stop_m2()
  start_m2()

  M <- matrix(c(
       1,    2,    3,
       1,   34,   45,
    2213, 1123, 6543,
       0,    0,    0
  ), nrow = 4, byrow = TRUE)

  actual <- snf(M)

  expect <- list(
    D = m2_structure(
      matrix(c(
        135654L, 0L, 0L,
        0L, 1L, 0L,
        0L, 0L, 1L,
        0L, 0L, 0L
      ), nrow = 4, byrow = TRUE),
      m2_name = m2_name(actual$D),
      m2_class = "m2_matrix",
      m2_meta = list(ring = coefring_as_ring("ZZ")),
      base_class = "matrix"
    ),
    P = m2_structure(
      matrix(c(
        1L, 33471L, -43292L, 0L,
        0L, 1L, 0L, 0L,
        0L, 0L, 1L, 0L,
        0L, 0L, 0L, 1L
      ), nrow = 4, byrow = TRUE),
      m2_name = m2_name(actual$P),
      m2_class = "m2_matrix",
      m2_meta = list(ring = coefring_as_ring("ZZ")),
      base_class = "matrix"
    ),
    Q = m2_structure(
      matrix(c(
        171927L, -42421L,  54868L,
        93042L, -22957L,  29693L,
        -74119L,  18288L, -23654L
      ), nrow = 3, byrow = TRUE),
      m2_name = m2_name(actual$Q),
      m2_class = "m2_matrix",
      m2_meta = list(ring = coefring_as_ring("ZZ")),
      base_class = "matrix"
    )
  )

  expect_equal(actual, expect)

  # this is testing M2 more than m2r, skipping
  #
  # # decomposition
  # expect_equal(D, P %*% M %*% Q)
  #
  # # unitary
  # expect_equal(abs(det(P)), 1)
  # expect_equal(abs(det(Q)), 1)
  #
  # # D is diag
  # expect_equal(
  #   D,
  #   rbind(diag(diag(D)), matrix(0L, ncol = ncol(D), nrow = nrow(D) - length(diag(D))))
  # )

})











test_that("snf(m2_matrix(mat))", {
  skip_on_cran()

  M <- matrix(c(
       1,    2,    3,
       1,   34,   45,
    2213, 1123, 6543,
       0,    0,    0
  ), nrow = 4, byrow = TRUE)

  actual <- snf(m2_matrix(M))

  expect <- list(
    D = m2_structure(
      matrix(c(
        135654L, 0L, 0L,
        0L, 1L, 0L,
        0L, 0L, 1L,
        0L, 0L, 0L
      ), nrow = 4, byrow = TRUE),
      m2_name = m2_name(actual$D),
      m2_class = "m2_matrix",
      m2_meta = list(ring = coefring_as_ring("ZZ")),
      base_class = "matrix"
    ),
    P = m2_structure(
      matrix(c(
        1L, 33471L, -43292L, 0L,
        0L, 1L, 0L, 0L,
        0L, 0L, 1L, 0L,
        0L, 0L, 0L, 1L
      ), nrow = 4, byrow = TRUE),
      m2_name = m2_name(actual$P),
      m2_class = "m2_matrix",
      m2_meta = list(ring = coefring_as_ring("ZZ")),
      base_class = "matrix"
    ),
    Q = m2_structure(
      matrix(c(
        171927L, -42421L,  54868L,
        93042L, -22957L,  29693L,
        -74119L,  18288L, -23654L
      ), nrow = 3, byrow = TRUE),
      m2_name = m2_name(actual$Q),
      m2_class = "m2_matrix",
      m2_meta = list(ring = coefring_as_ring("ZZ")),
      base_class = "matrix"
    )
  )

  expect_equal(actual, expect)

})








test_that("snf(m2_matrix.(mat))", {
  skip_on_cran()

  M <- matrix(c(
       1,    2,    3,
       1,   34,   45,
    2213, 1123, 6543,
       0,    0,    0
  ), nrow = 4, byrow = TRUE)

  actual <- snf(m2_matrix.(M))

  expect <- list(
    D = m2_structure(
      matrix(c(
        135654L, 0L, 0L,
        0L, 1L, 0L,
        0L, 0L, 1L,
        0L, 0L, 0L
      ), nrow = 4, byrow = TRUE),
      m2_name = m2_name(actual$D),
      m2_class = "m2_matrix",
      m2_meta = list(ring = coefring_as_ring("ZZ")),
      base_class = "matrix"
    ),
    P = m2_structure(
      matrix(c(
        1L, 33471L, -43292L, 0L,
        0L, 1L, 0L, 0L,
        0L, 0L, 1L, 0L,
        0L, 0L, 0L, 1L
      ), nrow = 4, byrow = TRUE),
      m2_name = m2_name(actual$P),
      m2_class = "m2_matrix",
      m2_meta = list(ring = coefring_as_ring("ZZ")),
      base_class = "matrix"
    ),
    Q = m2_structure(
      matrix(c(
        171927L, -42421L,  54868L,
        93042L, -22957L,  29693L,
        -74119L,  18288L, -23654L
      ), nrow = 3, byrow = TRUE),
      m2_name = m2_name(actual$Q),
      m2_class = "m2_matrix",
      m2_meta = list(ring = coefring_as_ring("ZZ")),
      base_class = "matrix"
    )
  )

  expect_equal(actual, expect)

})

context("m2_matrix")


test_that("m2_matrix(mat)", {
  skip_on_cran()


  stop_m2()
  start_m2()

  M <- matrix(c(
       1,    2,    3,
       1,   34,   45,
    2213, 1123, 6543,
       0,    0,    0
  ), nrow = 4, byrow = TRUE)

  actual <- m2_matrix(M)

  expect <- m2_structure(
    M,
    m2_name = m2_name(actual),
    m2_class = "m2_matrix",
    m2_meta = list(ring = m2_structure(
      m2_name = "ZZ",
      m2_meta = list(vars = NULL, coefring = "ZZ", order = "grevlex"),
      m2_class = "m2_polynomialring"
    )),
    base_class = "matrix"
  )

  expect_equal(actual, expect)

})









test_that("m2_matrix.(mat)", {
  skip_on_cran()

  M <- matrix(c(
       1,    2,    3,
       1,   34,   45,
    2213, 1123, 6543,
       0,    0,    0
  ), nrow = 4, byrow = TRUE)

  actual <- m2_matrix.(M)

  expect <- structure(NA,
    class = c("m2_pointer", "m2"),
    m2_name = m2_name(actual),
    m2_meta = list(
      ext_str = "map((ZZ)^4,(ZZ)^3,{{1, 2, 3}, {1, 34, 45}, {2213, 1123, 6543}, {0, 0, 0}})",
      m2_class = "Matrix",
      m2_class_class = "Type"
    )
  )

  expect_equal(actual, expect)

})

#' Convert a M2 object into an R object
#'
#' Convert a M2 object into an R object
#'
#' @param s a character(1), typically the result of running toExternalString on
#'   an M2 object
#' @param x an object to be printed
#' @param ... ...
#' @return an R object
#' @name m2_parser
#' @references D. Kahle, C. O'Neill, and J. Sommars (2020). "A Computer Algebra
#' System for R: Macaulay2 and the m2r Package." Journal of Statistical
#' Software, 93(9):1-31.
#' @examples
#'
#' \dontrun{ requires Macaulay2
#'
#' m2("1+1")
#' m2.("1+1")
#' m2_parse(m2.("1+1"))
#'
#' m2("QQ[x,y]")
#' m2.("QQ[x,y]")
#' m2_parse(m2.("QQ[x,y]"))
#'
#' get_m2_gmp()
#' m2("3/2") %>% m2_parse()
#' m2_toggle_gmp() # gmp on
#' m2("3/2") %>% m2_parse()
#' m2("6/4") %>% m2_parse()
#' m2("3345234524352435432/223454325235432524352433245") %>% m2_parse()
#' m2_toggle_gmp() # gmp off
#'
#'
#'
#' m2("50!") %>% m2_parse()
#' m2_toggle_gmp() # gmp on
#' m2("50!") %>% m2_parse()
#' m2_toggle_gmp() # gmp off
#'
#' }






#' @rdname m2_parser
#' @export
m2_parse <- function(s) {

  if (is.m2_pointer(s)) {
    tokens <- m2_tokenize(m2_meta(s, "ext_str"))
  } else if (is.m2(s)) {
    return(s)
  } else {
    tokens <- m2_tokenize(s)
  }

  memoise::forget(mem_m2.)
  memoise::forget(mem_m2_parse)

  ret <- m2_parse_internal(tokens)
  ret <- ret$result

  if (
    is.m2_pointer(s) && is.m2(ret) &&
    !is.null(m2_name(ret)) && m2_name(ret) == ""
  ) {
    m2_name(ret) <- m2_name(s)
  }

  memoise::forget(mem_m2.)
  memoise::forget(mem_m2_parse)

  ret

}










# m2 symbol name character
m2_symbol_chars <- function() {
  c(letters, toupper(letters), 0:9, "'")
}


# m2 operators, sorted by length for easier tokenizing
m2_operators <- function() {
  c(
    "===>", "<==>", "<===",
    "==>", "===", "=!=", "<==", "^**", "(*)", "..<",
    "||", "|-", ">>", ">=", "=>", "==", "<=", "<<", "<-", "++", "^^",
    "^*", "#?", "//", "**", "@@", "..", ".?", "!=", ":=", "->", "_*",
    "~", "|", ">", "=", "<", "+", "^", "%", "#", "&", "\\", "/", "*",
    "@", ".", "?", "!", ":", ";", ",", "-", "_",
    "[", "]", "{", "}", "(", ")"
  )
}






# splits a string containing M2 code into tokens to ease parsing
# places an empty string between each line
m2_tokenize <- function(s) {
  # operatorchars <- unlist(strsplit("=<>!&|_^{}[]()+-*/\\:;.,?`~@#$", "", fixed = TRUE))
  operatorstartchars <- unlist(lapply(m2_operators(), function(s) substr(s,1,1)))

  tokens <- character()

  i <- 1
  while (i <= nchar(s)) {
    curchar <- substr(s, i, i)
    if (curchar %in% m2_symbol_chars()) {

      start <- i
      i <- i + 1
      while (i <= nchar(s) && substr(s, i, i) %in% m2_symbol_chars()) {
        i <- i + 1
      }
      i <- i - 1
      end <- i

      tokens <- append(tokens, substr(s,start,end))

    } else if (curchar %in% operatorstartchars) {

      # substr() is smart enough to not index past the end of the string
      for (op in m2_operators()) {
        if (op == substr(s, i, i + nchar(op) - 1)) {
          tokens <- append(tokens, op)
          i <- i + nchar(op) - 1
          break
        }
      }

    } else if (curchar == "\"") {

      i <- i + 1

      start <- i
      while (i <= nchar(s) && substr(s, i, i) != "\"") {
        if (substr(s, i, i) == "\\") i <- i + 1
        i <- i + 1
      }
      end <- i - 1

      tokens <- append(tokens, c("\"", substr(s,start,end), "\""))

    } else if (curchar == "\n") {

      tokens <- append(tokens, "")

    }

    # skip other whitespace, etc.
    i <- i + 1
  }

  tokens
}







# only used for ring parsing!  Don't get greedy!!!!
mem_m2. <- memoise::memoise(function(x) m2.(x))
mem_m2_parse <- memoise::memoise(function(x) m2_parse(x))


m2_parse_internal <- function(tokens, start = 1) {

  i <- start

  if (tokens[i] == "{") {
    # list: {A, A2 => B2, A3 => B3, C, ...}

    elem <- m2_parse_list(tokens, start = i)
    ret <- elem$result
    i <- elem$nIndex

  } else if (tokens[i] == "[") {
    # array: [A, B, ...]

    elem <- m2_parse_array(tokens, start = i)
    ret <- elem$result
    i <- elem$nIndex

  } else if (tokens[i] == "(") {
    # sequence: (A, B, ...) returned as classed list OR (A) returned as A

    elem <- m2_parse_sequence(tokens, start = i)
    ret <- elem$result
    i <- elem$nIndex

  } else if (tokens[i] == "\"") {
    # string: "stuff"

    error_on_fail(tokens[i+2] == "\"", "Parsing error: malformed string.")
    ret <- m2_structure(tokens[i+1], m2_class = "m2_string")
    i <- i + 3

  } else if (substr(tokens[i], 1, 1) %in% 0:9) {
    # positive integer or rational

    if (length(tokens) > i && str_detect(tokens[i+1], "/")) { # is fraction

      if (get_m2_gmp()) {
        ret <- m2_structure(
          as.bigq(tokens[i], tokens[i+2]),
          m2_class = "m2_rational"
        )
        class(ret) <- c(class(ret), "bigq")
        i <- i + 3
      } else {
        ret <- m2_structure(
          as.integer(tokens[i]) / as.integer(tokens[i+2]),
          m2_class = "m2_float"
        )
        i <- i + 3
      }

    } else if (length(tokens) > i && tokens[i+1] == ".") { # is an unusual float

      ret <- m2_structure(
        as.double(paste0(tokens[i], ".", str_replace(tokens[i+2], "p[0-9]+", ""))),
        m2_class = "m2_float"
      )
      i <- i + 3

    } else { # positive integer

      if (get_m2_gmp()) {
        ret <- m2_structure(
          as.bigz(tokens[i]),
          m2_class = "m2_integer"
        )
        class(ret) <- c(class(ret), "bigz")
        i <- i + 1
      } else {
        ret <- m2_structure(
          as.integer(tokens[i]),
          m2_class = "m2_integer"
        )
        i <- i + 1
      }

    }

  } else if (tokens[i] == ".") {
    # positive float

    ret <- m2_structure(
      as.double(paste0(".", str_replace(tokens[i+1], "p[0-9]+", ""))),
      m2_class = "m2_float"
    )
    i <- i + 2

  } else if (tokens[i] == "-" && i != length(tokens) && tokens[i+1] == ".") {
    # negative float

    ret <- m2_structure(
      -as.double(paste0(".", str_replace(tokens[i+2], "p[0-9]+", ""))),
      m2_class = "m2_float"
    )
    i <- i + 3

  } else if (tokens[i] == "-") {
    # -expression

    elem <- m2_parse_internal(tokens,start = i+1)
    ret <- elem$result
    i <- elem$nIndex

    if (is.integer(ret)) {
      ret <- -ret
    } else {
      ret <- m2_structure(paste0("-", ret), m2_class = "m2_string")
    }

  } else if (tokens[i] == "new") {
    # object creation: new TYPENAME from DATA

    elem <- m2_parse_new(tokens, start = i)
    ret <- elem$result
    i <- elem$nIndex

  } else if (substr(tokens[i], 1, 1) %in% m2_symbol_chars()) {
    # symbol, must be final case handled

    elem <- m2_parse_symbol(tokens, start = i)
    ret <- elem$result
    i <- elem$nIndex

  } else {
    # we can't handle this input

    stop(paste("Parsing error: format not supported: ", tokens[i]))

  }

  if (i > length(tokens)) {
    return(list(result = ret, nIndex = i))
  }

  if (tokens[i] == "=>") {
    # option: A => B

    key <- ret

    elem <- m2_parse_internal(tokens, start = i+1)
    val <- elem$result
    i <- elem$nIndex

    ret <- list(key, val)
    class(ret) <- c("m2_option","m2")

  } else if (tokens[i] == "..") {
    # sequence: (a..c) = (a, b, c)

    start <- ret

    elem <- m2_parse_internal(tokens, start = i+1)
    end <- elem$result
    i <- elem$nIndex

    if (all(c(start,end) %in% letters) && start <= end) {
      ret <- as.list(start %:% end)
      ret <- lapply(ret, `class<-`, c("m2_symbol","m2"))
    } else if (all(c(start,end) %in% toupper(letters)) && start <= end) {
      ret <- as.list(start %:% end)
      ret <- lapply(ret, `class<-`, c("m2_symbol","m2"))
    } else if (is.integer(start) && is.integer(end) && start <= end) {
      ret <- as.list(start:end)
    } else {
      ret <- list()
    }

    class(ret) <- c("m2_sequence","m2")

  } else if (tokens[i] == ":") {
    # sequence: (n:x) = (x,...,x)

    num_copies <- ret

    elem <- m2_parse_internal(tokens, start = i+1)
    item <- elem$result
    i <- elem$nIndex

    ret <- replicate(num_copies, item, simplify = FALSE)
    class(ret) <- c("m2_sequence","m2")

  } else if (#class(ret)[1] %in% c("m2_ring","m2_symbol") &&
             (tokens[i] %notin% c(m2_operators(),",") ||
              tokens[i] %in% c("(","{","["))) {
    # function call

    if (tokens[i] == "(") {
      elem <- m2_parse_sequence(tokens, start = i, save_paren = TRUE)
    } else {
      elem <- m2_parse_internal(tokens, start = i)
      elem$result <- list(elem$result)
    }

    params <- elem$result
    i <- elem$nIndex

    ret <- m2_parse_object_as_function(ret, params)

  } else if (tokens[i] %in% c("+","-","*","^")) {
    # start of an expression, consume rest of expression

    lhs <- ret
    operand <- tokens[i]

    elem <- m2_parse_internal(tokens, start = i + 1)
    rhs <- elem$result
    i <- elem$nIndex

    if (is.m2_polynomialring(lhs)) {
      ret <- list(lhs, rhs)
      class(ret) <- c("m2_module","m2")
    } else {
      ret <- paste0(lhs, operand, rhs)

      if ((is.integer(lhs) || class(lhs)[1] %in% c("m2_expression", "m2_symbol")) &&
          (is.integer(rhs) || class(rhs)[1] %in% c("m2_expression", "m2_symbol"))) {
        class(ret) <- c("m2_expression", "m2")
      }
    }

  }

  list(result = ret, nIndex = i)

}







# x is a list interpreted as a M2 list
# class name is m2_M2CLASSNAME in all lower case
# example: x = list(1,2,3), class(x) = c("m2_verticallist","m2")
m2_parse_class <- function(x) UseMethod("m2_parse_class")
m2_parse_class.default <- function(x) x

m2_parse_class.m2_hashtable <- m2_parse_class.default
m2_parse_class.m2_optiontable <- m2_parse_class.m2_hashtable
m2_parse_class.m2_verticallist <- m2_parse_class.m2_hashtable





# x is a list of function parameters
# class name is m2_M2FUNCTIONNAME in all lower case
# example: x = list(mpoly("x")), class(x) = c("m2_symbol","m2")
m2_parse_function <- function(x) UseMethod("m2_parse_function")
m2_parse_function.default <- function(x) stop(paste0("Unsupported function ", class(x)[1]))

m2_parse_function.m2_hashtable <- function(x) x[[1]]
m2_parse_function.m2_optiontable <- m2_parse_function.m2_hashtable
m2_parse_function.m2_verticallist <- m2_parse_function.m2_hashtable


m2_parse_function.m2_symbol <- function(x) {

  class(x[[1]]) <- c("m2_symbol","m2")
  x[[1]]

}


m2_parse_function.m2_monoid <- function(x) {

  class(x[[1]]) <- c("m2_monoid","m2")
  x[[1]]

}


m2_parse_function.m2_tocc <- function(x) {
  m2_structure(complex(real = x[[1]], imaginary = x[[2]]), m2_class = "m2_complex")
}


# x is an object being applied (as a function) to params
# example: x = monoid, params = [x,y,z]
# example: x = QQ, params = monoid [x..z]
m2_parse_object_as_function <- function(x, params) UseMethod("m2_parse_object_as_function")
m2_parse_object_as_function.default <- function(x, params) stop(paste0("Unsupported object ", class(x)[1], " used as function"))


# x is a function name
# dispatch for function call
m2_parse_object_as_function.m2_symbol <- function(x, params) {

  class(params) <- c(paste0("m2_",tolower(x)),"m2")

  ret <- m2_parse_function(params)

}






m2_parse_new <- function(tokens, start = 1) {

  i <- start

  error_on_fail(tokens[i] == "new", "Parsing error: malformed new object")
  error_on_fail(tokens[i+2] == "from", "Parsing error: malformed new object")

  elem <- m2_parse_internal(tokens, start = i+3)
  ret <- elem$result
  i <- elem$nIndex

  class(ret) <- c(paste0("m2_",tolower(tokens[start+1])),"m2")

  m2_parse_class(ret)

  list(result = ret, nIndex = i)

}






m2_parse_symbol <- function(tokens, start = 1) {

  i <- start + 1
  sym_name <- tokens[i-1]

  ptr <- mem_m2.(sym_name)

  if (m2_meta(ptr, "m2_class") %in% m2_ring_class_names()) {

    ret <- ""
    if (sym_name %in% m2_coefrings()) {
      ret <- coefring_as_ring(sym_name)
    } else {
      ret <- mem_m2_parse(ptr)
      m2_name(ret) <- sym_name
    }

    while (i <= length(tokens) && tokens[i] == "_") i <- i + 2

    return(list(result = ret, nIndex = i))

  }

  ret <- sym_name
  while (i <= length(tokens) && tokens[i] == "_") {
    i <- i + 1

    if (tokens[i] == "(") {
      seqret <- m2_parse_sequence(tokens, start = i)
      ret <- paste0(
        ret,
        paste0(unlist(tokens[i:seqret$nIndex-1]), collapse = "")
      )
      i <- seqret$nIndex
    } else {
      ret <- paste0(ret,"_",tokens[i])
    }

    i <- i + 1

  }

  if (ret == "true") {
    ret <- m2_structure(TRUE, m2_class = "m2_boolean")
  } else if (ret == "false") {
    ret <- m2_structure(FALSE, m2_class = "m2_boolean")
  } else if (ret == "null") {
    ret <- NULL
  } else {
    # this is an actual symbol
    class(ret) <- c("m2_symbol","m2")
  }

  list(result = ret, nIndex = i)

}





# {A1 => B1, A2 => B2, ...}
m2_parse_list <- function(tokens, start = 1, open_char = "{", close_char = "}", type_name = "list") {

  ret <- list()
  i <- start + 1

  error_on_fail(tokens[i-1] == open_char, paste0("Parsing error: malformed ", type_name))

  if (tokens[i] == close_char) {
    i <- i + 1
  } else {
    repeat {

      elem <- m2_parse_internal(tokens, start = i)
      ret <- append(ret, list(elem$result))
      i <- elem$nIndex + 1

      if (tokens[i-1] == close_char) {
        break()
      }

      error_on_fail(tokens[i-1] == ",", paste0("Parsing error: malformed ", type_name))
      error_on_fail(i <= length(tokens), paste0("Parsing error: malformed ", type_name))

    }
  }

  class(ret) <- c(paste0("m2_",type_name),"m2")

  list(result = ret, nIndex = i)

}





# [A, B, ...]
m2_parse_array <- function(tokens, start = 1) {

  m2_parse_list(tokens, start = start, open_char = "[", close_char = "]", type_name = "array")

}






# (A, B, ...) as classed list
# (A1) as A1
m2_parse_sequence <- function(tokens, start = 1, save_paren = FALSE) {

  elem <- m2_parse_list(tokens, start = start, open_char = "(", close_char = ")", type_name = "sequence")

  # if sequence has only one element
  if (length(elem$result) == 1 && !save_paren) {
    elem$result <- elem$result[[1]]
  }

  elem

}



#' @rdname m2_parser
#' @export
print.m2_integer <- function(x, ...) {
  if(inherits(x, "bigz")) return(get("print.bigz", envir = asNamespace("gmp"))(x))
  class(x) <- "numeric"
  print(x)
  invisible(x)
}


#' @rdname m2_parser
#' @export
print.m2_float <- function(x, ...) {
  class(x) <- "numeric"
  print(x)
  invisible(x)
}


#' @rdname m2_parser
#' @export
print.m2_complex <- function(x, ...) {
  class(x) <- "complex"
  print(x)
  invisible(x)
}


#' @rdname m2_parser
#' @export
print.m2_string <- function(x, ...) {
  class(x) <- "character"
  print(x)
  invisible(x)
}

#' @rdname m2_parser
#' @export
print.m2_boolean <- function(x, ...) {
  print(unclass(x))
  invisible(x)
}


#' @rdname m2_parser
#' @export
print.m2_list <- function(x, ...) {
  cat("M2 List\n")
  print(unclass(x))
  invisible(x)
}


#' @rdname m2_parser
#' @export
print.m2_array <- function(x, ...) {
  cat("M2 Array\n")
  print(unclass(x))
  invisible(x)
}


#' @rdname m2_parser
#' @export
print.m2_sequence <- function(x, ...) {
  cat("M2 Sequence\n")
  print(unclass(x))
  invisible(x)
}


#' @rdname m2_parser
#' @export
print.m2_symbol <- function(x, ...) {
  cat("M2 Symbol:", x, "\n")
  invisible(x)
}


#' @rdname m2_parser
#' @export
print.m2_option <- function(x, ...) {
  cat("M2 Option\n")
  cat(x[[1]], "=>", x[[2]])
  invisible(x)
}


#' @rdname m2_parser
#' @export
print.m2_hashtable <- function(x, ...) {
  cat("M2 HashTable\n")
  print(unclass(x))
  invisible(x)
}

#' @rdname m2_parser
#' @export
print.m2_module <- function(x, ...) {
  cat("M2 Module\n")
  print(unclass(x))
  invisible(x)
}




error_on_fail <- function(t, e) {
  if (!t) stop(e)
}







#' @rdname m2_parser
#' @export
m2_toggle_gmp <- function() {
  options <- getOption("m2r")
  options$gmp <- !options$gmp
  if (options$gmp) {
    message("m2r is now using gmp.")
  } else {
    message("m2r is no longer using gmp.")
  }
  options(m2r = options)
}



#' @rdname m2_parser
#' @export
get_m2_gmp <- function() getOption("m2r")$gmp

#' Macaulay2 object tests
#'
#' Predicate functions for Macaulay2 objects.
#'
#' @param x an object
#' @return logical(1)
#' @name is
#' @examples
#'
#' \dontrun{ requires Macaulay2
#'
#' R <- ring(c("x1", "x2", "x3"))
#' is.m2(R)
#' is.ring(R)
#' is.ring(10)
#' is.ring(mp("x+1"))
#'
#' }


#' @export
#' @rdname is
is.m2 <- function (x) inherits(x, "m2")



#' @export
#' @rdname is
is.m2_pointer <- function (x) inherits(x, "m2_pointer")



#' @export
#' @rdname is
is.ring <- function (x) inherits(x, "m2_polynomialring")


#' @export
#' @rdname is
is.m2_polynomialring <- function (x) inherits(x, "m2_polynomialring")

#' @export
#' @rdname is
is.m2_polynomialring_pointer <- function (x) is.m2_pointer(x) && (m2_meta(x, "m2_class") == "PolynomialRing")


#' @export
#' @rdname is
is.m2_grobner_basis <- function (x) inherits(x, "m2_grobner_basis")

#' @export
#' @rdname is
is.m2_ideal <- function (x) inherits(x, "m2_ideal")

#' @export
#' @rdname is
is.m2_ideal_pointer <- function (x) is.m2_pointer(x) && (m2_meta(x, "m2_class") == "Ideal")
# is.m2_ideal_pointer <- function (x) is.m2_pointer(x) && (m2_meta(x, "m2_class") == "PolynomialRing")

#' @export
#' @rdname is
is.m2_ideal_list <- function (x) inherits(x, "m2_ideal_list")

#' @export
#' @rdname is
is.m2_ideal_list_pointer <- function (x) inherits(x, "m2_ideal_list_pointer")

#' @export
#' @rdname is
is.m2_module <- function (x) inherits(x, "m2_module")


#' @export
#' @rdname is
is.m2_option <- function (x) inherits(x, "m2_option")


#' @export
#' @rdname is
is.m2_matrix <- function (x) inherits(x, "m2_matrix")


#' @export
#' @rdname is
is.m2_matrix_pointer <- function (x) is.m2_pointer(x) && (m2_meta(x, "m2_class") == "Matrix")


#' @export
#' @rdname is
is.m2_list <- function (x) inherits(x, "m2_list")


#' @export
#' @rdname is
is.m2_array <- function (x) inherits(x, "m2_array")


#' @export
#' @rdname is
is.m2_sequence <- function (x) inherits(x, "m2_sequence")

#' PHCpack
#'
#' Call PHCpack to solve a zero-dimensional system
#'
#' Note that \code{solve_system()} doesn't take in an input ring
#' because the solver only works over the complex numbers.
#'
#' @param mpolyList An mpolyList object
#' @return (currently) the output of an m2() call (string?)
#' @name phc
#' @examples
#'
#' \dontrun{ requires Macaulay2
#'
#' # for this to work, you need to have modified your
#' # init-PHCpack.m2 file instead of changing your .bashrc
#' # file to establish the path of phc
#' # (**clarify**, maybe checkout algstat::polySolve)
#'
#' (mpolyList <- mp(c("t^4 - x", "t^3 - y", "t^2 - z", "x+y+z")))
#' solve_system(mpolyList)
#' mixed_volume(mpolyList)
#'
#' }


#' @rdname phc
#' @export
solve_system <- function (mpolyList) {

  # run m2
  pointer <- solve_system.(mpolyList)

  # parse output
  m2_structure(
    m2_name = m2_name(pointer),
    m2_class = "m2_solutions",
    m2_meta = list(
      sols = m2_pts_str_to_list(m2_meta(pointer, "ext_str"))
    )
  )

}



#' @rdname phc
#' @export
solve_system. <- function (mpolyList) {

  poly_str <- mpolyList_to_m2_str(mpolyList)
  var_str <- suppressMessages(paste0(vars(mpolyList), collapse=","))

  # create m2 code
  m2_code <- sprintf('
    needsPackage "PHCpack"
    R := CC[%s];
    pts := solveSystem %s;
    for i in 0..(#pts-1) list (toExternalString pts#i#Coordinates)
  ', var_str, listify(poly_str))

  # run m2 code and return pointer
  m2.(m2_code)
}



#' @rdname phc
#' @export
mixed_volume <- function (mpolyList) {
  # If mpoly supported complex coefficients, then this should be modified to
  # support a start system
  poly_str <- mpolyList_to_m2_str(mpolyList)
  var_str <- suppressMessages(paste0(vars(mpolyList), collapse=","))
  m2_code <- sprintf('
    needsPackage "PHCpack"
    R := CC[%s];
    mixedVolume( %s )
  ', var_str, listify(poly_str))

  m2_meta(m2.(m2_code), "ext_str")
}






m2_pts_str_to_list <-function(m2_out) {
  m2_out <- str_sub(m2_out,2,-2)
  m2_out <- str_replace_all(m2_out, "p53", "")
  m2_out <- str_replace_all(m2_out, "\"", "")
  m2_out <- str_replace_all(m2_out, "toCC\\(", "complex(real=")
  m2_out <- str_replace_all(m2_out, ",complex", "Dcomplex")
  m2_out <- str_replace_all(m2_out, "\\},", "\\}")
  m2_out <- str_replace_all(m2_out, ",",",imaginary=")
  m2_out <- str_replace_all(m2_out, "Dcomplex", ",complex")
  m2_out <- str_replace_all(m2_out, "\\{", "c\\(")
  m2_out <- str_replace_all(m2_out, "\\}", "\\),")
  m2_out <- paste0("list(",str_sub(m2_out,0,-2),")")
  m2_out <- eval(parse(text=m2_out))
  m2_out
}

#' Create a new ring in Macaulay2
#'
#' Create a new ring in Macaulay2
#'
#' @param vars vector of variable names
#' @param coefring coefficient ring (default: \code{"CC"})
#' @param order a term order (default: \code{"grevlex"})
#' @param code return only the M2 code? (default: \code{FALSE})
#' @param x formal argument for print method
#' @param ... ...
#' @return a reference to a Macaulay2 ring
#' @name ring
#' @examples
#'
#' \dontrun{ requires Macaulay2
#'
#' ##### basic usage
#' ########################################
#'
#' ring("x", "y")
#' ring("x", "y", coefring = "QQ")
#'
#'
#' ##### standard evaluation
#' ########################################
#'
#' ring_(c("x", "y"))
#' ring_(c("x", "y"), code = TRUE)
#'
#' (myring <- ring_(c("x1","x2","x3","y"), coefring = "QQ", order = "lex"))
#'
#' m2_name(myring)
#' m2_meta(myring, "vars")
#' m2_meta(myring, "coefring")
#' m2_meta(myring, "order")
#'
#' ##### other options
#' ########################################
#'
#' ring_.(c("x", "y"))
#' ring_.(c("x", "y"), code = TRUE)
#'
#' }









#' @rdname ring
#' @export
ring <- function(..., coefring = m2_coefrings(), order = m2_termorders(),
  code = FALSE
) {

  # grab args
  x <- list(vars = lapply(dots(...), eval, envir = parent.frame()))
  otherArgs <- as.list(match.call(expand.dots = FALSE))[-c(1:2)]

  # eval
  args <- lapply(c(x, otherArgs), eval)

  # run standard evaluation gb
  do.call("ring_", args)

}





#' @rdname ring
#' @export
ring. <- function(..., coefring = m2_coefrings(), order = m2_termorders(),
  code = FALSE
) {

  # grab args
  x <- list(vars = lapply(dots(...), eval, envir = parent.frame()))
  otherArgs <- as.list(match.call(expand.dots = FALSE))[-c(1:2)]

  # eval
  args <- lapply(c(x, otherArgs), eval)

  # run standard evaluation gb
  do.call("ring_.", args)

}








#' @rdname ring
#' @export
ring_ <- function(vars, coefring = m2_coefrings(), order = m2_termorders(),
  code = FALSE, ...
) {

  # arg checking
  coefring <- match.arg(coefring)
  order <- match.arg(order)

  # run ideal.
  args <- as.list(match.call())[-1]
  eargs <- lapply(args, eval, envir = parent.frame())
  pointer <- do.call(ring_., eargs)
  if(code) return(invisible(pointer))

  # run ring.
  # pointer <- ring_.(vars, coefring, order, code)
  # if(code) return(invisible(pointer))

  # construct R-side ring, class and return
  m2_structure(
    m2_name = m2_name(pointer),
    m2_class = "m2_polynomialring",
    m2_meta = list(
      vars = vars,
      coefring = coefring,
      order = order
    )
  )

}



#' @rdname ring
#' @export
ring_. <- function(vars, coefring = m2_coefrings(), order = m2_termorders(),
  code = FALSE, ...
) {

  # check args
  coefring <- match.arg(coefring)
  order <- match.arg(order)

  # caps order for M2
  order <- switch(order, lex = "Lex", glex = "GLex", grevlex = "GRevLex")

  # set glex order
  if (order == "GLex") {
    # {Weights => {1,1,1,1,1}, Lex => 4}
    m2order <- sprintf(
      "{Weights => {%s}, Lex => %d}",
      paste(rep("1",length(vars)), collapse = ","),
      length(vars)-1
    )
  } else {
    # {Lex => 5}
    m2order <- paste0("{", order, " => ", length(vars), "}")
  }

  # make ring name
  ringname <- name_and_increment("ring", "m2_ring_count")

  # sortedvars <- vars(reorder(mp(paste(vars, collapse = " ")), order = order))
  sortedvars <- vars

  # construct code and message
  line <- sprintf(
    "%s = %s[%s,MonomialOrder=>%s]",
    ringname, coefring, paste(sortedvars, collapse = ","), m2order
  )
  if(code) { message(line); return(invisible(line)) }

  # run m2
  ret <- m2.(line)

  m2_name(ret) <- ringname
  ret
}





# m2 coefficient rings currently supported
coefring_as_ring <- function(coefring) {

  m2_structure(
    m2_name = coefring,
    m2_class = "m2_polynomialring",
    m2_meta = list(
      vars = NULL,
      coefring = coefring,
      order = "grevlex"
    )
  )

}



m2_parse_object_as_function.m2_polynomialring <- function(x, params) {

  monoid <- params[[c(1)]]
  vars <- c(m2_meta(x, "vars"))
  order <- "grevlex"

  for (i in 1:length(monoid)) {
    if (!is.m2_option(monoid[[i]])) {
      vars <- c(vars, unlist(monoid[[i]]))
    } else if (monoid[[c(i,1)]] == "MonomialOrder") {
      for (j in 1:length(monoid[[c(i,2)]])) {
        if (
          is.m2_option(monoid[[c(i,2,j)]]) &&
          monoid[[c(i,2,j,1)]] %in% c("Lex", "Weights", "GRevLex")
        ) {

          order <- monoid[[c(i,2,j,1)]]

          # extra checking for glex
          if (
            order == "Weights" &&
            all(unlist(lapply(monoid[[c(i,2,j,2)]], function(x) x==1)))
          ) {
            order <- "glex"
            break()
          }

          order <- switch(order, Lex = "lex", GRevLex = "grevlex")

        }
      }
    }
  }

  m2_structure(
    m2_name = "",
    m2_class = "m2_polynomialring",
    m2_meta = list(
      vars = vars,
      coefring = m2_meta(x, "coefring"),
      order = order
    )
  )

}




# m2 coefficient rings currently supported
#' @rdname ring
#' @export
m2_coefrings <- function() c("CC", "RR", "QQ", "ZZ")




# m2 term orders currently supported
#' @rdname ring
#' @export
m2_termorders <- function() c("grevlex", "lex", "glex")




m2_ring_class_names <- function() {
  c(
    "Ring","PolynomialRing","QuotientRing",
    "InexactFieldFamily","InexactField"
  )
}


#' @rdname ring
#' @export
print.m2_polynomialring <- function(x, ...){

  s <- sprintf(
    "M2 Ring: %s[%s], %s order",
    m2_meta(x, "coefring"),
    paste(m2_meta(x, "vars"), collapse = ","),
    m2_meta(x, "order")
  )
  cat(s, "\n", sep = "")

  invisible(x)
}

context("ideal  ")














context("ideal_ ")



test_that("ideal_(poly_chars)",{
  skip_on_cran()

  R <- ring_(c("x","y"), "QQ")

  poly_chars <- c("x + y", "x^2 + y^2")
  actual <- ideal_(poly_chars)

  expect <- m2_structure(
    m2_name = m2_name(actual),
    m2_class = "m2_ideal",
    m2_meta = list(ring = R, gens = mp(poly_chars))
  )

  expect_equal(actual, expect)
})


test_that("ideal_(mpolyList)",{
  skip_on_cran()

  R <- ring_(c("x","y"), "QQ")

  mpolyList <- mp(c("x + y", "x^2 + y^2"))
  actual <- ideal_(mpolyList)

  expect <- m2_structure(
    m2_name = m2_name(actual),
    m2_class = "m2_ideal",
    m2_meta = list(ring = R, gens = mpolyList)
  )

  expect_equal(actual, expect)
})


test_that("ideal_(list o chars)",{
  skip_on_cran()

  R <- ring_(c("x","y"), "QQ")

  list_o_chars <- list("x + y", "x^2 + y^2")
  actual <- ideal_(list_o_chars)

  expect <- m2_structure(
    m2_name = m2_name(actual),
    m2_class = "m2_ideal",
    m2_meta = list(ring = R, gens = mp(unlist(list_o_chars)))
  )

  expect_equal(actual, expect)
})


test_that("ideal_(list o mps)",{
  skip_on_cran()

  R <- ring_(c("x","y"), "QQ")

  list_o_mps <- list(mp("x + y"), mp("x^2 + y^2"))
  actual <- ideal_(list_o_mps)

  expect <- m2_structure(
    m2_name = m2_name(actual),
    m2_class = "m2_ideal",
    m2_meta = list(ring = R, gens = structure(list_o_mps, class = "mpolyList"))
  )

  expect_equal(actual, expect)
})


test_that("ideal_(list o chars, raw_chars = TRUE)",{
  skip_on_cran()

  R <- ring_(c("x","y"), "QQ")

  list_o_chars <- list("x + y", "x^2 + y^2")
  actual <- ideal_(list_o_chars, raw_chars = TRUE)

  expect <- m2_structure(
    m2_name = m2_name(actual),
    m2_class = "m2_ideal",
    m2_meta = list(ring = R, gens = mp(unlist(list_o_chars)))
  )

  expect_equal(actual, expect)
})


test_that("ideal_(poly_chars, raw_chars = TRUE)",{
  skip_on_cran()

  R <- ring_(c("x","y"), "QQ")

  poly_chars <- c("x + y", "x^2 + y^2")
  actual <- ideal_(poly_chars, raw_chars = TRUE)

  expect <- m2_structure(
    m2_name = m2_name(actual),
    m2_class = "m2_ideal",
    m2_meta = list(ring = R, gens = mp(poly_chars))
  )

  expect_equal(actual, expect)
})


test_that("ideal_(c(mpoly, mpoly) errors",{
  skip_on_cran()

  R <- ring_(c("x","y"), "QQ")

  expect_error(
    ideal_(c(mp("x+y"), mp("x^2+y^2"))),
    "you appear to have used c*"
  )
})


test_that('ideal_(list("x+y x^2+y^2"), raw_chars = TRUE) errors',{
  skip_on_cran()

  R <- ring_(c("x","y"), "QQ")

  expect_error(
    ideal_(list("x+y x^2+y^2"), raw_chars = TRUE),
    "Macaulay2 Error"
  )
})













context("ideal_.")



test_that("ideal_.(poly_chars)",{
  skip_on_cran()

  R <- ring_.(c("x","y"), "QQ")

  poly_chars <- c("x + y", "x^2 + y^2")
  actual <- ideal_.(poly_chars)

  expect <- m2_structure(
    m2_name = m2_name(actual),
    m2_class = "m2_pointer",
    m2_meta = list(
      ext_str = sprintf("ideal map((%1$s)^1,(%1$s)^{{-1},{-2}},{{x+y, x^2+y^2}})", m2_name(R)),
      m2_class = "Ideal",
      m2_class_class = "Type"
    )
  )

  expect_equal(actual, expect)
})


test_that("ideal_.(mpolyList)",{
  skip_on_cran()

  R <- ring_.(c("x","y"), "QQ")

  mpolyList <- mp(c("x + y", "x^2 + y^2"))
  actual <- ideal_.(mpolyList)

  expect <- m2_structure(
    m2_name = m2_name(actual),
    m2_class = "m2_pointer",
    m2_meta = list(
      ext_str = sprintf("ideal map((%1$s)^1,(%1$s)^{{-1},{-2}},{{x+y, x^2+y^2}})", m2_name(R)),
      m2_class = "Ideal",
      m2_class_class = "Type"
    )
  )

  expect_equal(actual, expect)
})


test_that("ideal_.(list o chars)",{
  skip_on_cran()

  R <- ring_.(c("x","y"), "QQ")

  list_o_chars <- list("x + y", "x^2 + y^2")
  actual <- ideal_.(list_o_chars)

  expect <- m2_structure(
    m2_name = m2_name(actual),
    m2_class = "m2_pointer",
    m2_meta = list(
      ext_str = sprintf("ideal map((%1$s)^1,(%1$s)^{{-1},{-2}},{{x+y, x^2+y^2}})", m2_name(R)),
      m2_class = "Ideal",
      m2_class_class = "Type"
    )
  )

  expect_equal(actual, expect)
})


test_that("ideal_.(list o mps)",{
  skip_on_cran()

  R <- ring_.(c("x","y"), "QQ")

  list_o_mps <- list(mp("x + y"), mp("x^2 + y^2"))
  actual <- ideal_.(list_o_mps)

  expect <- m2_structure(
    m2_name = m2_name(actual),
    m2_class = "m2_pointer",
    m2_meta = list(
      ext_str = sprintf("ideal map((%1$s)^1,(%1$s)^{{-1},{-2}},{{x+y, x^2+y^2}})", m2_name(R)),
      m2_class = "Ideal",
      m2_class_class = "Type"
    )
  )

  expect_equal(actual, expect)
})


test_that("ideal_.(list o chars, raw_chars = TRUE)",{
  skip_on_cran()

  R <- ring_.(c("x","y"), "QQ")

  list_o_chars <- list("x + y", "x^2 + y^2")
  actual <- ideal_.(list_o_chars, raw_chars = TRUE)

  expect <- m2_structure(
    m2_name = m2_name(actual),
    m2_class = "m2_pointer",
    m2_meta = list(
      ext_str = sprintf("ideal map((%1$s)^1,(%1$s)^{{-1},{-2}},{{x+y, x^2+y^2}})", m2_name(R)),
      m2_class = "Ideal",
      m2_class_class = "Type"
    )
  )

  expect_equal(actual, expect)
})


test_that("ideal_.(poly_chars, raw_chars = TRUE)",{
  skip_on_cran()

  R <- ring_.(c("x","y"), "QQ")

  poly_chars <- c("x + y", "x^2 + y^2")
  actual <- ideal_.(poly_chars, raw_chars = TRUE)

  expect <- m2_structure(
    m2_name = m2_name(actual),
    m2_class = "m2_pointer",
    m2_meta = list(
      ext_str = sprintf("ideal map((%1$s)^1,(%1$s)^{{-1},{-2}},{{x+y, x^2+y^2}})", m2_name(R)),
      m2_class = "Ideal",
      m2_class_class = "Type"
    )
  )

  expect_equal(actual, expect)
})








test_that("ideal_.(c(mpoly, mpoly) errors",{
  skip_on_cran()

  R <- ring_.(c("x","y"), "QQ")

  expect_error(
    ideal_.(c(mp("x+y"), mp("x^2+y^2"))),
    "you appear to have used c*"
  )
})


test_that('ideal_.(list("x+y x^2+y^2"), raw_chars = TRUE) errors',{
  skip_on_cran()

  R <- ring_.(c("x","y"), "QQ")

  expect_error(
    ideal_.(list("x+y x^2+y^2"), raw_chars = TRUE),
    "Macaulay2 Error"
  )
})

context("ring  ")



context("ring_ ")





test_that("ring_(formats)",{
  skip_on_cran()

  param1 <- list(
    c("x"),
    c("x","y"),
    letters
  )

  param2 <- as.list(m2_coefrings())

  param3 <- as.list(m2_termorders())

  apply(expand.grid(param1, param2, param3), 1, FUN = function(x) {

    R <- ring_(x[[1]], coefring = x[[2]], order = x[[3]])

    expected_output <- m2_structure(
      m2_name = m2_name(R),
      m2_class = "m2_polynomialring",
      m2_meta = list(
        vars = x[[1]],
        coefring = x[[2]],
        order = x[[3]]
      )
    )

    expect_equal(R, expected_output)
    expect_equal(R, m2_parse(m2_name(R)))

  })

})

#' Set path to Macaulay2 (M2)
#'
#' These are helper functions that deal with pathing to Macaulay2 and asking if it
#' is present. When the Macaulay2 package is loaded it attempts to find the
#' Macaulay2 executable by looking for an environment variable indicating where it
#' is, i.e. its path as specified in your .Renviron file.
#'
#' For easiest use, you'll want to specify the path the Macaulay2 executable in
#' your ~/.Renviron file. It should look something like
#'
#' \code{M2=/Applications/Macaulay2-1.10/bin}
#'
#' You can set this permanently with [edit_r_environ()]. Note that absolute
#' paths should be specified, not relative paths, e.g. don't use ~/path/to/exe.
#'
#' You can change this for the current session using [set_m2_path()], which
#' accepts a character string or, if missing, uses [file.choose()] to let you
#' interactively; you just select an arbitrary executable.
#'
#' On Windows, m2r just defaults to the cloud implementation. Local M2 instances
#' are not currently supported on Windows.
#'
#' @param path A character string, the path to M2
#' @return An invisible character string, the path found.  More importantly, the
#'   function has the side effect of setting the global m2r option "m2_path"
#' @export
#' @name m2_path
#' @author David Kahle \email{david@@kahle.com}
#' @examples
#'
#' \dontrun{ requires Macaulay2
#'
#'
#' getOption("m2r")
#' get_m2_path()
#' set_m2_path()
#'
#'
#' ## each of these functions can be used statically as well
#' (m2_path <- get_m2_path())
#' set_m2_path("/path/to/m2/directory")
#' get_m2_path()
#' set_m2_path(m2_path) # undoes example
#'
#'
#' # if you'd like to use the cloud, after you library(m2r)
#' # and before you use m2() type
#' set_m2_path(NULL)
#'
#' # alternatively, if you have already been using m2, do:
#' stop_m2()
#' set_m2_path(NULL)
#' m2("1+1")
#'
#'
#' }






#' @rdname m2_path
#' @export
set_m2_path <- function(path = NULL){

  if(missing(path) && interactive()){

    path <- dirname(file.choose())
    if(is.win() && str_detect(path,"C:/")){
      mpath <- str_replace(dirname(path), "C:/", "/cygdrive/c/")
    }
    set_m2r_option(m2_path = path)
    return(invisible(path))

  } else if (!missing(path)) {

    set_m2r_option(m2_path = path)
    return(invisible(path))

  } else {
    stop(
      "If the session is not interactive, a path must be specified.",
      call. = FALSE
    )
  }
}





#' @rdname m2_path
#' @export
get_m2_path <- function() getOption("m2r")$m2_path



#' @rdname m2_path
#' @export
get_m2_connection <- function() getOption("m2r")$m2_con


#' @rdname m2_path
#' @export
get_m2_con <- function() {
  .Deprecated("get_m2_connection")
  getOption("m2r")$m2_con
}



#' @rdname m2_path
#' @export
get_m2_procid <- function() getOption("m2r")$m2_procid



#' @rdname m2_path
#' @export
get_m2_port <- function() getOption("m2r")$m2_port



#' @importFrom usethis edit_r_environ
#' @export
usethis::edit_r_environ

#' Utility tools for M2
#'
#' Utility tools for M2
#'
#' @param x an object of class \code{m2}
#' @param m2_attr the name of an M2 attribute
#' @param name a string; the name of a M2 object
#' @param value the value to assign
#' @param m2_name  \code{m2_name}  M2 attribute
#' @param m2_class \code{m2_class} M2 attribute
#' @param base_class a base class; an R class to use for dispatching
#'   if there is no relevant method for the other classes (e.g.
#'   \code{m2})
#' @param m2_meta  \code{m2_meta}  M2 attribute
#' @param all.names if \code{TRUE}, all registered Macaulay2
#'   variables, including ones internally used by m2r, will be
#'   returned
#' @name m2_utility
#' @examples
#'
#' \dontrun{ requires Macaulay2
#'
#' m2("a = 5")
#' m2_ls()
#' m2_exists("a")
#' m2("b = 1")
#' m2_exists(c("a","b","c"))
#'
#' m2_getwd()
#'
#' x <- 1
#' class(x) <- "m2"
#' attr(x, "m2_meta") <- list(a = 1, b = 2)
#' m2_meta(x)
#' m2_meta(x, "b")
#' m2_meta(x, "b") <- 5
#' m2_meta(x, "b")
#'
#' # R <- ring(c("x1", "x2", "x3"))
#' # m2_name(R)
#' # m2(sprintf("class %s", m2_name(R)))
#' # m2_ls()
#' # m2_rm(m2_name(R))
#' # m2_ls()
#' # m2(paste("class", m2_name(R)))
#'
#' m2_ls()
#' m2_ls(all.names = TRUE)
#'
#'
#' }



#' @rdname m2_utility
#' @export
m2_name <- function (x) {
  if ( is.m2(x) ) {
    attr(x, "m2_name")
  } else {
    character(0)
  }
}


#' @rdname m2_utility
#' @export
`m2_name<-` <- function (x, value) {
  stopifnot( is.m2(x) )
  attr(x, "m2_name") <- value
  x
}


#' @rdname m2_utility
#' @export
m2_meta <- function (x, m2_attr) {
  if ( !is.m2(x) ) return(NULL)
  if ( missing(m2_attr) ) return(attr(x, "m2_meta"))
  attr(x, "m2_meta")[[m2_attr]]
}


#' @rdname m2_utility
#' @export
`m2_meta<-` <- function (x, m2_attr, value) {
  stopifnot( is.m2(x) )
  if (missing(m2_attr)) {
    attr(x, "m2_meta") <- value
  } else {
    meta <- m2_meta(x)
    meta[[m2_attr]] <- value
    attr(x, "m2_meta") <- meta
  }
  x
}


#' @rdname m2_utility
#' @export
m2_structure <- function (x = NA, m2_name, m2_class, m2_meta, base_class) {

  if (!missing(m2_class)) class(x) <- c(m2_class, "m2")
  if (!missing(base_class)) class(x) <- c(class(x), base_class)
  if (!missing(m2_name)) m2_name(x) <- m2_name
  # if (m2_meta(x) != NULL) m2_meta(x)
  if (!missing(m2_meta)) m2_meta(x) <- m2_meta

  x
}


#' @rdname m2_utility
#' @export
m2_exists <- function(name) {
  if(!is.character(name)) name <- deparse(substitute(name))
  name %in% m2_ls()
}


#' @rdname m2_utility
#' @export
m2_ls <- function(all.names = FALSE) {
  out <- m2("userSymbols()")
  out <- str_sub(out, 2, -2)
  out <- str_split(out, ",")[[1]]

  # "symbols m2o1" -> "m2o1"
  out <- str_sub(out, 8)

  # remove internals and m2o#'s
  if(!all.names) {
    out <- out[!str_detect(out, "m2rint")]
    out <- out[!str_detect(out, "m2o[0-9]+")]
  }

  # return
  out
}


#' @rdname m2_utility
#' @export
m2_rm <- function(name) {
  stop("broken.")
  if (!is.m2(name)) return(invisible())
  m2(paste(m2_name(name), "=symbol", m2_name(name)))
  invisible()
}


#' @rdname m2_utility
#' @export
m2_getwd <- function() {
  str_sub(m2("currentDirectory()"), 2, -2)
}

context("m2r parser")

m2_parse_test <- function(m2_str, m2_expected_output) {
  out <- m2_parse(m2_str)
  if (is.m2(out) && !is.null(m2_name(out))) m2_name(out) <- ""

  expect_equal(out, m2_expected_output)
}

test_that("m2_parse parses basic data structures",{
  skip_on_cran()

  m2_list <- "{1,2,3}"
  m2_list_output <- structure(
    list(
      structure(1L, class = c("m2_integer", "m2")),
      structure(2L, class = c("m2_integer", "m2")),
      structure(3L, class = c("m2_integer", "m2"))
    ),
    class = c("m2_list","m2")
  )
  m2_parse_test(m2_list, m2_list_output)

  m2_array <- "[1,2,3]"
  m2_array_output <- structure(
    list(
      structure(1L, class = c("m2_integer", "m2")),
      structure(2L, class = c("m2_integer", "m2")),
      structure(3L, class = c("m2_integer", "m2"))
    ),
    class = c("m2_array", "m2")
  )
  m2_parse_test(m2_array, m2_array_output)

  m2_sequence <- "(1,2,3)"
  m2_sequence_output <- structure(
    list(
      structure(1L, class = c("m2_integer", "m2")),
      structure(2L, class = c("m2_integer", "m2")),
      structure(3L, class = c("m2_integer", "m2"))
    ),
    class = c("m2_sequence", "m2")
  )
  m2_parse_test(m2_sequence, m2_sequence_output)

  m2_str <- "\"my string\""
  m2_str_output <- structure("my string", class = c("m2_string", "m2"))
  m2_parse_test(m2_str, m2_str_output)

  m2_opt <- "c => 4"
  m2_opt_output <- structure(
    list(
      structure("c", class = c("m2_symbol", "m2")),
      structure(4L, class = c("m2_integer", "m2"))
    ),
    class = c("m2_option", "m2")
  )
  m2_parse_test(m2_opt, m2_opt_output)

  m2_new_thing <- "new HashTable from {A => 1}"
  m2_new_thing_output <- structure(
    list(structure(
      list(
        structure("A", class = c("m2_symbol", "m2")),
        structure(1L, class = c("m2_integer", "m2"))
      ),
      class = c("m2_option", "m2")
    )),
    class = c("m2_hashtable", "m2"))
  m2_parse_test(m2_new_thing, m2_new_thing_output)

  # toExternalString(matrix{{1,2,3},{4,5,6}})
  m2_matrix <- "map((ZZ)^2,(ZZ)^3,{{1, 2, 3}, {4, 5, 6}})"
  m2_matrix_output <- m2_structure(
    matrix(c(1,4,2,5,3,6), nrow = 2, ncol = 3),
    m2_name = "",
    m2_class = "m2_matrix",
    m2_meta = list(
      ring = coefring_as_ring("ZZ")
    ),
    base_class = "matrix"
  )
  m2_parse_test(m2_matrix,m2_matrix_output)

  # toExternalString(matrix{{}})
  m2_empty_matrix <- "map((ZZ)^1,(ZZ)^0,0)"
  m2_empty_matrix_output <- m2_structure(
    matrix(numeric(0), nrow = 0, ncol = 0),
    m2_name = "",
    m2_class = "m2_matrix",
    m2_meta = list(
      ring = coefring_as_ring("ZZ")
    ),
    base_class = "matrix"
  )
  m2_parse_test(m2_empty_matrix,m2_empty_matrix_output)

  # Test case where one ring has another ring as a prefix
  m2_ring1 <- m2("R = CC[x]")
  m2_ring2 <- m2("S = R[y]")

  m2_second_ring <- "R(monoid[y, Degrees => {1}, Heft => {1}, MonomialOrder => VerticalList{MonomialSize => 32, GRevLex => {1}, Position => Up}, DegreeRank => 1])"
  m2_second_ring_output <- m2_structure(
    m2_name = "",
    m2_class = "m2_polynomialring",
    m2_meta = list(
      vars = c("x","y"),
      coefring = "CC",
      order = "grevlex"
    )
  )
  m2_parse_test(m2_second_ring,m2_second_ring_output)

  # map(S,R,{s^3-t^2, s^3-t, s-t})
  # m2_ring1 <- m2("R = QQ[a,b,c]")
  # m2_ring2 <- m2("S = QQ[s,t]")
  # m2_ring_map <- "map(S,R,{s^3-t^2, s^3-t, s-t})"
  # m2_parse(m2_ring_map)
  # m2_ring_map_output <- " "
})




test_that("m2 parses harder data structures", {
  skip_on_cran()

  m2_poly_list <- "{b*c*e-a*d*f, a*c*e-b*d*f}"
  m2_poly_list_output <- structure(
    list(
      structure("b*c*e-a*d*f", class = c("m2_expression", "m2")),
      structure("a*c*e-b*d*f", class = c("m2_expression", "m2"))
    ),
    class = c("m2_list", "m2")
  )
  m2_parse_test(m2_poly_list, m2_poly_list_output)

  # toExternalString(factor(2^4*3^3))
  m2_factor <- "new Product from {new Power from {2,4},new Power from {3,3}}"
  m2_factor_output <- structure(
    list(
      structure(
        list(
          structure(2, class = c("m2_integer", "m2")),
          structure(4, class = c("m2_integer", "m2"))
        ), class = c("m2_power" , "m2")
      ),
      structure(
        list(
          structure(3, class = c("m2_integer", "m2")),
          structure(3, class = c("m2_integer", "m2"))
        ), class = c("m2_power" , "m2")
      )
    ),
    class = c("m2_product", "m2")
  )
  m2_parse_test(m2_factor, m2_factor_output)

  m2_compound_test <- "new OptionTable from {Headline => \"test\", Configuration => new OptionTable from {}}"
  m2_compound_test_output <- structure(
    list(
      structure(
        list(
          structure("Headline", class = c("m2_symbol", "m2")),
          structure("test", class = c("m2_string", "m2"))
        ),
        class = c("m2_option","m2")
      ),
      structure(
        list(
          structure("Configuration", class = c("m2_symbol", "m2")),
          structure(list(), class = c("m2_optiontable", "m2"))
        ),
        class = c("m2_option", "m2")
      )
    ),
    class = c("m2_optiontable", "m2")
  )
  m2_parse_test(m2_compound_test, m2_compound_test_output)

  # R = QQ[a,b,c,d,e,f]
  # toExternalString(ideal(a*b*c-d*e*f,a*c*e-b*d*f))
  m2("R = QQ[a,b,c,e,d,f]")
  m2_ideal <- "map((R)^1,(R)^{1},{{a*b*c-d*e*f, a*c*e-b*d*f}})"
  m2_ideal_output <- m2_structure(
    structure(
      list(
        structure(
          list(c("a"=1,"b"=1,"c"=1,"coef"=1), c("d"=1,"e"=1,"f"=1,"coef"=-1)),
          class=c("mpoly")
        ),
        structure(
          list(c("a"=1,"c"=1,"e"=1,"coef"=1), c("b"=1,"d"=1,"f"=1,"coef"=-1)),
          class=c("mpoly")
        )
      ),
      dim=c(1,2)
    ),
    m2_name = "",
    m2_class = "m2_matrix",
    m2_meta = list(
      ring = m2_structure(
        m2_name = "R",
        m2_class = "m2_polynomialring",
        m2_meta = list(
          vars = c("a","b","c","e","d","f"),
          coefring = "QQ",
          order="grevlex"
        )
      )
    ),
    base_class = "matrix"
  )
  m2_parse_test(m2_ideal, m2_ideal_output)

  # toExternalString(CC[x,y])
  m2_ring <- "CC_53(monoid[x..z, a..d, Degrees => {2:1}, Heft => {1}, MonomialOrder => VerticalList{MonomialSize => 32, GRevLex => {2:1}, Position => Up}, DegreeRank => 1])"
  m2_ring_output <- m2_structure(
    m2_name = "",
    m2_class = "m2_polynomialring",
    m2_meta = list(
      vars = c("x", "y", "z", "a", "b", "c", "d"),
      coefring = "CC",
      order = "grevlex"
    )
  )
  m2_parse_test(m2_ring, m2_ring_output)
})

context("m2 ")

test_that("m2 works properly", {
  skip_on_cran()

  expect_equal(m2("1 + 1"), "2")
})

test_that("mutable lists parse to NULL", {
  skip_on_cran()

  expect_equal(m2("new MutableList from {1,2,3}"), NULL)
})

test_that("basic m2 computations work", {
  skip_on_cran()

  expect_equal(m2("1 + 1"), "2")
})



context("m2.")

test_that("m2. works properly", {
  skip_on_cran()

  out <- m2.("1 + 1")
  expect_equal(m2_meta(out), list(ext_str = "2", m2_class = "ZZ", m2_class_class = "Ring"))
  expect_equal(str_detect(m2_name(out), "m2o[0-9]+"), TRUE)
  expect_equal(class(out), c("m2_pointer", "m2"))
})

context("factor_n ")

test_that("factor_n(formats)", {
  skip_on_cran()

  param1 <- list(
    218700,
    "218700",
    m2.("218700")
  )

  expected_output <- list(prime = c(2L, 3L, 5L), power = c(2L, 7L, 2L))

  apply(expand.grid(param1, list(1)), 1, FUN = function(x) {

    factors <- factor_n(x[[1]])

    expect_equal(factors, expected_output)

  })

})


test_that("factor_n(1)", {
  skip_on_cran()

  expect_equal(
    factor_n(1),
    list(prime = integer(0), power = integer(0))
  )

})


test_that("factor_n(negative)", {
  skip_on_cran()

  expect_equal(
    factor_n(-55),
    list(prime = c(-1L, 5L, 11L), power = c(1L, 1L, 1L))
  )

})




context("factor_n.")

test_that("factor_n.(formats)", {
  skip_on_cran()

  param1 <- list(
    218700,
    "218700",
    m2.("218700")
  )

  apply(expand.grid(param1, list(1)), 1, FUN = function(x) {

    factors <- factor_n.(x[[1]])

    expected_output <- m2_structure(
      m2_name = m2_name(factors),
      m2_class = "m2_pointer",
      m2_meta = list(
        ext_str = "new Product from {new Power from {2,2},new Power from {3,7},new Power from {5,2}}",
        m2_class = "Product",
        m2_class_class = "WrapperType"
      )
    )

    expect_equal(factors, expected_output)

  })

})

context("factor_poly ")


test_that("factor_poly(formats)", {
  skip_on_cran()

  # define ring
  ring_(c("x","y"), "QQ")

  # define expected output
  expected_output <- list(
    factor = mp(c("x-y", "x+y", "x^2+y^2")),
    power = c(1L, 1L, 1L)
  )

  expect_equal(
    factor_poly("x^4 - y^4"),
    expected_output
  )

  expect_equal(
    factor_poly(mp("x^4 - y^4")),
    expected_output
  )

  expect_equal(
    factor_poly(m2.("x^4 - y^4")),
    expected_output
  )


})


test_that("factor_poly(1)", {
  skip_on_cran()

  # define ring
  ring_(c("x","y"), "QQ")

  # define expected output
  expected_output <- list(
    factor = structure(list(), class = "mpolyList"),
    power = integer(0)
  )

  # run tests
  expect_equal(
    factor_poly(1),
    expected_output
  )

  expect_equal(
    factor_poly("1"),
    expected_output
  )

  expect_equal(
    factor_poly(mp("1")),
    expected_output
  )

  expect_equal(
    factor_poly(m2.("1")),
    expected_output
  )

})





context("factor_poly.")


test_that("factor_poly(formats)", {
  skip_on_cran()

  # define ring
  ring_(c("x","y"), "QQ")

  # define expected
  expected <- function(name) m2_structure(
    m2_name = name,
    m2_class = "m2_pointer",
    m2_meta = list(
      ext_str = "new Product from {new Power from {x-y,1},new Power from {x+y,1},new Power from {x^2+y^2,1}}",
      m2_class = "Product",
      m2_class_class = "WrapperType"
    )
  )

  # run tests
  actual <- factor_poly.("x^4 - y^4")
  expect_equal(actual, expected(m2_name(actual)))

  actual <- factor_poly.(mp("x^4 - y^4"))
  expect_equal(actual, expected(m2_name(actual)))

  actual <- factor_poly.(m2.("x^4 - y^4"))
  expect_equal(actual, expected(m2_name(actual)))

})

context("gb  ")

test_that("gb(... = char vec)", {
  skip_on_cran()

  ring_(c("x","y","z"), "QQ")

  actual <- gb("x y", "x z", "x")

  expect <- list(mp("x"))
  class(expect) <- c("m2_grobner_basis", "m2", "mpolyList")
  m2_name(expect) <- m2_name(actual)
  m2_meta(expect) <- list(ideal = m2_meta(actual, "ideal"))

  expect_equal(actual, expect)
})







context("gb_ ")


test_that("gb_(char vec)",{
  skip_on_cran()

  ring_(c("x", "y", "z"), "QQ")

  actual <- gb_(c("x y", "x z", "x"))

  expect <- list(mp("x"))
  class(expect) <- c("m2_grobner_basis", "m2", "mpolyList")
  m2_name(expect) <- m2_name(actual)
  m2_meta(expect) <- list(ideal = m2_meta(actual, "ideal"))

  expect_equal(actual, expect)
})


test_that("gb_(mpolyList)",{
  skip_on_cran()

  ring_(c("x", "y", "z"), "QQ")

  mpolys <- mp(c("x y", "x z", "x"))
  actual <- gb_(mpolys)

  expect <- list(mp("x"))
  class(expect) <- c("m2_grobner_basis", "m2", "mpolyList")
  m2_name(expect) <- m2_name(actual)
  m2_meta(expect) <- list(ideal = m2_meta(actual, "ideal"))

  expect_equal(actual, expect)
})


test_that("gb_(list o chars)",{
  skip_on_cran()

  ring_(c("x", "y", "z"), "QQ")

  actual <- gb_(list("x y", "x z", "x"))

  expect <- list(mp("x"))
  class(expect) <- c("m2_grobner_basis", "m2", "mpolyList")
  m2_name(expect) <- m2_name(actual)
  m2_meta(expect) <- list(ideal = m2_meta(actual, "ideal"))

  expect_equal(actual, expect)
})


test_that("gb_(list o mpolys)",{
  skip_on_cran()

  ring_(c("x", "y", "z"), "QQ")

  actual <- gb_(list(mp("x y"), mp("x z"), mp("x")))

  expect <- list(mp("x"))
  class(expect) <- c("m2_grobner_basis", "m2", "mpolyList")
  m2_name(expect) <- m2_name(actual)
  m2_meta(expect) <- list(ideal = m2_meta(actual, "ideal"))

  expect_equal(actual, expect)
})


test_that("gb_(c'd mpolys)",{
  skip_on_cran()

  ring_(c("x", "y", "z"), "QQ")

  expect_error(
    gb_(c(mp("x y"), mp("x z"), mp("x"))),
    "you appear to have used c*"
  )
})


test_that("gb_(m2_ideal)",{
  skip_on_cran()

  ring_(c("x", "y", "z"), "QQ")

  I <- ideal_(mp(c("x y", "x z", "x")))
  actual <- gb_(I)

  expect <- list(mp("x"))
  class(expect) <- c("m2_grobner_basis", "m2", "mpolyList")
  m2_name(expect) <- m2_name(actual)
  m2_meta(expect) <- list(ideal = m2_meta(actual, "ideal"))

  expect_equal(actual, expect)
})


test_that("gb_(m2_ideal_pointer)",{
  skip_on_cran()

  ring_(c("x", "y", "z"), "QQ")

  I. <- ideal_.(mp(c("x y", "x z", "x")))
  actual <- gb_(I.)

  expect <- list(mp("x"))
  class(expect) <- c("m2_grobner_basis", "m2", "mpolyList")
  m2_name(expect) <- m2_name(actual)
  m2_meta(expect) <- list(ideal = m2_meta(actual, "ideal"))

  expect_equal(actual, expect)
})










context("gb_.")

is.mac <- function() grepl("darwin", R.version$platform)
is.win <- function() .Platform$OS.type == "windows"
is.linux <- function() (.Platform$OS.type == "unix") && (is.mac() == FALSE)
is.unix  <- function() .Platform$OS.type == "unix"
is.solaris <- function() grepl("solaris", R.version$os)


file.path2 <- function(...){
	sep <- if(is.unix()) "/" else "\\"
	paste0(list(...), collapse = sep)
}

write_to_temp <- function(contents) {
  tempname <- tempfile()
  writeLines(contents, tempname)
  tempname
}

name_and_increment <- function(prefix, option) {
  # grab # of current rings, set ring number, and increment
  num <- getOption(option)
  num <- ifelse(is.null(num), 1L, strtoi(num))
  setOption(option, num + 1)

  # make name
  sprintf("m2rint%s%08d", prefix, num)
}




listify <- function (strings) paste0("{", paste(strings, collapse = ","), "}")

listify_mat <- function (mat) listify(apply(mat, 1, listify))
# (mat <- matrix(1:9, nrow = 3))
# listify_mat(mat)




delistify <- function (string, f, collapse, ...) {

  # dewhiten
  s <- str_replace_all(string, "[\\s]+", "")

  # kill outside {}
  s <- str_sub(s, 3, -3)

  # break it up
  s <- str_split(s, fixed("},{"))[[1]]
  s <- str_split(s, fixed(","))

  # f, if available
  if ( !missing(f) ) s <- lapply(s, f, ...)

  # collapse, if available
  if ( !missing(collapse) ) s <- do.call(collapse, s)

  # return
  s

}
# string <- "{{1,2,3}, {1,34,45}, {2213,1123,6543}, {0,0,0}}"
# delistify(string)
# str(delistify(string))
# delistify(string, as.integer)
# delistify(string, as.integer, rbind)






mpolyList_to_m2_str <- function(mpolyList) {

  # allow for character vectors
  if (inherits(mpolyList, "mpoly")) mpolyList <- mpolyList(mpolyList)

  # parse it if it's a numeric or character string
  if(is.numeric(mpolyList)) mpolyList <- as.character(mpolyList)
  if(is.character(mpolyList)) mpolyList <- mp(mpolyList)

  # convert mpolylist to strings readable by m2
  vec <- vapply(mpolyList, print, character(1),
    silent = TRUE, stars = TRUE
  )
  vec <- str_replace_all(vec, "[\\s]+", "")
  vec <- str_replace_all(vec, "\\*\\*", "^")

  # return
  vec
}
# mpolyList_to_m2_str( mp(c("x^3","x + y z", "1")) )
# mpolyList_to_m2_str( c(1, 2, 3) )






`%notin%` <- function(x, y) { !(x %in% y) }
# 2 %notin% 1:5
# 2 %notin% (1:5)[-2]





`%:%` <- function(x, y) {
  # do type checking
  stopifnot(
    is.character(x), length(x) == 1,
    is.character(y), length(y) == 1
  )

  # check letters
  if(all(c(x,y) %in% letters)) { # lower case letters given
    x_ndx <- which(x == letters)
    y_ndx <- which(y == letters)
    letters[x_ndx:y_ndx]
  } else if (all(c(x,y) %in% LETTERS)) { # upper case letters given
    x_ndx <- which(x == LETTERS)
    y_ndx <- which(y == LETTERS)
    LETTERS[x_ndx:y_ndx]
  } else {
    stop("letters must be of the same case")
  }

}


# copy pryr::dots so that we don't have to import pryr
dots <- function(...) {
  eval(substitute(alist(...)))
}

#' Macaulay2 in R
#'
#' m2r provides a persistent interface to Macaulay2
#' (\url{http://www.math.uiuc.edu/Macaulay2/}) and front-end tools facilitating
#' its use in the R ecosystem. For details, see vignette("m2r").
#'
#' @docType package
#' @import mpoly gmp
#' @importFrom memoise memoise forget
#' @importFrom stringr str_replace_all str_sub str_split fixed str_sub
#'   str_detect str_replace str_replace_all str_c
#' @useDynLib m2r
#' @importFrom Rcpp sourceCpp
#' @name m2r
#' @aliases m2r package-m2r
#' @references D. Kahle, C. O'Neill, and J. Sommars (2020). "A Computer Algebra
#' System for R: Macaulay2 and the m2r Package." Journal of Statistical
#' Software, 93(9):1-31.
NULL

#' Create a new ideal in Macaulay2
#'
#' Create a new ideal in Macaulay2
#'
#' @param x a listing of polynomials. several formats are accepted, see
#'   examples.
#' @param ring the referent ring in Macaulay2
#' @param raw_chars if \code{TRUE}, the character vector will not be parsed by
#'   [mp()], saving time (default: \code{FALSE}). the down-side is that the
#'   strings must be formated for M2 use directly, as opposed to for [mp()].
#'   (e.g. \code{"x*y+3"} instead of \code{"x y + 3"})
#' @param code return only the M2 code? (default: \code{FALSE})
#' @param ideal an ideal object of class \code{m2_ideal} or
#'   \code{m2_ideal_pointer}
#' @param e1,e2 ideals for arithmetic
#' @param I,J ideals or objects parsable into ideals
#' @param ... ...
#' @return a reference to a Macaulay2 ideal
#' @name ideal
#' @examples
#'
#' \dontrun{ requires Macaulay2
#'
#'
#' ##### basic usage
#' ########################################
#'
#' ring("x", "y", coefring = "QQ")
#' ideal("x + y", "x^2 + y^2")
#'
#'
#'
#' ##### different versions of gb
#' ########################################
#'
#' # standard evaluation version
#' poly_chars <- c("x + y", "x^2 + y^2")
#' ideal_(poly_chars)
#'
#' # reference nonstandard evaluation version
#' ideal.("x + y", "x^2 + y^2")
#'
#' # reference standard evaluation version
#' ideal_.(poly_chars)
#'
#'
#'
#' ##### different inputs to gb
#' ########################################
#'
#' ideal_(   c("x + y", "x^2 + y^2") )
#' ideal_(mp(c("x + y", "x^2 + y^2")))
#' ideal_(list("x + y", "x^2 + y^2") )
#'
#'
#'
#' ##### predicate functions
#' ########################################
#'
#' I  <- ideal ("x + y", "x^2 + y^2")
#' I. <- ideal.("x + y", "x^2 + y^2")
#' is.m2_ideal(I)
#' is.m2_ideal(I.)
#' is.m2_ideal_pointer(I)
#' is.m2_ideal_pointer(I.)
#'
#'
#'
#' ##### ideal radical
#' ########################################
#'
#' I <- ideal("(x^2 + 1)^2 y", "y + 1")
#' radical(I)
#' radical.(I)
#'
#'
#'
#' ##### ideal dimension
#' ########################################
#'
#' I <- ideal_(c("(x^2 + 1)^2 y", "y + 1"))
#' dimension(I)
#'
#' # dimension of a line
#' ring("x", "y", coefring = "QQ")
#' I <- ideal("y - (x+1)")
#' dimension(I)
#'
#' # dimension of a plane
#' ring("x", "y", "z", coefring = "QQ")
#' I <- ideal("z - (x+y+1)")
#' dimension(I)
#'
#'
#'
#' ##### ideal quotients and saturation
#' ########################################
#'
#' ring("x", "y", "z", coefring = "QQ")
#' (I <- ideal("x^2", "y^4", "z + 1"))
#' (J <- ideal("x^6"))
#'
#' quotient(I, J)
#' quotient.(I, J)
#'
#' saturate(I)
#' saturate.(I)
#' saturate(I, J)
#' saturate(I, mp("x"))
#' saturate(I, "x")
#'
#'
#' ring("x", "y", coefring = "QQ")
#' saturate(ideal("x y"), "x^2")
#'
#' # saturation removes parts of varieties
#' # solution over R is x = -1, 0, 1
#' ring("x", coefring = "QQ")
#' I <- ideal("(x-1) x (x+1)")
#' saturate(I, "x") # remove x = 0 from solution
#' ideal("(x-1) (x+1)")
#'
#'
#'
#' ##### primary decomposition
#' ########################################
#'
#' ring("x", "y", "z", coefring = "QQ")
#' I <- ideal("(x^2 + 1) (x^2 + 2)", "y + 1")
#' primary_decomposition(I)
#' primary_decomposition.(I)
#'
#' I <- ideal("x (x + 1)", "y")
#' primary_decomposition(I)
#'
#' # variety = z axis union x-y plane
#' (I <- ideal("x z", "y z"))
#' dimension(I) # =  max dimension of irreducible components
#' (Is <- primary_decomposition(I))
#' dimension(Is)
#'
#'
#'
#' ##### ideal arithmetic
#' ########################################
#'
#' ring("x", "y", "z", coefring = "RR")
#'
#' # sums (cox et al., 184)
#' (I <- ideal("x^2 + y"))
#' (J <- ideal("z"))
#' I + J
#'
#' # products (cox et al., 185)
#' (I <- ideal("x", "y"))
#' (J <- ideal("z"))
#' I * J
#'
#' # equality
#' (I <- ideal("x", "y"))
#' (J <- ideal("z"))
#' I == J
#' I == I
#'
#' # powers
#' (I <- ideal("x", "y"))
#' I^3
#'
#' }










#' @rdname ideal
#' @export
ideal <- function(..., raw_chars = FALSE, code = FALSE) {

  # grab args
  x <- list(x = lapply(dots(...), eval, envir = parent.frame()))
  otherArgs <- as.list(match.call(expand.dots = FALSE))[-c(1:2)]

  # eval
  args <- lapply(c(x, otherArgs), eval)

  # run standard evaluation gb
  do.call("ideal_", args)

}






#' @rdname ideal
#' @export
ideal. <- function(..., raw_chars = FALSE, code = FALSE) {

  # grab args
  x <- list(x = lapply(dots(...), eval, envir = parent.frame()))
  otherArgs <- as.list(match.call(expand.dots = FALSE))[-c(1:2)]

  # eval
  args <- lapply(c(x, otherArgs), eval)

  # run standard evaluation gb
  do.call("ideal_.", args)

}









#' @rdname ideal
#' @export
ideal_ <- function(x, raw_chars = FALSE, code = FALSE, ...) {

  # run ideal.
  args <- as.list(match.call())[-1]
  eargs <- lapply(args, eval, envir = parent.frame())
  pointer <- do.call(ideal_., eargs)
  if(code) return(invisible(pointer))

  # parse output
  parsed_out <- m2_parse(pointer)

  # construct R-side ideal, class and return
  m2_name(parsed_out) <- m2_name(pointer)
  parsed_out

}




#' @rdname ideal
#' @export
ideal_. <- function(x, raw_chars = FALSE, code = FALSE, ...) {

  # make ideal name
  ideal_name <- name_and_increment("ideal", "m2_ideal_count")

  # make ideal_param
  if (raw_chars) {
    ideal_param <- listify(x)
  } else {
    if (is.character(x)) {
      mpolys <- mpolyList_to_m2_str(mp(x))
      ideal_param <- listify(mpolys)
    } else if (is.list(x) && all(vapply(x, is.character, logical(1)))) {
      mpolys <- mpolyList_to_m2_str(mp(unlist(x)))
      ideal_param <- listify(mpolys)
    } else if (is.list(x) && all(vapply(x, is.mpoly, logical(1)))) {
      mpolys <- structure(x, class = "mpolyList")
      mpoly_strings_for_m2 <- mpolyList_to_m2_str(mpolys)
      ideal_param <- listify(mpoly_strings_for_m2)
    } else if (is.list(x) && all(vapply(x, is.numeric, logical(1)))) {
      # this is like c(mp("x y"), mp("x z"), mp("x"))
      stop(
        "you appear to have used c() on mpolys.\n",
        "  this input format is not accepted, use list() instead.",
        call. = FALSE
      )
    } else if (is.mpolyList(x)) {
      mpoly_strings_for_m2 <- mpolyList_to_m2_str(x)
      ideal_param <- listify(mpoly_strings_for_m2)
    } else {
      stop("unrecognized input x. see ?ideal", call. = FALSE)
    }
  }

  # construct m2_code and message
  m2_code <- sprintf("%s = ideal(%s)", ideal_name, ideal_param)
  if(code) { message(m2_code); return(invisible(m2_code)) }

  # run m2
  out <- m2.(m2_code)

  # change name and return
  m2_name(out) <- ideal_name
  out

}




m2_parse_function.m2_ideal <- function(x) {
  m2_structure(
    m2_name = "",
    m2_class = "m2_ideal",
    m2_meta = list(
      ring = m2_meta(x[[1]], "ring"),
      gens = structure(x[[1]][1,], class = "mpolyList")
    )
  )
}


m2_parse_function.m2_monomialideal <- function(x) {
  m2_structure(
    m2_name = "",
    m2_class = "m2_ideal",
    m2_meta = list(
      ring = m2_meta(x[[1]], "ring"),
      gens = structure(x[[1]][1,], class = "mpolyList")
    )
  )
}

# m2_parse_function.m2_ideal_list <- function(x) {
#   m2_structure(
#     m2_name = "",
#     m2_class = "m2_ideal_list",
#     m2_meta = list(
#       ring = m2_meta(x[[1]], "ring"),
#       gens = structure(x[[1]][1,], class = "mpolyList")
#     )
#   )
# }



#' @rdname ideal
#' @export
print.m2_ideal <- function(x, ...) {

  # from print.m2_polynomialring
  s <- sprintf(
    "ring %s[%s] (%s)",
    m2_meta(m2_meta(x, "ring"), "coefring"),
    paste(m2_meta(m2_meta(x, "ring"), "vars"), collapse = ","),
    m2_meta(m2_meta(x, "ring"), "order")
  )

  # ideal stuff
  if (length(m2_meta(x, "gens")) > 1) {
    with_gen <- "with generators :"
  } else {
    with_gen <- "with generator :"
  }
  cat("M2 Ideal of", s, with_gen, "\n")
  gens_strings <- print(m2_meta(x, "gens"), silent = TRUE)
  cat(paste("<", paste(gens_strings, collapse = ",  "), ">\n"))
  # cat(str_pad(gens_strings, nchar(gens_strings)+2, side = "left"), sep = "\n")
  invisible(x)

}






#' @rdname ideal
#' @export
print.m2_ideal_list <- function(x, ...) {

  # from print.m2_polynomialring
  s <- sprintf(
    "%s[%s] (%s)",
    m2_meta(m2_meta(x, "ring"), "coefring"),
    paste(m2_meta(m2_meta(x, "ring"), "vars"), collapse = ","),
    m2_meta(m2_meta(x, "ring"), "order")
  )

  cat("M2 List of ideals of", s, ":", "\n")
  lapply(x, function(ideal){
    gens_strings <- print(m2_meta(ideal, "gens"), silent = TRUE)
    cat(paste("<", paste(gens_strings, collapse = ",  "), ">\n"))
  })

  invisible(x)

}





#' @rdname ideal
#' @export
radical <- function(ideal, ring, code = FALSE, ...) {

  # run radical.
  args <- as.list(match.call())[-1]
  eargs <- lapply(args, eval, envir = parent.frame())
  pointer <- do.call(radical., eargs)
  if(code) return(invisible(pointer))

  # parse output
  parsed_out <- m2_parse(pointer)

  # construct R-side ideal, class and return
  m2_name(parsed_out) <- m2_name(pointer)
  m2_meta(parsed_out, "radical_of") <- ideal
  parsed_out

}



#' @rdname ideal
#' @export
radical. <- function(ideal, ring, code = FALSE, ...) {

  # arg check
  if (!is.m2_ideal(ideal) && !is.m2_ideal_pointer(ideal))
    stop("unrecognized input ideal. see ?ideal", call. = FALSE)

  # make radical ideal name
  radical_name <- name_and_increment("ideal", "m2_ideal_count")

  # check ring to be QQ or ZZ/p

  # construct code and message
  m2_code <- sprintf("%s = radical(%s)", radical_name, m2_name(ideal))
  if(!missing(ring)) m2_code <- paste(sprintf("use %s;", ring), m2_code)
  if(code) { message(m2_code); return(invisible(m2_code)) }

  # run m2
  out <- m2.(m2_code)

  # change name and return
  m2_name(out) <- radical_name
  out
}



#' @rdname ideal
#' @export
saturate <- function(I, J, code = FALSE, ...) {

  # run saturate.
  args <- as.list(match.call())[-1]
  eargs <- lapply(args, eval, envir = parent.frame())
  pointer <- do.call(saturate., eargs)
  if(code) return(invisible(pointer))

  # parse output
  parsed_out <- m2_parse(pointer)

  # construct R-side ideal, class and return
  m2_name(parsed_out) <- m2_name(pointer)
  m2_meta(parsed_out, "saturation_of") <- I
  parsed_out

}



#' @rdname ideal
#' @export
saturate. <- function(I, J, code = FALSE, ...) {

  # arg check
  if (!is.m2_ideal(I) && !is.m2_ideal_pointer(I))
    stop("unrecognized input ideal. see ?ideal", call. = FALSE)

  if (!missing(J)) {
    if (is.m2_ideal(J) || is.m2_ideal_pointer(J)) {
      second_param <- paste0(",", m2_name(J))
    } else if (is.mpoly(J)) {
      second_param <- paste0(",", mpolyList_to_m2_str(J))
    } else if (is.character(J)) {
      second_param <- paste0(",", mpolyList_to_m2_str(mp(J)))
    } else {
      stop("unrecognized input J. see ?ideal", call. = FALSE)
    }
  } else {
    second_param <- ""
  }

  # make saturation ideal name
  saturate_name <- name_and_increment("ideal", "m2_ideal_count")

  # construct code and message
  m2_code <- sprintf("%s = saturate(%s%s)", saturate_name, m2_name(I), second_param)
  if(code) { message(m2_code); return(invisible(m2_code)) }

  # run m2
  out <- m2.(m2_code)

  # change name and return
  m2_name(out) <- saturate_name
  out
}








#' @rdname ideal
#' @export
quotient <- function(I, J, code = FALSE, ...) {

  # run quotient.
  args <- as.list(match.call())[-1]
  eargs <- lapply(args, eval, envir = parent.frame())
  pointer <- do.call(quotient., eargs)
  if(code) return(invisible(pointer))

  # parse output
  parsed_out <- m2_parse(pointer)

  # construct R-side ideal, class and return
  m2_name(parsed_out) <- m2_name(pointer)
  m2_meta(parsed_out, "quotient_of") <- I
  parsed_out

}



#' @rdname ideal
#' @export
quotient. <- function(I, J, code = FALSE, ...) {

  # arg check
  if (!is.m2_ideal(I) && !is.m2_ideal_pointer(I))
    stop("unrecognized input ideal. see ?ideal", call. = FALSE)

  if (!missing(J)) {
    if (is.m2_ideal(J) || is.m2_ideal_pointer(J)) {
      second_param <- paste0(",", m2_name(J))
    } else if (is.mpoly(J)) {
      second_param <- paste0(",", mpolyList_to_m2_str(J))
    } else if (is.character(J)) {
      second_param <- paste0(",", mpolyList_to_m2_str(mp(J)))
    } else {
      stop("unrecognized input J. see ?ideal", call. = FALSE)
    }
  } else {
    second_param <- ""
  }

  # make saturation ideal name
  quotient_name <- name_and_increment("ideal", "m2_ideal_count")

  # construct code and message
  m2_code <- sprintf("%s = quotient(%s%s)", quotient_name, m2_name(I), second_param)
  if(code) { message(m2_code); return(invisible(m2_code)) }

  # run m2
  out <- m2.(m2_code)

  # change name and return
  m2_name(out) <- quotient_name
  out
}












#' @rdname ideal
#' @export
primary_decomposition <- function(ideal, code = FALSE, ...) {

  # run primary_decomposition.
  args <- as.list(match.call())[-1]
  eargs <- lapply(args, eval, envir = parent.frame())
  pointer <- do.call(primary_decomposition., eargs)
  if(code) return(invisible(pointer))

  # parse output
  parsed_out <- m2_parse(pointer)

  # construct m2 ideal list, class and return
  m2_name(parsed_out) <- m2_name(pointer)
  attr(parsed_out, "m2_meta") <- list(ring = m2_meta(ideal, "ring"))
  # m2_meta(parsed_out, "ring") <-  m2_meta(ideal, "ring")
  class(parsed_out) <- c("m2_ideal_list", class(parsed_out))
  parsed_out

}



#' @rdname ideal
#' @export
primary_decomposition. <- function(ideal, code = FALSE, ...) {

  # arg check
  if (!is.m2_ideal(ideal) && !is.m2_ideal_pointer(ideal))
    stop("unrecognized input ideal. see ?ideal", call. = FALSE)

  # make resulting structure name
  ideal_list_name <- name_and_increment("ideallist", "m2_ideal_list_count")

  # construct code and message
  m2_code <- sprintf("%s = primaryDecomposition(%s)", ideal_list_name, m2_name(ideal))
  if(code) { message(m2_code); return(invisible(m2_code)) }

  # run m2
  out <- m2.(m2_code)
  m2_name(out) <- ideal_list_name
  class(out) <- c("m2_ideal_list_pointer", class(out))

  # return
  out
}







#' @rdname ideal
#' @export
dimension <- function(ideal, code = FALSE, ...) {

  # arg check
  if (!(
    is.m2_ideal(ideal) || is.m2_ideal_pointer(ideal) ||
    is.m2_ideal_list(ideal) || is.m2_ideal_list_pointer(ideal)
  )) stop("unrecognized input ideal. see ?ideal", call. = FALSE)

  # construct code and message
  if(is.m2_ideal(ideal) || is.m2_ideal_pointer(ideal)) {
    m2_code <- sprintf("dim(%s)", m2_name(ideal))
  } else { # an ideal list or ideal list pointer
    m2_code <- sprintf("apply(%s, dim)", m2_name(ideal))
  }
  if(code) { message(m2_code); return(invisible(m2_code)) }

  # run m2
  out <- m2.(m2_code)

  # parse and return
  m2_parse(out)

}












#' @rdname ideal
#' @export
`+.m2_ideal` <- function(e1, e2) {

  # arg check
  if (!(
    (is.m2_ideal(e1) || is.m2_ideal_pointer(e1)) &&
    (is.m2_ideal(e2) || is.m2_ideal_pointer(e2))
  )) stop("unrecognized input ideal. see ?ideal", call. = FALSE)

  # construct code and message
  m2_code <- sprintf("%s + %s", m2_name(e1), m2_name(e2))

  # run m2
  out <- m2.(m2_code)

  # parse and return
  m2_parse(out)

}





#' @rdname ideal
#' @export
`*.m2_ideal` <- function(e1, e2) {

  # arg check
  if (!(
    (is.m2_ideal(e1) || is.m2_ideal_pointer(e1)) &&
    (is.m2_ideal(e2) || is.m2_ideal_pointer(e2))
  )) stop("unrecognized input ideal. see ?ideal", call. = FALSE)

  # construct code and message
  m2_code <- sprintf("%s * %s", m2_name(e1), m2_name(e2))

  # run m2
  out <- m2.(m2_code)

  # parse and return
  m2_parse(out)

}




#' @rdname ideal
#' @export
`==.m2_ideal` <- function(e1, e2) {

  # arg check
  if (!(
    (is.m2_ideal(e1) || is.m2_ideal_pointer(e1)) &&
    (is.m2_ideal(e2) || is.m2_ideal_pointer(e2))
  )) stop("unrecognized input ideal. see ?ideal", call. = FALSE)

  # construct code and message
  m2_code <- sprintf("%s == %s", m2_name(e1), m2_name(e2))

  # run m2
  out <- m2.(m2_code)

  # parse and return
  m2_parse(out)

}




#' @rdname ideal
#' @export
`^.m2_ideal` <- function(e1, e2) {

  # arg check
  if (!(
    (is.m2_ideal(e1) || is.m2_ideal_pointer(e1)) && is.numeric(e2)
  )) stop("unrecognized input ideal. see ?ideal", call. = FALSE)

  # construct code and message
  m2_code <- sprintf("%s^%s", m2_name(e1), e2)

  # run m2
  out <- m2.(m2_code)

  # parse and return
  m2_parse(out)

}

.onAttach <- function(...) {

  packageStartupMessage('  Please cite m2r! See citation("m2r") for details.')

  # set gmp
  set_m2r_option(gmp = FALSE)

  # set pathing
  if (is.mac() || is.linux()) {
    set_m2r_option(m2_path = Sys.getenv("M2"))
  } else {
    set_m2r_option(m2_path = "")
  }


  # check that M2 was found in ~/.Renviron
  startup_check_for_program()


  # return
  invisible(TRUE)
}




.onDetach <- function(...) {
  stop_m2()
  options(m2r = NULL)
}
# restart R
# library(m2r)
# m2("1+1")
# getOption("m2r")
# detach("package:m2r")
# getOption("m2r")





psm  <- packageStartupMessage

psms <- function(fmt, ...) packageStartupMessage(sprintf(fmt, ...))


startup_check_for_program <- function(){

  if(get_m2_path() != ""){

    psms("  M2 found in %s", get_m2_path())
    return(invisible(FALSE))

  } else {

    psms("  M2 not found; defaulting to cloud.")
    psms("  Use set_m2_path(\"/path/to/m2\") to run M2 locally.")
    return(invisible(FALSE))

  }

  invisible(TRUE)
}



setOption <- function(optionName, value){
  eval(parse(text = sprintf('options("%s" = "%s")', optionName, value)))
}






# set_m2r_option both sets options for m2r in the list m2r in options
# and initialized the list when m2r is attached to the search path
# (search())
set_m2r_option <- function(...) {

  # if there is no m2r option (package is being initialized)
  # create the list with the arguments and return
  if ("m2r" %notin% names(options())) {
    options(m2r = list(...))
    return(invisible())
  }

  # otherwise, go through arguments sequentially and add/update
  # them in the list m2r in options
  m2r <- getOption("m2r")
  arg_list <- lapply(as.list(match.call())[-1], eval, envir = parent.frame())
  for (k in seq_along(arg_list)) {
    if (names(arg_list)[k] %in% names(m2r)) {
      m2r[names(arg_list)[k]] <- arg_list[k]
    } else {
      m2r <- c(m2r, arg_list[k])
    }
  }

  # set new m2r
  options(m2r = m2r)

  # return
  invisible()
}
# (l <- list(a = 1, b = 2, c = 3))
# l[d] <- 5
# l

#' Compute a Grobner basis with Macaulay2
#'
#' Compute a Grobner basis with Macaulay2
#'
#' \code{gb} uses nonstandard evaluation; \code{gb_} is the standard evaluation
#' equivalent.
#'
#' @param x a character vector of polynomials to be parsed by [mp()], a
#'   \code{mpolyList} object, an [ideal()] or pointer to an ideal
#' @param control a list of options, see examples
#' @param raw_chars if \code{TRUE}, the character vector will not be parsed by
#'   [mp()], saving time (default: \code{FALSE}). the down-side is that the
#'   strings must be formated for M2 use directly, as opposed to for [mp()].
#'   (e.g. \code{"x*y+3"} instead of \code{"x y + 3"})
#' @param code return only the M2 code? (default: \code{FALSE})
#' @param ... ...
#' @return an \code{mpolyList} object of class \code{m2_grobner_basis} or a
#'   \code{m2_grobner_basis_pointer} pointing to the same. See [mpolyList()].
#' @seealso [mp()], [use_ring()]
#' @name gb
#' @examples
#'
#' \dontrun{ requires Macaulay2
#'
#'
#' ##### basic usage
#' ########################################
#'
#' # the last ring evaluated is the one used in the computation
#' ring("t","x","y","z", coefring = "QQ")
#' gb("t^4 - x", "t^3 - y", "t^2 - z")
#'
#' # here's the code it's running in M2
#' gb("t^4 - x", "t^3 - y", "t^2 - z", code = TRUE)
#'
#'
#'
#' ##### different versions of gb
#' ########################################
#'
#' # standard evaluation version
#' poly_chars <- c("t^4 - x", "t^3 - y", "t^2 - z")
#' gb_(poly_chars)
#'
#' # reference nonstandard evaluation version
#' gb.("t^4 - x", "t^3 - y", "t^2 - z")
#'
#' # reference standard evaluation version
#' gb_.(poly_chars)
#'
#'
#'
#' ##### different inputs to gb
#' ########################################
#'
#' # ideals can be passed to gb
#' I <- ideal("t^4 - x", "t^3 - y", "t^2 - z")
#' gb_(I)
#'
#' # note that gb() works here, too, since there is only one input
#' gb(I)
#'
#' # ideal pointers can be passed to gb
#' I. <- ideal.("t^4 - x", "t^3 - y", "t^2 - z")
#' gb_(I.)
#'
#' # setting raw_chars is a bit faster, because it doesn't use ideal()
#' gb("t^4 - x", "t^3 - y", "t^2 - z", raw_chars = TRUE, code = TRUE)
#' gb("t^4 - x", "t^3 - y", "t^2 - z", raw_chars = TRUE)
#'
#'
#'
#' ##### more advanced usage
#' ########################################
#'
#' # the control argument accepts a named list with additional
#' # options
#' gb_(
#'   c("t^4 - x", "t^3 - y", "t^2 - z"),
#'   control = list(StopWithMinimalGenerators = TRUE),
#'   code = TRUE
#' )
#'
#' gb_(
#'   c("t^4 - x", "t^3 - y", "t^2 - z"),
#'   control = list(StopWithMinimalGenerators = TRUE)
#' )
#'
#'
#'
#' ##### potential issues
#' ########################################
#'
#' # when specifying raw_chars, be sure to add asterisks
#' # between variables to create monomials; that's the M2 way
#' ring("x", "y", "z", coefring = "QQ")
#' gb("x y", "x z", "x", raw_chars = TRUE, code = TRUE) # errors without code = TRUE
#' gb("x*y", "x*z", "x", raw_chars = TRUE, code = TRUE) # correct way
#' gb("x*y", "x*z", "x", raw_chars = TRUE)
#'
#'
#'
#'
#'
#'
#'
#'
#'
#' }




#' @export
#' @rdname gb
gb <- function(..., control = list(), raw_chars = FALSE, code = FALSE) {

  # grab args
  x <- list(x = lapply(dots(...), eval, envir = parent.frame()))
  if(is.list(x) && (length(x) == 1) && is.m2_ideal(x[[c(1,1)]])) x <- x[[1]]
  if(is.list(x) && (length(x) == 1) && is.m2_ideal_pointer(x[[c(1,1)]])) x <- x[[1]]
  otherArgs <- as.list(match.call(expand.dots = FALSE))[-c(1:2)]

  # eval
  args <- lapply(c(x, otherArgs), eval)

  # run standard evaluation gb
  do.call("gb_", args)

}






#' @export
#' @rdname gb
gb. <- function(..., control = list(), raw_chars = FALSE, code = FALSE) {

  # grab args
  x <- list(x = lapply(dots(...), eval, envir = parent.frame()))
  if(is.list(x) && (length(x) == 1) && is.m2_ideal(x[[c(1,1)]])) x <- x[[1]]
  if(is.list(x) && (length(x) == 1) && is.m2_ideal_pointer(x[[c(1,1)]])) x <- x[[1]]
  otherArgs <- as.list(match.call(expand.dots = FALSE))[-c(1:2)]

  # eval
  args <- lapply(c(x, otherArgs), eval)

  # run standard evaluation gb
  do.call("gb_.", args)

}







# value version of f (standard user version)
#' @rdname gb
#' @export
gb_ <- function(x, control = list(), raw_chars = FALSE, code = FALSE, ...) {

  # run m2
  args <- as.list(match.call())[-1]
  eargs <- lapply(args, eval, envir = parent.frame())
  pointer <- do.call(gb_., eargs)
  if(code) return(invisible(pointer))

  # parse output
  parsed_out <- m2_parse(pointer)

  # more parsing
  out <- m2_structure(
    parsed_out[1,],
    m2_name = m2_name(pointer),
    m2_class = "m2_grobner_basis",
    m2_meta = list(ideal = m2_meta(pointer, "ideal")),
    base_class = "mpolyList"
  )

  # return
  out

}








# reference version of f (returns pointer to m2 object)
#' @rdname gb
#' @export
gb_. <- function(x, control = list(), raw_chars = FALSE, code = FALSE, ...) {

  # make gb name
  gb_name <- name_and_increment("gb", "m2_gb_count")

  # create m2_code params
  if (raw_chars) {

    # x <- vapply(x, str_replace_all, character(1), "[ ]+", "*")
    ideal_name <- paste0("ideal(", paste0(x, collapse = ", "), ")")

  } else {

    if (is.m2_ideal(x)) {
      ideal_name <- m2_name(x)
    } else if (is.m2_ideal_pointer(x)) {
      ideal_name <- m2_name(x)
    } else {
      x <- do.call(ideal_., list(x = x))
      ideal_name <- m2_name(x)
    }

  }

  # prepare control string
  if (length(control) > 0) {
    control_string <- paste0(", ",
      paste(names(control), r_to_m2(unlist(control)), sep = " => ", collapse = ", ")
    )
  } else {
    control_string <- ""
  }

  # construct m2_code and message
  m2_code <- sprintf("%1$s = gb(%2$s%3$s); gens %1$s", gb_name, ideal_name, control_string)
  if(code) { message(m2_code); return(invisible(m2_code)) }

  # run m2
  out <- m2.(m2_code)

  # change name and return pointer
  m2_name(out) <- gb_name
  if(!raw_chars) m2_meta(out) <- c(m2_meta(out), list(ideal = ideal_name))
  out

}












r_to_m2 <- function(x) {
  if (length(x) > 1) return(vapply(x, r_to_m2, character(1)))
  if(is.logical(x)) return(tolower(as.character(x)))
  if(is.numeric(x)) return(x)
  if(is.character(x)) return(x)
  stop("unexpected input.")
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

m2_tokenize_cpp <- function(s) {
    .Call('_m2r_m2_tokenize_cpp', PACKAGE = 'm2r', s)
}

#' Smith normal form
#'
#' For an integer matrix M, this computes the matrices D, P, and Q such that
#' \emph{D = PMQ}, which can be seen as an analogue of the singular value
#' decomposition. All are integer matrices, and P and Q are unimodular (have
#' determinants +- 1).
#'
#' @param mat a matrix (integer entries)
#' @param code return only the M2 code? (default: \code{FALSE})
#' @name snf
#' @seealso [m2_matrix()]
#' @return a list of \code{m2_matrix} objects with names \code{D}, \code{P}, and
#'   \code{Q}
#' @examples
#'
#' \dontrun{ requires Macaulay2
#'
#' ##### basic usage
#' ########################################
#'
#' M <- matrix(c(
#'    2,  4,   4,
#'   -6,  6,  12,
#'   10, -4, -16
#' ), nrow = 3, byrow = TRUE)
#'
#' snf(M)
#'
#' (mats <- snf(M))
#' P <- mats$P; D <- mats$D; Q <- mats$Q
#'
#' P %*% M %*% Q                # = D
#' solve(P) %*% D %*% solve(Q)  # = M
#'
#' det(P)
#' det(Q)
#'
#'
#' M <- matrix(c(
#'      1,    2,    3,
#'      1,   34,   45,
#'   2213, 1123, 6543,
#'      0,    0,    0
#' ), nrow = 4, byrow = TRUE)
#' (mats <- snf(M))
#' P <- mats$P; D <- mats$D; Q <- mats$Q
#' P %*% M %*% Q                # = D
#'
#'
#'
#' ##### understanding lattices
#' ########################################
#'
#'
#'
#'
#' # cols of m generate the lattice L
#' M <- matrix(c(2,-1,1,3), nrow = 2)
#' row.names(M) <- c("x", "y")
#' M
#'
#' # plot lattice
#' df <- expand.grid(x = -20:20, y = -20:20)
#' pts <- t(apply(df, 1, function(v) M %*% v))
#' w <- c(-15, 15)
#' plot(pts, xlim = w, ylim = w)
#'
#' # decompose m
#' (mats <- snf(M))
#' P <- mats$P; D <- mats$D; Q <- mats$Q
#'
#' # PMQ = D, the columns of MQ = P^(-1) D  are a simpler basis of
#' # the lattice generated by (the cols of) M
#' (basis <- solve(P) %*% D)
#'
#' # plot lattice generated by new basis
#' pts2 <- t(apply(df, 1, function(v) basis %*% v))
#' points(pts2, pch = "*", col = "red")
#'
#'
#'
#' ##### other options
#' ########################################
#'
#' snf.(M)
#' snf(M, code = TRUE)
#'
#'
#'
#'
#' }
#'






#' @rdname snf
#' @export
snf <- function (mat, code = FALSE) {

  # run m2
  args <- as.list(match.call())[-1]
  eargs <- lapply(args, eval, envir = parent.frame())
  pointer <- do.call(snf., eargs)
  if(code) return(invisible(pointer))

  # parse output
  parsed_out <- m2_parse(pointer)

  # list and out
  list(
    D = parsed_out[[1]],
    P = parsed_out[[2]],
    Q = parsed_out[[3]]
  )
}





#' @rdname snf
#' @export
snf. <- function (mat, code = FALSE) {

  # arg checking
  # if (is.m2_matrix(mat)) mat <- mat$rmatrix
  if (is.m2_pointer(mat)) {
    param <- m2_name(mat)
  } else {
    if (!is.integer(mat)) stopifnot(all(mat == as.integer(mat)))
    param <- paste0("matrix", listify_mat(mat))
  }

  # create code and message
  m2_code <- sprintf("smithNormalForm %s", param)
  if(code) { message(m2_code); return(invisible(m2_code)) }

  # run m2 and return pointer
  m2.(m2_code)
}

#' Factor an integer into primes
#'
#' Factor an integer into primes
#'
#' @param n an integer or a polynomial
#' @param code return only the M2 code? (default: \code{FALSE})
#' @param ... ...
#' @return a data frame with integer columns \code{prime} and
#'   \code{power} or \code{m2_pointer} referencing the factorization
#'   in M2.
#' @name factor_n
#' @examples
#'
#' \dontrun{ requires Macaulay2
#'
#' ##### basic usage
#' ########################################
#'
#' 2^2 * 3^7 * 5^2 # = 218700
#' factor_n(218700)
#' factor_n.(218700)
#'
#' (df <- factor_n(218700))
#' df$prime
#' df$power
#' str(df)
#'
#'
#' factor_n(218700, code = TRUE)
#'
#'
#' ##### other options
#' ########################################
#'
#' (integer_pointer <- m2.("218700"))
#' m2_name(integer_pointer)
#' factor_n(integer_pointer, code = TRUE)
#' factor_n(integer_pointer)
#'
#'
#'
#' factor_n(3234432540)
#' factor_n(323443254223453)
#' factor_n(rpois(1, 1e4))
#'
#'
#' ##### known issues
#' ########################################
#'
#' # R doesn't handle big ints well. note in the following
#' # the m2 code number is different than the supplied number
#' factor_n(32344325422364353453, code = TRUE)
#'
#' # this can be circumvented by passing a string instead
#' factor_n("32344325422364353453", code = TRUE)
#'
#' # but if the factors are large, R can't handle the parsing well
#' factor_n("32344325422364353453")
#'
#' # here's a workaround:
#' factor_pointer <- factor_n.("32344325422364353453")
#' m2_meta(factor_pointer, "ext_str")
#' extract_factors <- function(pointer) {
#'   require(stringr)
#'   str <- m2_meta(pointer, "ext_str")
#'   str <- str_sub(str, 19, -2)
#'   str <- str_extract_all(str, "\\{[0-9]+,[0-9]+\\}")[[1]]
#'   str <- str_sub(str, 2, -2)
#'   str <- str_split(str, ",")
#'   df <- as.data.frame(t(simplify2array(str)))
#'   names(df) <- c("prime", "power")
#'   df
#' }
#' (df <- extract_factors(factor_pointer))
#'
#'
#' # using gmp (currently broken)
#' # factor_n("32344325422364353453", gmp = TRUE)
#' m2("11 * 479 * 6138607975396537")
#' 11 * 479 * 6138607975396537
#'
#' }
#'




#' @rdname factor_n
#' @export
factor_n <- function (n, code = FALSE, ...) {

  # run m2
  args <- as.list(match.call())[-1]
  eargs <- lapply(args, eval, envir = parent.frame())
  pointer <- do.call(factor_n., eargs)
  if (code) return(invisible(pointer))

  # parse output
  parsed_out <- m2_parse(pointer)

  # reformat and return
  list(
    prime = vapply(parsed_out, `[[`, integer(1), 1),
    power = vapply(parsed_out, `[[`, integer(1), 2)
  )

}


#' @rdname factor_n
#' @export
factor_n. <- function (n, code = FALSE, ...) {

  # arg checking
  # compute factor with m2, e.g. "2^2*67*97*9433"
  if (is.m2_pointer(n)) {
    param <- m2_name(n)
  } else if (is.numeric(n)) {
    param <- as.character(format(n, scientific = FALSE)) # for big Z's
  } else {
    param <- n
  }

  # create code and message
  m2_code <- sprintf("factor(%s)", param)
  if(code) { message(m2_code); return(invisible(m2_code)) }

  # run m2 and return pointer
  m2.(m2_code)

}

#' Call and reset a Macaulay2 process
#'
#' Call and reset a Macaulay2 process
#'
#' @param port port for Macaulay2 socket
#' @param timeout number of seconds before aborting
#' @param attempts numer of times to try to make connection
#' @param cloud use a cloud?
#' @param hostname the remote host to connect to; defaults to the Amazon EC2
#'   instance
#' @param code Macaulay2 code
#' @param x formal argument for print method
#' @param ... ...
#' @return m2 return value
#' @name m2_call
#' @examples
#'
#' \dontrun{ requires Macaulay2
#'
#' m2("1 + 1")
#' m2.("1 + 1")
#'
#' m2("factor 32004")
#'
#' # run a chunk of m2 code, only pulling the end value back into R
#' m2("
#'   R = QQ[a..d]
#'   I = ideal(a^3-b^2*c, b*c^2-c*d^2, c^3)
#'   G = gens gb I
#' ")
#'
#' # illustrate the persistent connection
#' m2("a = 1 + 1")
#' m2("a")
#' reset_m2()
#' m2("a")
#'
#'
#' # forcing a cloud start
#' if(has_m2_connection()) stop_m2()
#' start_m2(cloud = TRUE)
#' m2("1 + 1")
#' stop_m2()
#'
#'
#'
#' m2.("peek(QQ[x,y,z])")
#' m2("peek(QQ[x,y,z])")
#'
#' # m2 returns in its ext_str position the result of running
#' # toExternalString on the return value of the chunk of code
#' # you run. in principle, toExternalString provides the code
#' # needed to recreate the m2 object of interest. however,
#' # does not work for all objects representable in the m2 language.
#' # in particular, mutable objects are not supported.
#' # this is what happens when you look at those:
#' m2.("new MutableList from {1,2,3}")
#' m2("new MutableList from {1,2,3}")
#'
#' }


#' @export
#' @rdname m2_call
m2r_version_number <- function() {
  "1.0.0"
}



#' @export
#' @rdname m2_call
m2r_cloud_url <- function() {
  "ec2-52-10-66-241.us-west-2.compute.amazonaws.com"
}


#' @export
#' @rdname m2_call
has_m2_connection <- function() {

  if (is.null(get_m2_connection())) {
    FALSE
  } else {
    TRUE
  }

}



#' @export
#' @rdname m2_call
start_m2 <- function(
  port = 27436L, timeout = 10, attempts = 10,
  cloud = FALSE, hostname = m2r_cloud_url()
) {

  # don't increment port if supplied
  if (!missing(port)) attempts <- 1

  # if already running M2, break
  if (has_m2_connection()) return(invisible(0L))

  if(!is.null(get_m2_path()) && !cloud && missing(hostname)) { # m2 found locally

    if (do_start_m2_local(port = port, timeout = timeout, attempts = attempts) != 0L)
      stop("m2r unable to connect to local instance")

  } else { # default to cloud

    cloud_out <- do_start_m2_cloud(hostname)
    if(cloud_out == 1L) stop("m2r unable to connect to the cloud instance, are you online?")

  }

}



do_start_m2_cloud <- function(hostname = m2r_cloud_url()) {

  # if already running M2, break
  if (has_m2_connection()) return(invisible(0L))

  # launch M2 on cloud
  message("Connecting to M2 in the cloud... ", appendLF = FALSE)

  # request port
  port <- request_port(hostname = hostname, port = 27435L)

  # connect to local server
  out <- connect_to_m2_server(hostname = hostname, port = port)
  message("done.")

  # return
  invisible(out)
}



do_start_m2_local <- function(port = 27436L, timeout = 10, attempts = 10) {

  # if already running M2, break
  if (has_m2_connection()) return(invisible(0L))

  # find the first open port
  openPortFound <- FALSE
  for (i in seq.int(0, attempts-1)) {

    out <- system(paste0("netstat -an | grep '[\\.:]", port, "' | wc -l"), intern = TRUE)
    if (as.integer(out) == 0) break()
    # openPortFound <- TRUE
    # tempservercon <- NULL
    # tryCatch(
    #   tempservercon <- suppressWarnings(
    #     socketConnection(
    #       port = port, blocking = FALSE,
    #       server = TRUE, open = "r+", timeout = 1
    #     )
    #   ),
    #   error = function(e) { openPortFound <- FALSE }
    # )
    # if (!is.null(tempservercon)) close(tempservercon)
    # if (openPortFound) break()

    if (i == attempts - 1) {
      message(sprintf("%s attempts made at finding an open port. Aborting start_m2", attempts))
      return(invisible(1L))
    } else {
      # message(sprintf("Unable to connect to M2 on port %s. Attempting to connect on port %s", port, port + 1))
      port <- port + 1
    }

  }

  # launch M2 on local server
  message("Starting M2... ", appendLF = FALSE)
  if(is.mac() || is.unix()) {
    system2(
      file.path2(get_m2_path(), "M2"), args = c("--script", system.file("server", "m2rserverscript.m2", package = "m2r"), toString(port)),
      stdout = NULL, stderr = NULL, stdin = "",
      wait = FALSE
    )
  } else if(is.win()) {
    stop("Running local instances of M2 is not yet supported.")
  }

  # connect to local server
  out <- connect_to_m2_server(port = port, timeout = timeout)
  message("done.")

  # post process id to m2r global options
  if (out == 0) set_m2r_option(m2_procid = strtoi(m2("processID()")))

  # return
  out
}







request_port <- function(
  hostname = m2r_cloud_url(),
  port = 27435L, timeout = 10
) {

  # initialize client socket
  con <- NULL
  for (i in 0:(20*timeout)) {

    tryCatch(
      con <- suppressWarnings(
        socketConnection(
          host = hostname, port = port,
          blocking = FALSE, server = FALSE,
          open = "r+", timeout = 60*60*24*7
        )
      ),
      error = function(e) {  }
    )

    if (!is.null(con)) { break } else { Sys.sleep(0.05) }

  }

  if (is.null(con)) stop("m2r unable to connect to the cloud instance, are you online?")

  repeat {
    # read output info
    port_number <- readLines(con, 1)

    if (length(port_number) > 0) break

    i <- i + 1
    if (timeout > 0 && i >= timeout * 2000) {
      break
    } else {
      Sys.sleep(0.0005)
    }
  }

  close(con)

  if (length(port_number) == 0 || port_number == "0") {
    stop(sprintf("Macaulay2 cloud is full; please try again later."))
  }

  return(strtoi(port_number))

}



connect_to_m2_server <- function(hostname = "localhost", port = 27436L, timeout = 10) {

  # initialize client socket
  con <- NULL
  for (i in 0:(20*timeout)) {

    tryCatch(
      con <- suppressWarnings(
        socketConnection(
          host = hostname, port = port,
          blocking = FALSE, server = FALSE,
          open = "r+", timeout = 60*60*24*7
        )
      ),
      error = function(e) {  }
    )

    if (!is.null(con)) {
      break
    } else {
      Sys.sleep(0.05)
    }

  }

  if (is.null(con)) return(invisible(1L))

  repeat {
    # read output info
    server_version <- readLines(con, 1)

    if (length(server_version) > 0) break

    i <- i + 1
    if (timeout > 0 && i >= 2000*timeout) {
      break
    } else {
      Sys.sleep(0.0005)
    }
  }

  if (server_version != m2r_version_number()) {
    close(con)
    stop(sprintf("Internal error: server version is %s and client version is %s.",
                 server_version, m2r_version_number()))
  }

  # set options
  set_m2r_option(
    m2_con = con,
    m2_port = port,
    m2_timeout = timeout
  )

  set_m2r_option(m2_procid = -1L)

  invisible(0L)
}





#' @export
#' @rdname m2_call
stop_m2 <- function() {

  if (has_m2_connection()) { # for detaching when m2 never run

    # send kill code
    writeLines("", get_m2_connection())
    close(get_m2_connection())

    # not elegant, but a necessary safety measure
    Sys.sleep(0.01)
    if (get_m2_procid() >= 0) tools::pskill(get_m2_procid())

    set_m2r_option(m2_con = NULL, m2_procid = NULL)

  }

}




#' @export
#' @rdname m2_call
reset_m2 <- function(
  port = 27436L, timeout = 10, attempts = 10,
  hostname = "ec2-52-10-66-241.us-west-2.compute.amazonaws.com"
) {

  stop_m2()
  start_m2(port, timeout, attempts, hostname)

}









#' @export
#' @rdname m2_call
m2 <- function(code, timeout = -1) {
  m2_meta(do.call(m2., as.list(match.call())[-1]), "ext_str")
}





#' @export
#' @rdname m2_call
m2. <- function(code, timeout = -1) {

  # ensure m2 is running
  start_m2()

  # preempt m2 kill code
  if (code == "") return("")

  # write to connection
  writeLines(code, get_m2_connection())

  i <- 0
  outinfo <- NULL
  repeat {
    # read output info
    outinfo <- readLines(get_m2_connection(), 1)

    if (length(outinfo) > 0) break

    i <- i + 1
    if (timeout > 0 && i >= timeout * 2000) {
      break
    } else {
      Sys.sleep(0.0005)
    }
  }

  if (length(outinfo) > 0) {
    # read output from connection and return
    info <- strsplit(outinfo, " ", fixed = TRUE)[[1]]

    retcode <- strtoi(info[1])
    numlines <- strtoi(info[2])
    m2_name <- info[3]
    m2_class <- info[4]
    m2_class_class <- info[5]
  } else {
    # cancel command if needed
    tools::pskill(get_m2_procid(), tools::SIGINT)
    Sys.sleep(0.01)

    retcode <- -1L
    numlines <- -1L
  }

  output <- paste(readLines(get_m2_connection(), numlines), collapse = "\n")

  if (retcode == -1L) {
    # timeout occurred, kill M2 instance and stop
    stop_m2()
    # start_m2(getOption("m2_port"), getOption("m2_timeout"))
    stop("Command timed out, M2 connection lost")
  } else if (retcode == 1L) {
    # user's code string errored, alert them
    stop(output, call. = FALSE)
  } else if (retcode == 2L) {
    # toExternalString failed, make ext_str NULL
    output <- NULL
  }

  # assemble pointer object and return
  m2_structure(
    m2_name = m2_name,
    m2_class = "m2_pointer",
    m2_meta = list(
      ext_str = output,
      m2_class = m2_class,
      m2_class_class = m2_class_class
    )
  )
}






#' @export
#' @rdname m2_call
print.m2_pointer <- function (x, ...) {
  cat("M2 Pointer Object\n")
  if(is.null(m2_meta(x, "ext_str"))) m2_meta(x, "ext_str") <- ""
  w <- min(c(options()$width, 80), na.rm = TRUE) - 19
  if(nchar(m2_meta(x, "ext_str")) > w) {
    ext_str <- str_c(str_sub(m2_meta(x, "ext_str"), 1, w-4), "...")
  } else {
    ext_str <- m2_meta(x, "ext_str")
  }
  cat(sprintf("  ExternalString : %s\n", ext_str))
  # cat(sprintf("          R Name : %s\n", deparse(substitute(x))))
  cat(sprintf("         M2 Name : %s\n", m2_name(x)))
  cat(sprintf("        M2 Class : %s (%s)\n", m2_meta(x, "m2_class"), m2_meta(x, "m2_class_class")))
  invisible(x)
}

#' Set Macaulay2 ring
#'
#' \code{use_ring()} sets the default referent ring on the Macaulay2
#' side using the \code{use} function.
#'
#' @param ring a \code{m2_ring} (see [ring()]),
#'   \code{m2_ring_pointer} (see [ring.()]), or a character
#'   string containing the name of a ring in Macaulay2
#' @name use_ring
#' @examples
#'
#' \dontrun{ requires Macaulay2
#'
#'
#' ##### basic usage
#' ########################################
#'
#' ring("x", coefring = "QQ")
#' factor_poly("x^4 + 1")
#'
#' QQtxyz <- ring("t","x","y","z", coefring = "QQ")
#' gb("t^4 - x", "t^3 - y", "t^2 - z")
#'
#' ring("x", "y", "z", "t", coefring = "QQ")
#' gb("t^4 - x", "t^3 - y", "t^2 - z")
#'
#' use_ring(QQtxyz)
#' gb("t^4 - x", "t^3 - y", "t^2 - z")
#'
#' }




#' @export
#' @rdname use_ring
use_ring <- function(ring) {

  # arg check
  stopifnot(is.m2_polynomialring(ring) || is.m2_polynomialring_pointer(ring) || is.character(ring))

  # construct code
  if (is.m2(ring)) {
    m2_code <- sprintf("use %s", m2_name(ring))
  } else {
    m2_code <- sprintf("use %s", ring)
  }

  # run m2
  out <- m2.(m2_code)

  # return
  invisible(out)

}

#' Give the structure of a Macaulay2 ring
#'
#' Give the structure of a Macaulay2 ring
#'
#' @param object An \code{m2} object
#' @param ... ...
#' @return Invisible the object passed in.
#' @export
#' @examples
#'
#' \dontrun{ requires Macaulay2
#'
#' a <- m2("1")
#'
#' R <- ring(c("x1", "x2", "x3"))
#' str_m2(R)
#' str_m2.default(R)
#'
#' }
#'
str_m2 <- function (object, ...) UseMethod("str_m2")



#' @export
str_m2.default <- function(object, ...) {
  cat("M2 Object\n")
  cat(sprintf("    Type : %s\n", class(object)[1]))
  # cat(sprintf("  R Name : %s\n", deparse(substitute(object))))
  cat(sprintf(" M2 Name : %s\n", m2_name(object)))
}



#' @export
str_m2.m2_polynomialring <- function(object, ...){

  cat("M2 Object\n")
  cat(sprintf("     Type : %s\n", class(object)[1]))
  # cat(sprintf("  R Name : %s\n", deparse(substitute(object))))
  cat(sprintf("  M2 Name : %s\n", m2_name(object)))
  cat(sprintf(" Coefring : %s\n", m2_meta(object, "coefring")))
  cat(sprintf("     Vars : %s\n", paste(m2_meta(object, "vars"), collapse = ", ")))
  cat(sprintf("    Order : %s\n", m2_meta(object, "order")))

  invisible(object)
}

#' Enter a Macaulay2 session
#'
#' Enter a Macaulay2 session
#'
#' @param port port for Macaulay2 socket
#' @param timeout number of seconds before aborting
#' @return \code{TRUE} invisibly
#' @export
#' @examples
#'
#' \dontrun{ requires Macaulay2 be installed and an interactive session
#'
#' enter_m2()
#'
#' # m2 code below
#' 1 + 1
#' a = 1
#' a
#' R = QQ[t,x,y,z]
#' I = ideal(t^4  -  x, t^3  -  y, t^2  -  z)
#' gens gb I
#' exit
#'
#' # back in R, the variable persists using m2()
#' m2("a")
#' m2("I")
#'
#'
#' # we can also define variables in R that persist in m2
#' m2("b = 5")
#'
#' enter_m2()
#' b
#' exit
#'
#' }
#'
enter_m2 <- function (port = 27436L, timeout = 10) {
  if(!interactive()) stop("enter_m2() is only available in interactive sessions.")
  stop("enter_m2() is currently not working, check back soon!")
  suppressMessages( start_m2(port, timeout) )
  message("Entering M2 mode. Type 'exit' to go back into R.")
  repeat {
    i <- readline("i : ")
    if(i == "exit") break
    o <- m2(i)
    cat(paste0("o : ", o))
  }
  invisible(TRUE)
}

#' LLL algorithm
#'
#' Macaulay2's implementation of the LLL algorithm. This implementation is still
#' under development and is currently untested.
#'
#' @param mat a matrix (integer entries)
#' @param control additional arguments to pass to LLL; see examples
#' @param code return only the M2 code? (default: \code{FALSE})
#' @seealso [m2_matrix()]
#' @name LLL
#'
#' @return an object of class \code{m2_matrix}
#' @examples
#'
#' \dontrun{ requires Macaulay2
#'
#' ##### basic usage
#' ########################################
#'
#' # example 1
#' M <- matrix(c(
#'   1, 1, 1, 1,
#'   2, 0, 3, 4,
#'   1, 0, 0, 0,
#'   0, 1, 0, 0,
#'   0, 0, 1, 0,
#'   0, 0, 0, 1
#' ), nrow = 6, byrow = TRUE)
#'
#' LLL(M)
#'
#'
#'
#'
#' # example 2 (wikipedia)
#' M <- matrix(c(
#'   1, -1, 3,
#'   1,  0, 5,
#'   1,  2, 6
#' ), nrow = 3, byrow = TRUE)
#'
#' LLL(M)
#'
#'
#' ##### control
#' ########################################
#'
#' M <- matrix(c(
#'   1, 1, 1, 1,
#'   2, 0, 3, 4,
#'   1, 0, 0, 0,
#'   0, 1, 0, 0,
#'   0, 0, 1, 0,
#'   0, 0, 0, 1
#' ), nrow = 6, byrow = TRUE)
#'
#' LLL(M, code = TRUE)
#' LLL(M, control = list(Strategy = "NTL"), code = TRUE)
#' LLL(M, control = list(Strategy = c("BKZ", "RealFP")), code = TRUE)
#'
#' LLL(M)
#' LLL(M, control = list(Strategy = "NTL"))
#' LLL(M, control = list(Strategy = c("BKZ", "RealFP")))
#' LLL(M, control = list(Strategy = c("BKZ", "RealQP")))
#'
#'
#'
#' # method timings with microbenchmark.  note they are roughly the same
#' # for this example matrix
#' microbenchmark::microbenchmark(
#'   "NTL" = LLL(M, control = list(Strategy = "NTL")),
#'   "BKZ_RealFP" = LLL(M, control = list(Strategy = c("BKZ", "RealFP"))),
#'   "BKZ_RealQP" = LLL(M, control = list(Strategy = c("BKZ", "RealQP"))),
#'   "BKZ_RealRR" = LLL(M, control = list(Strategy = c("BKZ", "RealRR")))
#' )
#'
#'
#'
#' ##### additional examples
#' ########################################
#'
#' LLL.(M)
#' LLL(M, code = TRUE)
#'
#'
#'
#' }
#'






#' @rdname LLL
#' @export
LLL <- function (mat, control = list(), code = FALSE) {

  # run m2
  args <- as.list(match.call())[-1]
  eargs <- lapply(args, eval, envir = parent.frame())
  pointer <- do.call(LLL., eargs)
  if(code) return(invisible(pointer))

  # parse output
  m2_parse(pointer)
}





#' @rdname LLL
#' @export
LLL. <- function (mat, control = list(), code = FALSE) {

  # arg checking
  # if (is.m2_matrix(mat)) mat <- mat$rmatrix
  if (is.m2_pointer(mat)) {
    param <- m2_name(mat)
  } else {
    if (!is.integer(mat)) stopifnot(all(mat == as.integer(mat)))
    param <- paste0("matrix", listify_mat(mat))
  }

  # prepare control string
  if (length(control) > 0) {
    if(length(names(control)) != 1 && names(control) != "Strategy") {
      stop("LLL only accepts control argument \"Strategy\".", call. = FALSE)
    }
    strategy <- listify(control$Strategy)
    if(length(control$Strategy) == 1L) strategy <- str_sub(strategy, 2, -2)
    control_string <- sprintf(", Strategy => %s", strategy)
  } else {
    control_string <- ""
  }

  # create code and message
  m2_code <- sprintf("LLL(%s%s)", param, control_string)
  if(code) { message(m2_code); return(invisible(m2_code)) }

  # run m2 and return pointer
  m2.(m2_code)
}

#' Factor a polynomial
#'
#' Factor a polynomial
#'
#' @param mpoly a character parseable by [mp()], an
#'   \code{mpoly} object, or a pointer to a polynomial in M2
#' @param code return only the M2 code? (default: \code{FALSE})
#' @param ... ...
#' @return a named list with elements \code{factor} (an
#'   \code{mpolyList} object) and \code{power}, an integer
#'   vector
#' @name factor_poly
#' @examples
#'
#' \dontrun{ requires Macaulay2 be installed and an interactive session
#'
#' ##### basic usage
#' ########################################
#'
#' ring("x", "y", coefring = "QQ")
#' factor_poly("x^4 - y^4")
#'
#' # reference function
#' factor_poly.("x^4 - y^4")
#'
#'
#' ##### different inputs
#' ########################################
#'
#' # factor_poly accepts mpoly objects:
#' # remember you must create the ring first!
#' (p <- mp("x^4 - y^4"))
#' factor_poly.(p)
#' factor_poly(p)
#' mp("(x-y) (x+y) (x^2+y^2)")
#'
#'
#'
#' ##### other examples
#' ########################################
#'
#' ring("x","y", "z", coefring = "QQ")
#' (p <- mp("(x^2 - y) (x^2 + y) (x + y)^2 (x - z)^2"))
#' factor_poly.(p)
#' factor_poly(p)
#'
#' (p <- mp("(x-1)^3 (y-1)^3"))
#' factor_poly.(p)
#' factor_poly(p)
#'
#' }



#' @rdname factor_poly
#' @export
factor_poly <- function (mpoly, code = FALSE) {

  # run m2
  args <- as.list(match.call())[-1]
  eargs <- lapply(args, eval, envir = parent.frame())
  pointer <- do.call(factor_poly., eargs)
  if(code) return(invisible(pointer))

  # parse output
  parsed_out <- m2_parse(pointer)

  # reformat and return
  list(
    factor = structure(
      lapply(parsed_out, function(.) mp(.[[1]])),
      class = "mpolyList"
    ),
    power = vapply(parsed_out, `[[`, integer(1), 2)
  )

}





#' @rdname factor_poly
#' @export
factor_poly. <- function (mpoly, code = FALSE, ...) {

  # prepare mpoly param
  if (is.m2_pointer(mpoly)) {
    mpoly_param <- m2_name(mpoly)
  } else if (is.mpoly(mpoly) || is.mpolyList(mpoly)) {
    mpoly_param <- mpolyList_to_m2_str(mpoly)
  } else {
    mpoly_param <- as.character(mpoly)
  }

  # create code
  m2_code <- sprintf("factor(%s)", mpoly_param)

  # message
  if (code) { message(m2_code); return(invisible(m2_code)) }

  # run m2 and return pointer
  m2.(m2_code)

}

context("print methods")










test_that("rings print properly", {

  r <- structure(NA,
    class = c("m2_polynomialring", "m2"),
    m2_name = "m2rintring00000000",
    m2_meta = list(
      vars = list("x", "y"),
      coefring = "CC",
      order = "grevlex"
    )
  )

  # check that print returns the input object
  expect_identical(r, print(r))

  # check that the printed format is correct
  expect_equal(
    capture.output(print(r)),
    "M2 Ring: CC[x,y], grevlex order"
  )

})

context("%:%")

test_that("Lower case forward sequences work", {
  expect_equal("b" %:% "t", letters[2:20])
})

test_that("Upper case forward sequences work", {
  expect_equal("B" %:% "T", LETTERS[2:20])
})

test_that("Lower case reverse sequences work", {
  expect_equal("t" %:% "b", letters[20:2])
})

test_that("Upper case reverse sequences work", {
  expect_equal("T" %:% "B", LETTERS[20:2])
})

test_that("Mixed string sequencing fails", {
  expect_error("T" %:% "b", "letters must be of the same case")
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_paths.R
\name{m2_path}
\alias{m2_path}
\alias{set_m2_path}
\alias{get_m2_path}
\alias{get_m2_connection}
\alias{get_m2_con}
\alias{get_m2_procid}
\alias{get_m2_port}
\title{Set path to Macaulay2 (M2)}
\usage{
set_m2_path(path = NULL)

get_m2_path()

get_m2_connection()

get_m2_con()

get_m2_procid()

get_m2_port()
}
\arguments{
\item{path}{A character string, the path to M2}
}
\value{
An invisible character string, the path found.  More importantly, the
function has the side effect of setting the global m2r option "m2_path"
}
\description{
These are helper functions that deal with pathing to Macaulay2 and asking if it
is present. When the Macaulay2 package is loaded it attempts to find the
Macaulay2 executable by looking for an environment variable indicating where it
is, i.e. its path as specified in your .Renviron file.
}
\details{
For easiest use, you'll want to specify the path the Macaulay2 executable in
your ~/.Renviron file. It should look something like

\code{M2=/Applications/Macaulay2-1.10/bin}

You can set this permanently with \code{\link[=edit_r_environ]{edit_r_environ()}}. Note that absolute
paths should be specified, not relative paths, e.g. don't use ~/path/to/exe.

You can change this for the current session using \code{\link[=set_m2_path]{set_m2_path()}}, which
accepts a character string or, if missing, uses \code{\link[=file.choose]{file.choose()}} to let you
interactively; you just select an arbitrary executable.

On Windows, m2r just defaults to the cloud implementation. Local M2 instances
are not currently supported on Windows.
}
\examples{

\dontrun{ requires Macaulay2


getOption("m2r")
get_m2_path()
set_m2_path()


## each of these functions can be used statically as well
(m2_path <- get_m2_path())
set_m2_path("/path/to/m2/directory")
get_m2_path()
set_m2_path(m2_path) # undoes example


# if you'd like to use the cloud, after you library(m2r)
# and before you use m2() type
set_m2_path(NULL)

# alternatively, if you have already been using m2, do:
stop_m2()
set_m2_path(NULL)
m2("1+1")


}
}
\author{
David Kahle \email{david@kahle.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ring.R
\name{ring}
\alias{ring}
\alias{ring.}
\alias{ring_}
\alias{ring_.}
\alias{m2_coefrings}
\alias{m2_termorders}
\alias{print.m2_polynomialring}
\title{Create a new ring in Macaulay2}
\usage{
ring(..., coefring = m2_coefrings(), order = m2_termorders(), code = FALSE)

ring.(..., coefring = m2_coefrings(), order = m2_termorders(), code = FALSE)

ring_(
  vars,
  coefring = m2_coefrings(),
  order = m2_termorders(),
  code = FALSE,
  ...
)

ring_.(
  vars,
  coefring = m2_coefrings(),
  order = m2_termorders(),
  code = FALSE,
  ...
)

m2_coefrings()

m2_termorders()

\method{print}{m2_polynomialring}(x, ...)
}
\arguments{
\item{...}{...}

\item{coefring}{coefficient ring (default: \code{"CC"})}

\item{order}{a term order (default: \code{"grevlex"})}

\item{code}{return only the M2 code? (default: \code{FALSE})}

\item{vars}{vector of variable names}

\item{x}{formal argument for print method}
}
\value{
a reference to a Macaulay2 ring
}
\description{
Create a new ring in Macaulay2
}
\examples{

\dontrun{ requires Macaulay2

##### basic usage
########################################

ring("x", "y")
ring("x", "y", coefring = "QQ")


##### standard evaluation
########################################

ring_(c("x", "y"))
ring_(c("x", "y"), code = TRUE)

(myring <- ring_(c("x1","x2","x3","y"), coefring = "QQ", order = "lex"))

m2_name(myring)
m2_meta(myring, "vars")
m2_meta(myring, "coefring")
m2_meta(myring, "order")

##### other options
########################################

ring_.(c("x", "y"))
ring_.(c("x", "y"), code = TRUE)

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PHCpack.R
\name{phc}
\alias{phc}
\alias{solve_system}
\alias{solve_system.}
\alias{mixed_volume}
\title{PHCpack}
\usage{
solve_system(mpolyList)

solve_system.(mpolyList)

mixed_volume(mpolyList)
}
\arguments{
\item{mpolyList}{An mpolyList object}
}
\value{
(currently) the output of an m2() call (string?)
}
\description{
Call PHCpack to solve a zero-dimensional system
}
\details{
Note that \code{solve_system()} doesn't take in an input ring
because the solver only works over the complex numbers.
}
\examples{

\dontrun{ requires Macaulay2

# for this to work, you need to have modified your
# init-PHCpack.m2 file instead of changing your .bashrc
# file to establish the path of phc
# (**clarify**, maybe checkout algstat::polySolve)

(mpolyList <- mp(c("t^4 - x", "t^3 - y", "t^2 - z", "x+y+z")))
solve_system(mpolyList)
mixed_volume(mpolyList)

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/use_ring.R
\name{use_ring}
\alias{use_ring}
\title{Set Macaulay2 ring}
\usage{
use_ring(ring)
}
\arguments{
\item{ring}{a \code{m2_ring} (see \code{\link[=ring]{ring()}}),
\code{m2_ring_pointer} (see \code{\link[=ring.]{ring.()}}), or a character
string containing the name of a ring in Macaulay2}
}
\description{
\code{use_ring()} sets the default referent ring on the Macaulay2
side using the \code{use} function.
}
\examples{

\dontrun{ requires Macaulay2


##### basic usage
########################################

ring("x", coefring = "QQ")
factor_poly("x^4 + 1")

QQtxyz <- ring("t","x","y","z", coefring = "QQ")
gb("t^4 - x", "t^3 - y", "t^2 - z")

ring("x", "y", "z", "t", coefring = "QQ")
gb("t^4 - x", "t^3 - y", "t^2 - z")

use_ring(QQtxyz)
gb("t^4 - x", "t^3 - y", "t^2 - z")

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/snf.R
\name{snf}
\alias{snf}
\alias{snf.}
\title{Smith normal form}
\usage{
snf(mat, code = FALSE)

snf.(mat, code = FALSE)
}
\arguments{
\item{mat}{a matrix (integer entries)}

\item{code}{return only the M2 code? (default: \code{FALSE})}
}
\value{
a list of \code{m2_matrix} objects with names \code{D}, \code{P}, and
\code{Q}
}
\description{
For an integer matrix M, this computes the matrices D, P, and Q such that
\emph{D = PMQ}, which can be seen as an analogue of the singular value
decomposition. All are integer matrices, and P and Q are unimodular (have
determinants +- 1).
}
\examples{

\dontrun{ requires Macaulay2

##### basic usage
########################################

M <- matrix(c(
   2,  4,   4,
  -6,  6,  12,
  10, -4, -16
), nrow = 3, byrow = TRUE)

snf(M)

(mats <- snf(M))
P <- mats$P; D <- mats$D; Q <- mats$Q

P \%*\% M \%*\% Q                # = D
solve(P) \%*\% D \%*\% solve(Q)  # = M

det(P)
det(Q)


M <- matrix(c(
     1,    2,    3,
     1,   34,   45,
  2213, 1123, 6543,
     0,    0,    0
), nrow = 4, byrow = TRUE)
(mats <- snf(M))
P <- mats$P; D <- mats$D; Q <- mats$Q
P \%*\% M \%*\% Q                # = D



##### understanding lattices
########################################




# cols of m generate the lattice L
M <- matrix(c(2,-1,1,3), nrow = 2)
row.names(M) <- c("x", "y")
M

# plot lattice
df <- expand.grid(x = -20:20, y = -20:20)
pts <- t(apply(df, 1, function(v) M \%*\% v))
w <- c(-15, 15)
plot(pts, xlim = w, ylim = w)

# decompose m
(mats <- snf(M))
P <- mats$P; D <- mats$D; Q <- mats$Q

# PMQ = D, the columns of MQ = P^(-1) D  are a simpler basis of
# the lattice generated by (the cols of) M
(basis <- solve(P) \%*\% D)

# plot lattice generated by new basis
pts2 <- t(apply(df, 1, function(v) basis \%*\% v))
points(pts2, pch = "*", col = "red")



##### other options
########################################

snf.(M)
snf(M, code = TRUE)




}

}
\seealso{
\code{\link[=m2_matrix]{m2_matrix()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ideal.R
\name{ideal}
\alias{ideal}
\alias{ideal.}
\alias{ideal_}
\alias{ideal_.}
\alias{print.m2_ideal}
\alias{print.m2_ideal_list}
\alias{radical}
\alias{radical.}
\alias{saturate}
\alias{saturate.}
\alias{quotient}
\alias{quotient.}
\alias{primary_decomposition}
\alias{primary_decomposition.}
\alias{dimension}
\alias{+.m2_ideal}
\alias{*.m2_ideal}
\alias{==.m2_ideal}
\alias{^.m2_ideal}
\title{Create a new ideal in Macaulay2}
\usage{
ideal(..., raw_chars = FALSE, code = FALSE)

ideal.(..., raw_chars = FALSE, code = FALSE)

ideal_(x, raw_chars = FALSE, code = FALSE, ...)

ideal_.(x, raw_chars = FALSE, code = FALSE, ...)

\method{print}{m2_ideal}(x, ...)

\method{print}{m2_ideal_list}(x, ...)

radical(ideal, ring, code = FALSE, ...)

radical.(ideal, ring, code = FALSE, ...)

saturate(I, J, code = FALSE, ...)

saturate.(I, J, code = FALSE, ...)

quotient(I, J, code = FALSE, ...)

quotient.(I, J, code = FALSE, ...)

primary_decomposition(ideal, code = FALSE, ...)

primary_decomposition.(ideal, code = FALSE, ...)

dimension(ideal, code = FALSE, ...)

\method{+}{m2_ideal}(e1, e2)

\method{*}{m2_ideal}(e1, e2)

\method{==}{m2_ideal}(e1, e2)

\method{^}{m2_ideal}(e1, e2)
}
\arguments{
\item{...}{...}

\item{raw_chars}{if \code{TRUE}, the character vector will not be parsed by
\code{\link[=mp]{mp()}}, saving time (default: \code{FALSE}). the down-side is that the
strings must be formated for M2 use directly, as opposed to for \code{\link[=mp]{mp()}}.
(e.g. \code{"x*y+3"} instead of \code{"x y + 3"})}

\item{code}{return only the M2 code? (default: \code{FALSE})}

\item{x}{a listing of polynomials. several formats are accepted, see
examples.}

\item{ideal}{an ideal object of class \code{m2_ideal} or
\code{m2_ideal_pointer}}

\item{ring}{the referent ring in Macaulay2}

\item{I, J}{ideals or objects parsable into ideals}

\item{e1, e2}{ideals for arithmetic}
}
\value{
a reference to a Macaulay2 ideal
}
\description{
Create a new ideal in Macaulay2
}
\examples{

\dontrun{ requires Macaulay2


##### basic usage
########################################

ring("x", "y", coefring = "QQ")
ideal("x + y", "x^2 + y^2")



##### different versions of gb
########################################

# standard evaluation version
poly_chars <- c("x + y", "x^2 + y^2")
ideal_(poly_chars)

# reference nonstandard evaluation version
ideal.("x + y", "x^2 + y^2")

# reference standard evaluation version
ideal_.(poly_chars)



##### different inputs to gb
########################################

ideal_(   c("x + y", "x^2 + y^2") )
ideal_(mp(c("x + y", "x^2 + y^2")))
ideal_(list("x + y", "x^2 + y^2") )



##### predicate functions
########################################

I  <- ideal ("x + y", "x^2 + y^2")
I. <- ideal.("x + y", "x^2 + y^2")
is.m2_ideal(I)
is.m2_ideal(I.)
is.m2_ideal_pointer(I)
is.m2_ideal_pointer(I.)



##### ideal radical
########################################

I <- ideal("(x^2 + 1)^2 y", "y + 1")
radical(I)
radical.(I)



##### ideal dimension
########################################

I <- ideal_(c("(x^2 + 1)^2 y", "y + 1"))
dimension(I)

# dimension of a line
ring("x", "y", coefring = "QQ")
I <- ideal("y - (x+1)")
dimension(I)

# dimension of a plane
ring("x", "y", "z", coefring = "QQ")
I <- ideal("z - (x+y+1)")
dimension(I)



##### ideal quotients and saturation
########################################

ring("x", "y", "z", coefring = "QQ")
(I <- ideal("x^2", "y^4", "z + 1"))
(J <- ideal("x^6"))

quotient(I, J)
quotient.(I, J)

saturate(I)
saturate.(I)
saturate(I, J)
saturate(I, mp("x"))
saturate(I, "x")


ring("x", "y", coefring = "QQ")
saturate(ideal("x y"), "x^2")

# saturation removes parts of varieties
# solution over R is x = -1, 0, 1
ring("x", coefring = "QQ")
I <- ideal("(x-1) x (x+1)")
saturate(I, "x") # remove x = 0 from solution
ideal("(x-1) (x+1)")



##### primary decomposition
########################################

ring("x", "y", "z", coefring = "QQ")
I <- ideal("(x^2 + 1) (x^2 + 2)", "y + 1")
primary_decomposition(I)
primary_decomposition.(I)

I <- ideal("x (x + 1)", "y")
primary_decomposition(I)

# variety = z axis union x-y plane
(I <- ideal("x z", "y z"))
dimension(I) # =  max dimension of irreducible components
(Is <- primary_decomposition(I))
dimension(Is)



##### ideal arithmetic
########################################

ring("x", "y", "z", coefring = "RR")

# sums (cox et al., 184)
(I <- ideal("x^2 + y"))
(J <- ideal("z"))
I + J

# products (cox et al., 185)
(I <- ideal("x", "y"))
(J <- ideal("z"))
I * J

# equality
(I <- ideal("x", "y"))
(J <- ideal("z"))
I == J
I == I

# powers
(I <- ideal("x", "y"))
I^3

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/factor_n.R
\name{factor_n}
\alias{factor_n}
\alias{factor_n.}
\title{Factor an integer into primes}
\usage{
factor_n(n, code = FALSE, ...)

factor_n.(n, code = FALSE, ...)
}
\arguments{
\item{n}{an integer or a polynomial}

\item{code}{return only the M2 code? (default: \code{FALSE})}

\item{...}{...}
}
\value{
a data frame with integer columns \code{prime} and
\code{power} or \code{m2_pointer} referencing the factorization
in M2.
}
\description{
Factor an integer into primes
}
\examples{

\dontrun{ requires Macaulay2

##### basic usage
########################################

2^2 * 3^7 * 5^2 # = 218700
factor_n(218700)
factor_n.(218700)

(df <- factor_n(218700))
df$prime
df$power
str(df)


factor_n(218700, code = TRUE)


##### other options
########################################

(integer_pointer <- m2.("218700"))
m2_name(integer_pointer)
factor_n(integer_pointer, code = TRUE)
factor_n(integer_pointer)



factor_n(3234432540)
factor_n(323443254223453)
factor_n(rpois(1, 1e4))


##### known issues
########################################

# R doesn't handle big ints well. note in the following
# the m2 code number is different than the supplied number
factor_n(32344325422364353453, code = TRUE)

# this can be circumvented by passing a string instead
factor_n("32344325422364353453", code = TRUE)

# but if the factors are large, R can't handle the parsing well
factor_n("32344325422364353453")

# here's a workaround:
factor_pointer <- factor_n.("32344325422364353453")
m2_meta(factor_pointer, "ext_str")
extract_factors <- function(pointer) {
  require(stringr)
  str <- m2_meta(pointer, "ext_str")
  str <- str_sub(str, 19, -2)
  str <- str_extract_all(str, "\\\\{[0-9]+,[0-9]+\\\\}")[[1]]
  str <- str_sub(str, 2, -2)
  str <- str_split(str, ",")
  df <- as.data.frame(t(simplify2array(str)))
  names(df) <- c("prime", "power")
  df
}
(df <- extract_factors(factor_pointer))


# using gmp (currently broken)
# factor_n("32344325422364353453", gmp = TRUE)
m2("11 * 479 * 6138607975396537")
11 * 479 * 6138607975396537

}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enter_m2.R
\name{enter_m2}
\alias{enter_m2}
\title{Enter a Macaulay2 session}
\usage{
enter_m2(port = 27436L, timeout = 10)
}
\arguments{
\item{port}{port for Macaulay2 socket}

\item{timeout}{number of seconds before aborting}
}
\value{
\code{TRUE} invisibly
}
\description{
Enter a Macaulay2 session
}
\examples{

\dontrun{ requires Macaulay2 be installed and an interactive session

enter_m2()

# m2 code below
1 + 1
a = 1
a
R = QQ[t,x,y,z]
I = ideal(t^4  -  x, t^3  -  y, t^2  -  z)
gens gb I
exit

# back in R, the variable persists using m2()
m2("a")
m2("I")


# we can also define variables in R that persist in m2
m2("b = 5")

enter_m2()
b
exit

}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LLL.R
\name{LLL}
\alias{LLL}
\alias{LLL.}
\title{LLL algorithm}
\usage{
LLL(mat, control = list(), code = FALSE)

LLL.(mat, control = list(), code = FALSE)
}
\arguments{
\item{mat}{a matrix (integer entries)}

\item{control}{additional arguments to pass to LLL; see examples}

\item{code}{return only the M2 code? (default: \code{FALSE})}
}
\value{
an object of class \code{m2_matrix}
}
\description{
Macaulay2's implementation of the LLL algorithm. This implementation is still
under development and is currently untested.
}
\examples{

\dontrun{ requires Macaulay2

##### basic usage
########################################

# example 1
M <- matrix(c(
  1, 1, 1, 1,
  2, 0, 3, 4,
  1, 0, 0, 0,
  0, 1, 0, 0,
  0, 0, 1, 0,
  0, 0, 0, 1
), nrow = 6, byrow = TRUE)

LLL(M)




# example 2 (wikipedia)
M <- matrix(c(
  1, -1, 3,
  1,  0, 5,
  1,  2, 6
), nrow = 3, byrow = TRUE)

LLL(M)


##### control
########################################

M <- matrix(c(
  1, 1, 1, 1,
  2, 0, 3, 4,
  1, 0, 0, 0,
  0, 1, 0, 0,
  0, 0, 1, 0,
  0, 0, 0, 1
), nrow = 6, byrow = TRUE)

LLL(M, code = TRUE)
LLL(M, control = list(Strategy = "NTL"), code = TRUE)
LLL(M, control = list(Strategy = c("BKZ", "RealFP")), code = TRUE)

LLL(M)
LLL(M, control = list(Strategy = "NTL"))
LLL(M, control = list(Strategy = c("BKZ", "RealFP")))
LLL(M, control = list(Strategy = c("BKZ", "RealQP")))



# method timings with microbenchmark.  note they are roughly the same
# for this example matrix
microbenchmark::microbenchmark(
  "NTL" = LLL(M, control = list(Strategy = "NTL")),
  "BKZ_RealFP" = LLL(M, control = list(Strategy = c("BKZ", "RealFP"))),
  "BKZ_RealQP" = LLL(M, control = list(Strategy = c("BKZ", "RealQP"))),
  "BKZ_RealRR" = LLL(M, control = list(Strategy = c("BKZ", "RealRR")))
)



##### additional examples
########################################

LLL.(M)
LLL(M, code = TRUE)



}

}
\seealso{
\code{\link[=m2_matrix]{m2_matrix()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/factor_poly.R
\name{factor_poly}
\alias{factor_poly}
\alias{factor_poly.}
\title{Factor a polynomial}
\usage{
factor_poly(mpoly, code = FALSE)

factor_poly.(mpoly, code = FALSE, ...)
}
\arguments{
\item{mpoly}{a character parseable by \code{\link[=mp]{mp()}}, an
\code{mpoly} object, or a pointer to a polynomial in M2}

\item{code}{return only the M2 code? (default: \code{FALSE})}

\item{...}{...}
}
\value{
a named list with elements \code{factor} (an
\code{mpolyList} object) and \code{power}, an integer
vector
}
\description{
Factor a polynomial
}
\examples{

\dontrun{ requires Macaulay2 be installed and an interactive session

##### basic usage
########################################

ring("x", "y", coefring = "QQ")
factor_poly("x^4 - y^4")

# reference function
factor_poly.("x^4 - y^4")


##### different inputs
########################################

# factor_poly accepts mpoly objects:
# remember you must create the ring first!
(p <- mp("x^4 - y^4"))
factor_poly.(p)
factor_poly(p)
mp("(x-y) (x+y) (x^2+y^2)")



##### other examples
########################################

ring("x","y", "z", coefring = "QQ")
(p <- mp("(x^2 - y) (x^2 + y) (x + y)^2 (x - z)^2"))
factor_poly.(p)
factor_poly(p)

(p <- mp("(x-1)^3 (y-1)^3"))
factor_poly.(p)
factor_poly(p)

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/m2_parser.R
\name{m2_parser}
\alias{m2_parser}
\alias{m2_parse}
\alias{print.m2_integer}
\alias{print.m2_float}
\alias{print.m2_complex}
\alias{print.m2_string}
\alias{print.m2_boolean}
\alias{print.m2_list}
\alias{print.m2_array}
\alias{print.m2_sequence}
\alias{print.m2_symbol}
\alias{print.m2_option}
\alias{print.m2_hashtable}
\alias{print.m2_module}
\alias{m2_toggle_gmp}
\alias{get_m2_gmp}
\title{Convert a M2 object into an R object}
\usage{
m2_parse(s)

\method{print}{m2_integer}(x, ...)

\method{print}{m2_float}(x, ...)

\method{print}{m2_complex}(x, ...)

\method{print}{m2_string}(x, ...)

\method{print}{m2_boolean}(x, ...)

\method{print}{m2_list}(x, ...)

\method{print}{m2_array}(x, ...)

\method{print}{m2_sequence}(x, ...)

\method{print}{m2_symbol}(x, ...)

\method{print}{m2_option}(x, ...)

\method{print}{m2_hashtable}(x, ...)

\method{print}{m2_module}(x, ...)

m2_toggle_gmp()

get_m2_gmp()
}
\arguments{
\item{s}{a character(1), typically the result of running toExternalString on
an M2 object}

\item{x}{an object to be printed}

\item{...}{...}
}
\value{
an R object
}
\description{
Convert a M2 object into an R object
}
\examples{

\dontrun{ requires Macaulay2

m2("1+1")
m2.("1+1")
m2_parse(m2.("1+1"))

m2("QQ[x,y]")
m2.("QQ[x,y]")
m2_parse(m2.("QQ[x,y]"))

get_m2_gmp()
m2("3/2") \%>\% m2_parse()
m2_toggle_gmp() # gmp on
m2("3/2") \%>\% m2_parse()
m2("6/4") \%>\% m2_parse()
m2("3345234524352435432/223454325235432524352433245") \%>\% m2_parse()
m2_toggle_gmp() # gmp off



m2("50!") \%>\% m2_parse()
m2_toggle_gmp() # gmp on
m2("50!") \%>\% m2_parse()
m2_toggle_gmp() # gmp off

}
}
\references{
D. Kahle, C. O'Neill, and J. Sommars (2020). "A Computer Algebra
System for R: Macaulay2 and the m2r Package." Journal of Statistical
Software, 93(9):1-31.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is.R
\name{is}
\alias{is}
\alias{is.m2}
\alias{is.m2_pointer}
\alias{is.ring}
\alias{is.m2_polynomialring}
\alias{is.m2_polynomialring_pointer}
\alias{is.m2_grobner_basis}
\alias{is.m2_ideal}
\alias{is.m2_ideal_pointer}
\alias{is.m2_ideal_list}
\alias{is.m2_ideal_list_pointer}
\alias{is.m2_module}
\alias{is.m2_option}
\alias{is.m2_matrix}
\alias{is.m2_matrix_pointer}
\alias{is.m2_list}
\alias{is.m2_array}
\alias{is.m2_sequence}
\title{Macaulay2 object tests}
\usage{
is.m2(x)

is.m2_pointer(x)

is.ring(x)

is.m2_polynomialring(x)

is.m2_polynomialring_pointer(x)

is.m2_grobner_basis(x)

is.m2_ideal(x)

is.m2_ideal_pointer(x)

is.m2_ideal_list(x)

is.m2_ideal_list_pointer(x)

is.m2_module(x)

is.m2_option(x)

is.m2_matrix(x)

is.m2_matrix_pointer(x)

is.m2_list(x)

is.m2_array(x)

is.m2_sequence(x)
}
\arguments{
\item{x}{an object}
}
\value{
logical(1)
}
\description{
Predicate functions for Macaulay2 objects.
}
\examples{

\dontrun{ requires Macaulay2

R <- ring(c("x1", "x2", "x3"))
is.m2(R)
is.ring(R)
is.ring(10)
is.ring(mp("x+1"))

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gb.R
\name{gb}
\alias{gb}
\alias{gb.}
\alias{gb_}
\alias{gb_.}
\title{Compute a Grobner basis with Macaulay2}
\usage{
gb(..., control = list(), raw_chars = FALSE, code = FALSE)

gb.(..., control = list(), raw_chars = FALSE, code = FALSE)

gb_(x, control = list(), raw_chars = FALSE, code = FALSE, ...)

gb_.(x, control = list(), raw_chars = FALSE, code = FALSE, ...)
}
\arguments{
\item{...}{...}

\item{control}{a list of options, see examples}

\item{raw_chars}{if \code{TRUE}, the character vector will not be parsed by
\code{\link[=mp]{mp()}}, saving time (default: \code{FALSE}). the down-side is that the
strings must be formated for M2 use directly, as opposed to for \code{\link[=mp]{mp()}}.
(e.g. \code{"x*y+3"} instead of \code{"x y + 3"})}

\item{code}{return only the M2 code? (default: \code{FALSE})}

\item{x}{a character vector of polynomials to be parsed by \code{\link[=mp]{mp()}}, a
\code{mpolyList} object, an \code{\link[=ideal]{ideal()}} or pointer to an ideal}
}
\value{
an \code{mpolyList} object of class \code{m2_grobner_basis} or a
\code{m2_grobner_basis_pointer} pointing to the same. See \code{\link[=mpolyList]{mpolyList()}}.
}
\description{
Compute a Grobner basis with Macaulay2
}
\details{
\code{gb} uses nonstandard evaluation; \code{gb_} is the standard evaluation
equivalent.
}
\examples{

\dontrun{ requires Macaulay2


##### basic usage
########################################

# the last ring evaluated is the one used in the computation
ring("t","x","y","z", coefring = "QQ")
gb("t^4 - x", "t^3 - y", "t^2 - z")

# here's the code it's running in M2
gb("t^4 - x", "t^3 - y", "t^2 - z", code = TRUE)



##### different versions of gb
########################################

# standard evaluation version
poly_chars <- c("t^4 - x", "t^3 - y", "t^2 - z")
gb_(poly_chars)

# reference nonstandard evaluation version
gb.("t^4 - x", "t^3 - y", "t^2 - z")

# reference standard evaluation version
gb_.(poly_chars)



##### different inputs to gb
########################################

# ideals can be passed to gb
I <- ideal("t^4 - x", "t^3 - y", "t^2 - z")
gb_(I)

# note that gb() works here, too, since there is only one input
gb(I)

# ideal pointers can be passed to gb
I. <- ideal.("t^4 - x", "t^3 - y", "t^2 - z")
gb_(I.)

# setting raw_chars is a bit faster, because it doesn't use ideal()
gb("t^4 - x", "t^3 - y", "t^2 - z", raw_chars = TRUE, code = TRUE)
gb("t^4 - x", "t^3 - y", "t^2 - z", raw_chars = TRUE)



##### more advanced usage
########################################

# the control argument accepts a named list with additional
# options
gb_(
  c("t^4 - x", "t^3 - y", "t^2 - z"),
  control = list(StopWithMinimalGenerators = TRUE),
  code = TRUE
)

gb_(
  c("t^4 - x", "t^3 - y", "t^2 - z"),
  control = list(StopWithMinimalGenerators = TRUE)
)



##### potential issues
########################################

# when specifying raw_chars, be sure to add asterisks
# between variables to create monomials; that's the M2 way
ring("x", "y", "z", coefring = "QQ")
gb("x y", "x z", "x", raw_chars = TRUE, code = TRUE) # errors without code = TRUE
gb("x*y", "x*z", "x", raw_chars = TRUE, code = TRUE) # correct way
gb("x*y", "x*z", "x", raw_chars = TRUE)









}
}
\seealso{
\code{\link[=mp]{mp()}}, \code{\link[=use_ring]{use_ring()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/m2r-package.R
\docType{package}
\name{m2r}
\alias{m2r}
\alias{package-m2r}
\title{Macaulay2 in R}
\description{
m2r provides a persistent interface to Macaulay2
(\url{http://www.math.uiuc.edu/Macaulay2/}) and front-end tools facilitating
its use in the R ecosystem. For details, see vignette("m2r").
}
\references{
D. Kahle, C. O'Neill, and J. Sommars (2020). "A Computer Algebra
System for R: Macaulay2 and the m2r Package." Journal of Statistical
Software, 93(9):1-31.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/m2.R
\name{m2_call}
\alias{m2_call}
\alias{m2r_version_number}
\alias{m2r_cloud_url}
\alias{has_m2_connection}
\alias{start_m2}
\alias{stop_m2}
\alias{reset_m2}
\alias{m2}
\alias{m2.}
\alias{print.m2_pointer}
\title{Call and reset a Macaulay2 process}
\usage{
m2r_version_number()

m2r_cloud_url()

has_m2_connection()

start_m2(
  port = 27436L,
  timeout = 10,
  attempts = 10,
  cloud = FALSE,
  hostname = m2r_cloud_url()
)

stop_m2()

reset_m2(
  port = 27436L,
  timeout = 10,
  attempts = 10,
  hostname = "ec2-52-10-66-241.us-west-2.compute.amazonaws.com"
)

m2(code, timeout = -1)

m2.(code, timeout = -1)

\method{print}{m2_pointer}(x, ...)
}
\arguments{
\item{port}{port for Macaulay2 socket}

\item{timeout}{number of seconds before aborting}

\item{attempts}{numer of times to try to make connection}

\item{cloud}{use a cloud?}

\item{hostname}{the remote host to connect to; defaults to the Amazon EC2
instance}

\item{code}{Macaulay2 code}

\item{x}{formal argument for print method}

\item{...}{...}
}
\value{
m2 return value
}
\description{
Call and reset a Macaulay2 process
}
\examples{

\dontrun{ requires Macaulay2

m2("1 + 1")
m2.("1 + 1")

m2("factor 32004")

# run a chunk of m2 code, only pulling the end value back into R
m2("
  R = QQ[a..d]
  I = ideal(a^3-b^2*c, b*c^2-c*d^2, c^3)
  G = gens gb I
")

# illustrate the persistent connection
m2("a = 1 + 1")
m2("a")
reset_m2()
m2("a")


# forcing a cloud start
if(has_m2_connection()) stop_m2()
start_m2(cloud = TRUE)
m2("1 + 1")
stop_m2()



m2.("peek(QQ[x,y,z])")
m2("peek(QQ[x,y,z])")

# m2 returns in its ext_str position the result of running
# toExternalString on the return value of the chunk of code
# you run. in principle, toExternalString provides the code
# needed to recreate the m2 object of interest. however,
# does not work for all objects representable in the m2 language.
# in particular, mutable objects are not supported.
# this is what happens when you look at those:
m2.("new MutableList from {1,2,3}")
m2("new MutableList from {1,2,3}")

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/str_m2.R
\name{str_m2}
\alias{str_m2}
\title{Give the structure of a Macaulay2 ring}
\usage{
str_m2(object, ...)
}
\arguments{
\item{object}{An \code{m2} object}

\item{...}{...}
}
\value{
Invisible the object passed in.
}
\description{
Give the structure of a Macaulay2 ring
}
\examples{

\dontrun{ requires Macaulay2

a <- m2("1")

R <- ring(c("x1", "x2", "x3"))
str_m2(R)
str_m2.default(R)

}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matrix.R
\name{m2_matrix}
\alias{m2_matrix}
\alias{m2_matrix.}
\alias{m2_numrows}
\alias{m2_numcols}
\alias{m2_length}
\alias{print.m2_matrix}
\alias{print.m2_image}
\alias{m2_kernel}
\alias{m2_kernel.}
\title{Create a new matrix in Macaulay2}
\usage{
m2_matrix(mat, ring, name, code = FALSE)

m2_matrix.(mat, ring, name, code = FALSE)

m2_numrows(x, code = FALSE, ...)

m2_numcols(x, code = FALSE, ...)

m2_length(x, code = FALSE, ...)

\method{print}{m2_matrix}(x, ...)

\method{print}{m2_image}(x, ...)

m2_kernel(mat, name, code = FALSE)

m2_kernel.(mat, name, code = FALSE)
}
\arguments{
\item{mat}{a matrix}

\item{ring}{a ring containing the matrix entries}

\item{name}{the \code{m2_name} of the object, which is it's name
on the M2 side}

\item{code}{return only the M2 code? (default: \code{FALSE})}

\item{x}{formal argument for print method}

\item{...}{...}
}
\value{
an object of class \code{m2_matrix}
}
\description{
Create a new matrix in Macaulay2
}
\examples{

\dontrun{ requires Macaulay2

##### basic usage
########################################

(mat <- m2_matrix(matrix(c(1,2,3,4,5,6), nrow = 3, ncol = 2)))
m2_matrix(matrix(c(1,2,3,4,5,6), nrow = 3, ncol = 2))

m2_name(mat)
m2(m2_name(mat))
m2(sprintf("class(\%s)", m2_name(mat)))
(mat <- m2_matrix.(matrix(c(1,2,3,4,5,6), nrow = 3, ncol = 2)))

##### known issues
########################################

ring("x", "y", "z", coefring = "QQ")
(mat <- matrix(mp(c("x","y","x+y","y-2","x-3","y-z")), nrow = 2, ncol = 3))
m2_matrix(mat, code = TRUE)
m2_matrix(mat)
# the above is an mpoly problem, not a m2r problem
# mpoly does not have a data structure for matrices (as of 12/2016)

mat_chars <- sapply(m2_matrix(mat), print, silent = TRUE)
dim(mat_chars) <- c(2, 3)
mat_chars


m2_numrows(mat)
m2_numcols(mat)
m2_parse(mat)

(mat <- m2_matrix(matrix(c(1,2),nrow=1)))
m2_kernel(mat)

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/m2_utility.R
\name{m2_utility}
\alias{m2_utility}
\alias{m2_name}
\alias{m2_name<-}
\alias{m2_meta}
\alias{m2_meta<-}
\alias{m2_structure}
\alias{m2_exists}
\alias{m2_ls}
\alias{m2_rm}
\alias{m2_getwd}
\title{Utility tools for M2}
\usage{
m2_name(x)

m2_name(x) <- value

m2_meta(x, m2_attr)

m2_meta(x, m2_attr) <- value

m2_structure(x = NA, m2_name, m2_class, m2_meta, base_class)

m2_exists(name)

m2_ls(all.names = FALSE)

m2_rm(name)

m2_getwd()
}
\arguments{
\item{x}{an object of class \code{m2}}

\item{value}{the value to assign}

\item{m2_attr}{the name of an M2 attribute}

\item{m2_name}{\code{m2_name}  M2 attribute}

\item{m2_class}{\code{m2_class} M2 attribute}

\item{m2_meta}{\code{m2_meta}  M2 attribute}

\item{base_class}{a base class; an R class to use for dispatching
if there is no relevant method for the other classes (e.g.
\code{m2})}

\item{name}{a string; the name of a M2 object}

\item{all.names}{if \code{TRUE}, all registered Macaulay2
variables, including ones internally used by m2r, will be
returned}
}
\description{
Utility tools for M2
}
\examples{

\dontrun{ requires Macaulay2

m2("a = 5")
m2_ls()
m2_exists("a")
m2("b = 1")
m2_exists(c("a","b","c"))

m2_getwd()

x <- 1
class(x) <- "m2"
attr(x, "m2_meta") <- list(a = 1, b = 2)
m2_meta(x)
m2_meta(x, "b")
m2_meta(x, "b") <- 5
m2_meta(x, "b")

# R <- ring(c("x1", "x2", "x3"))
# m2_name(R)
# m2(sprintf("class \%s", m2_name(R)))
# m2_ls()
# m2_rm(m2_name(R))
# m2_ls()
# m2(paste("class", m2_name(R)))

m2_ls()
m2_ls(all.names = TRUE)


}
}
